Package: biovizBase
Version: 1.60.0
Title: Basic graphic utilities for visualization of genomic data.
Description: The biovizBase package is designed to provide a set of
        utilities, color schemes and conventions for genomic data. It
        serves as the base for various high-level packages for
        biological data visualization. This saves development effort
        and encourages consistency.
Authors@R: c(person("Tengfei", "Yin", role=c("aut"),
                 email="yintengfei@gmail.com"),
          person("Michael", "Lawrence", role=c("aut", "ths", "cre"),
                 email="lawremi@gmail.com"),
          person("Dianne", "Cook", role=c("aut", "ths")),
          person("Johannes", "Rainer", role="ctb"))
Depends: R (>= 3.5.0), methods
Imports: grDevices, stats, scales, Hmisc, RColorBrewer, dichromat,
        BiocGenerics, S4Vectors (>= 0.23.19), IRanges (>= 1.99.28),
        Seqinfo, GenomeInfoDb (>= 1.45.5), GenomicRanges (>= 1.61.1),
        SummarizedExperiment (>= 1.39.1), Biostrings (>= 2.77.2),
        Rsamtools (>= 2.25.1), GenomicAlignments (>= 1.45.1),
        GenomicFeatures (>= 1.61.4), AnnotationDbi, VariantAnnotation
        (>= 1.55.1), ensembldb (>= 2.33.1), AnnotationFilter (>=
        0.99.8), rlang
Suggests: BSgenome.Hsapiens.UCSC.hg19,
        TxDb.Hsapiens.UCSC.hg19.knownGene, BSgenome, rtracklayer,
        EnsDb.Hsapiens.v75, RUnit
License: Artistic-2.0
LazyLoad: Yes
LazyData: Yes
Collate: utils.R color.R AllGenerics.R crunch-method.R mold-method.R
        addStepping-method.R getFragLength-method.R
        shrinkageFun-method.R maxGap-method.R spliceSummary-method.R
        ideogram.R pileup.R coverage.R labs.R original.R transform.R
        facets-method.R aes.R scale.R zzz.R biovizBase-package.R
biocViews: Infrastructure, Visualization, Preprocessing
git_url: https://git.bioconductor.org/packages/biovizBase
git_branch: RELEASE_3_23
git_last_commit: 142d671
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:56:20 UTC; biocbuild
Author: Tengfei Yin [aut],
  Michael Lawrence [aut, ths, cre],
  Dianne Cook [aut, ths],
  Johannes Rainer [ctb]
Maintainer: Michael Lawrence <lawremi@gmail.com>

useDynLib(biovizBase, .registration=TRUE)

## ======================================================================
## Import
## ======================================================================

import(methods)
import(BiocGenerics)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### base packages
###

importFrom(grDevices, colorRampPalette)
importFrom(graphics, par, plot, rect, text)
importFrom(stats, approx, as.formula, asOneSidedFormula, setNames)
importFrom(utils, menu)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Non-bioconductor packages
###

importFrom(RColorBrewer, brewer.pal)
importFrom(scales, dichromat_pal, dscale, expand_range, rescale)
importFrom(Hmisc, bezier)
importFrom(dichromat, colorschemes)
importFrom(rlang, quo_text)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Bioconductor packages
###

import(S4Vectors)
import(IRanges)
import(Seqinfo)
import(GenomeInfoDb)
import(GenomicRanges)
import(SummarizedExperiment)

## Biostrings
importFrom(Biostrings, AA_ALPHABET, DNA_ALPHABET, RNA_ALPHABET,
           IUPAC_CODE_MAP, AMINO_ACID_CODE)
importMethodsFrom(Biostrings, getSeq, letterFrequency,
                  letterFrequencyInSlidingView)

## Rsamtools
importClassesFrom(Rsamtools, BamFile)
importFrom(Rsamtools, index, scanBamFlag, PileupFiles, ApplyPileupsParam)
importMethodsFrom(Rsamtools, ScanBamParam, applyPileups, scanBam)

## GenomicAlignments
importFrom(GenomicAlignments, cigar, readGAlignments)
importClassesFrom(GenomicAlignments, GAlignments)

## GenomicFeatures
importClassesFrom(GenomicFeatures, TxDb)
importMethodsFrom(GenomicFeatures, transcripts, cdsBy, cdsByOverlaps, exonsBy,
                  exonsByOverlaps, isActiveSeq, "isActiveSeq<-", transcripts,
                  transcriptsByOverlaps)

## AnnotationDbi
importFrom(AnnotationDbi, unlist2, select, mappedRkeys, revmap, toTable)

## VariantAnnotation
importClassesFrom(VariantAnnotation, VRanges)

## AnnotationFilter
importClassesFrom(AnnotationFilter, AnnotationFilter, GRangesFilter,
                  TxIdFilter)
importFrom(AnnotationFilter, AnnotationFilter, GRangesFilter, TxIdFilter,
           AnnotationFilterList, GenenameFilter)

## ensembldb
importMethodsFrom(ensembldb, listColumns, exons, transcripts)

## ======================================================================
##    export
## ======================================================================
## color
export(getBioColor,
       plotColorLegend,
       genBrewerBlindPalInfo,
       genDichromatPalInfo,
       genBlindPalInfo,
       colorBlindSafePal,
       blind.pal.info,
       brewer.pal.blind.info,
       dichromat.pal.blind.info,
       estimateCoverage)



## transform
export(transformToCircle,
       transformToRectInCircle,
       transformToBarInCircle,
       transformToSegInCircle,
       transformToLinkInCircle,
       transformDfToGr,
       transformGRangesForEvenSpace)
exportMethods(transformToGenome,transformToDf)

## utils
exportMethods(addStepping,
              shrinkageFun,
              maxGap,
              splitByFacets,
              getGaps,
              getXScale,
              getYLab,
              getXLab,
              crunch,
	      mold)

export(isIdeogram, isSimpleIdeogram, getIdeogram, containLetters,
       pileupAsGRanges, pileupGRangesAsVariantTable, GCcontent,
       showColor, isJunctionRead, isMatchedWithModel,
       flatGrl, getIdeoGR, getScale, getFormalNames, subsetArgsByFormals,
       parseArgsForAes, parseArgsForNonAes, strip_formula_dots,
       is_coord_truncate_gaps, is_coord_genome)


## exportPattern("^[^\\.]")

## going to be moved to biovizBase
setGeneric("getYLab", function(obj,...) standardGeneric("getYLab"))
setMethod("getYLab", "TxDb", function(obj){
  md <- metadata(obj)
  ds <- md[md[,1] == "Data source",2]
  lb <- character()
  if(ds == "UCSC"){
    lb <- md[md[,1] == "UCSC Table",2]
  }
  if(ds == "BioMart"){
    lb <- md[md[,1] == "BioMart Database",2]
  }
  lb
})

## going to be moved to biovizBase
setGeneric("getXLab", function(obj,...) standardGeneric("getXLab"))
setMethod("getXLab", "GRanges", function(obj){
  chrs <- unique(as.character(seqnames(obj)))
  gms <- genome(obj)
  gm <- gms[chrs]
  if(length(chrs) > 1){
    xlab <- "Position"
  }else{
    xlab <- paste(ifelse(is.na(gm), chrs, paste0(gm, "::", chrs)))
    xlab <- paste("Position on", xlab)
  }
  xlab
})

setMethod("getXLab", "GRangesList", function(obj){
  chrs <- unique(as.character(seqnames(unlist(obj))))
  gms <- genome(obj)
  gm <- gms[chrs]
  if(length(chrs) > 1){
    xlab <- "Position"
  }else{
    xlab <- paste(ifelse(is.na(gm), chrs, paste0(gm, "::", chrs)))
    xlab <- paste("Position on", xlab)
  }
  xlab  
})

setMethod("getXLab", "GAlignments", function(obj){
  chrs <- unique(as.character(seqnames(obj)))
  gms <- genome(obj)
  gm <- gms[chrs]
  if(length(chrs) > 1){
    xlab <- "Position"
  }else{
    xlab <- paste(ifelse(is.na(gm), chrs, paste0(gm, "::", chrs)))
    xlab <- paste("Position on", xlab)
  }
  xlab
})

.onLoad <- function(libname, pkgname){
  options(biovizBase = list(
            DNABasesNColor = getBioColor("DNA_BASES_N", source = "default"),
            DNABasesColor = getBioColor("DNA_BASES", source = "default"),
            DNAAlphabetColor = getBioColor("DNA_ALPHABET", source = "default"),
            RNABasesNColor = getBioColor("RNA_BASES_N", source = "default"),
            RNABasesColor = getBioColor("RNA_BASES", source = "default"),
            RNAAlphabetColor = getBioColor("RNA_ALPHABET", source = "default"),
            IUPACCodeMapColor = getBioColor("IUPAC_CODE_MAP", source = "default"),
            AminoAcidCodeColor = getBioColor("AMINO_ACID_CODE", source = "default"),                  AAAlphabetColor = getBioColor("AA_ALPHABET", source = "default"),
            strandColor = getBioColor("STRAND", source = "default"),
            cytobandColor = getBioColor("CYTOBAND", source = "default")
            ))
}

newDataAfterFacetByGr <- function(gr, which, id.name){
  gr <- sort(gr)
  which <- sort(which)
  ## suppose which is a GRanges
  res <- lapply(seq_len(length(which)),function(i){
    res <- subsetByOverlaps(gr, which[i])
    values(res)$.bioviz.facetid <- i
    res
  })
  do.call(c, res)
}

strip_formula_dots <- function(formula){
  allvars <- all.vars(as.formula(formula))
  idx <- ggplot2_is_calculated_aes(allvars)
  if(sum(idx))
    formula[[which(idx)+1]] <- as.name(unlist(ggplot2_strip_dots(allvars[idx])))
  formula
}

setGeneric("splitByFacets", function(object, facets, ...) standardGeneric("splitByFacets"))

setMethod("splitByFacets", c("GRanges", "formula"), function(object, facets){
  .checkFacetsRestrict(facets, object)
  facets <- strip_formula_dots(facets)
  allvars <- all.vars(as.formula(facets))
  if(length(allvars) > 1){
  if(allvars[1] == "."){
    res <- split(object, seqnames(object))
  }else{
    if(allvars[1] != "strand")
      fts <- interaction(as.character(values(object)[,allvars[1]]),as.character(seqnames(object)))
    else
      fts <- interaction(as.character(strand(object)),as.character(seqnames(object)))
    res <- split(object, fts)    
  }}
  if(length(allvars) == 1)
    res <- split(object, seqnames(object))
  res <- res[elementNROWS(res)>0]
  res
})

setMethod("splitByFacets", c("GRanges", "GRanges"), function(object, facets){
  if(length(facets)){
      object <- newDataAfterFacetByGr(object, facets)
      grl <- split(object, values(object)$.bioviz.facetid)
    }else{
      stop("facets is an empty GenomicRanges object")
    }
  grl <- grl[elementNROWS(grl)>0]
  grl
})


setMethod("splitByFacets", c("GRanges", "missing"), function(object, facets){
  res <- split(object, seqnames(object))
  res <- res[elementNROWS(res)>0]
  res
})

setMethod("splitByFacets", c("GRanges", "NULL"), function(object, facets){
  res <- split(object, seqnames(object))
  res <- res[elementNROWS(res)>0]  
  res
})


isFacetByOnlySeq <- function(facets){
  allvars <- all.vars(as.formula(facets))
  allvars <- allvars[!allvars %in% c(".", "seqnames")]
  if(!length(allvars))
    TRUE
  else
    FALSE
}





## that's for linear or default!
.checkFacetsRestrict <- function(facets, object){
  facets <- strip_formula_dots(facets)
  allvars <- all.vars(as.formula(facets))
  if(length(allvars) == 1){
    if(allvars[1] != "seqnames")
      stop("Column of facets formula can only be seqnames, such as . ~ seqnames, in default restrict mode, you can only change row varaibles")
  }
  if(length(allvars) > 1){
    if(allvars[2] != "seqnames" & allvars[2] != "."){
      stop("Column of facets formula can only be seqnames or '.', such as . ~ seqnames, in default restrict mode, you can only change row varaibles")
    }
    if(allvars[1] != "."){
      if(!allvars[1] %in% c(colnames(values(object)), "strand"))
        stop(allvars[1]," doesn't exists in data columns")
    }
  }
}

setMethod("getFragLength", "character", function(obj, model, which,
                                                 type = "remove.gaps",
                                                 maxGap = 0L){
  require(Rsamtools)
  if(missing(which))
    which <- range(reduce(model))
  if(tools::file_ext(obj) == "bam")
    res <- scanBam(obj, param = ScanBamParam(which = which, 
                          flag = scanBamFlag(isProperPair = TRUE,
                              isFirstMateRead = TRUE)))

  if(!length(res[[1]]$pos))
    stop("no paired end reads found in the the data")
  res <- res[[1]]
  m1 <- GRanges(res$rname, IRanges(res$pos, width = res$qwidth),
                mapid = 1:length(res$pos), isize = res$isize)
  ## suppose they have the same width
  m2 <- GRanges(res$rname, IRanges(res$mpos, width = res$qwidth),
                mapid = 1:length(res$pos), isize = res$isize)
  m <- c(m1, m2)
  res <- getFragLength(m, model, type = type, maxGap = 0L)
  res
})

setMethod("getFragLength", "GenomicRanges", function(obj, model, maxGap = 0L,
                                                     type = "remove.gaps"){
  model <- reduce(model)
  if(type == "remove.gaps"){
    grl <- split(obj, values(obj)$mapid)
    gr.res <- unlist(range(grl))
    names(gr.res) <- names(grl)
    mapid <- names(gr.res)
    idx <- match(mapid, values(obj)$mapid)
    values(gr.res)$isize <- values(obj)$isize[idx]
    g.gap <- gaps(c(ranges(obj), ranges(model)))
    cut.fun <- shrinkageFun(g.gap, maxGap)
    obj.cut <- cut.fun(obj)
    grl <- split(obj.cut, values(obj.cut)$mapid)
    grl.r <- range(grl)
    .fragLength <- unlist(width(grl.r))
    values(gr.res)$.fragLength <- .fragLength[names(gr.res)]
    gr.res
  }
  list(gr = gr.res,
       cut.fun = cut.fun,
       gr.cut = unlist(grl.r),
       ## model.cut = cut.fun(model),
       ## model = model,
       reads = obj,
       ## gaps = g.gap,
       fragLength = .fragLength)
})

setMethod("spliceSummary", c("GenomicRanges", "GRangesList"), function(obj,
                                                             model,
                                                             weighted = TRUE){
  lst <- lapply(model, function(md){
    logi <- isMatchedWithModel(md, obj)
    if(!length(logi))
      logi <- rep(FALSE, length(obj))
    logi
  })
  res <- do.call(rbind, lst)
  if(weighted){
    res.s <- apply(res, 2, sum)
    idx.all <- res.s > 1
    res.n <- as.numeric(res)
    res.m <- matrix(res.n, nrow = nrow(res))
    res.m[,idx.all] <- res.m[,idx.all]/res.s[idx.all]
    freq <- apply(res.m, 1, sum)
  }else{
    freq <- apply(res, 1, sum)
  }
  names(freq) <- names(model)
  freq
})

setMethod("spliceSummary", c("character", "GRangesList"), function(obj,
                                                                   model,
                                                                   weighted = TRUE){

  if(tools::file_ext(obj) == "bam"){
    ## bf <- BamFile(obj)
    require(Rsamtools)    
  }else{
    stop("Only support file with .bam extention name now") 
  }
  
  ## if(missing(which))
  which <- range(reduce(unlist(model)))
  
  ga <- readGAlignments(obj, param = ScanBamParam(which = which),
                             use.names = TRUE)
  ## reduce model
  ## assumption here is that: contains unoverlaped exons.
  ## must contain exon id            
  exons <- reduce(unlist(model), ignore.strand = TRUE);
  ## first leave only junction read
  idx <- sapply(cigar(ga), isJunctionRead)
  ga.junction <- ga[idx]
  gr.junction <- as(ga.junction, "GRanges")
  spliceSummary(gr.junction, model = model, weighted = weighted)
})


isJunctionRead <- function(cigar){
  grepl("N", cigar)
}

## model is a GRanges containing exons, sgr is a single gr object
isMatchedWithModel <- function(model, gr){
  ## require on the same chromosome now
  if(length(unique(as.character(seqnames(model)))) > 1)
    stop("only support single chromosome summary now")
  if(unique(as.character(seqnames(model))) !=
     unique(as.character(seqnames(gr))))
    stop("Cannot be mapped due to different space")
  model <- sort(model)
  ## do we consider strand information?
  countOverlaps(ranges(gr), ranges(model)) == 2
}

setMethod("spliceSummary", c("GenomicRanges", "GenomicRanges"), function(obj,
                                                                         model,
                                                                       model_id = NULL){
  ov <- findOverlaps(ranges(obj), ranges(model))
  mm <- as.matrix(ov)
  res <- split(mm[,2], mm[,1])
  idx <- unlist(lapply(split(mm[,2], mm[,1]),length)) > 1
  res <- lapply(res[idx], function(x){
    if(!is.null(model_id))
      x <- values(model)[,model_id][x]
    paste(head(x, 1), tail(x, 1), sep = "-")
  })
  nms <- names(table(unlist(res)))
  res <- as.numeric(table(unlist(res)))
  names(res) <- nms
  res
})


setMethod("spliceSummary", c("character", "GenomicRanges"), function(obj,
                                                                   model,
                                                                   model_id = NULL){

  if(tools::file_ext(obj) == "bam"){
    require(Rsamtools)    
  }else{
    stop("Only support file with .bam extention name now") 
  }
  ## if(missing(which))
  which <- range(model)
  
  ga <- readGAlignments(obj, param = ScanBamParam(which = which),
                             use.names = TRUE)
  ## first leave only junction read
  grl <- grglist(ga)
  idx <- elementNROWS(grl) > 1
  ga.junction <- ga[idx]
  gr.junction <- as(ga.junction, "GRanges")
  spliceSummary(gr.junction, model = model, model_id = model_id)
})

## functions downbelow from workshop by Michael Lawrence
## will be implmented by into his package, so I will replace this function later
## splicefun <- function(files, txdb, which, id,  xlim, txdb.chr.pre = character(), weight = 1){
##   elementGaps <- function(x) {
##     x_flat <- unlist(x, use.names = FALSE)
##     egaps <- gaps(ranges(x))
##     first_segment <- start(PartitioningByWidth(x))
##     sn <- seqnames(x_flat)[first_segment][togroup(egaps)]
##     strand <- strand(x_flat)[first_segment][togroup(egaps)]
##     relist(GRanges(sn, unlist(egaps, use.names = FALSE), 
##                    strand, seqlengths = seqlengths(x)), 
##            egaps)
##   }

##   pairReadRanges <- function(reads) {
##     pairs <- split(unlist(reads, use.names=FALSE),
##                    factor(names(reads)[togroup(reads)], 
##                           unique(names(reads))))
##     metadata(pairs) <- metadata(reads)
##     xs <- values(reads)$XS
##     has_xs <- !is.na(xs)
##     pair_xs <- setNames(rep.int(NA, length(pairs)), 
##                         names(pairs))
##     pair_xs[names(reads)[has_xs]] <- xs[has_xs]
##     values(pairs)$XS <- unname(pair_xs)
##     pairs
##   }  

##   strandFromXS <- function(pairs) {
##     xs <- values(pairs)$XS
##     strand <- ifelse(!is.na(xs) & xs != "?", xs, "*")
##     strand(pairs) <- relist(Rle(strand, elementNROWS(pairs)), 
##                             pairs)
##     pairs
##   }

##   gr2key <- function(x) {
##     paste(seqnames(x), start(x), end(x), strand(x), 
##           sep = ":")
##   }

##   key2gr <- function(x, ...) {
##     key_mat <- matrix(unlist(strsplit(x, ":", fixed=TRUE)), 
##                       nrow = 4)
##     GRanges(key_mat[1,],
##             IRanges(as.integer(key_mat[2,]), 
##                     as.integer(key_mat[3,])),
##             key_mat[4,], ...)
##   }
##   findIsoformOverlaps <- function(pairs) {
##     splices <- values(pairs)$splices
##     hits <- findOverlaps(pairs, tx)
##     hit_pairs <- ranges(pairs)[queryHits(hits)]
##     hit_splices <- ranges(splices)[queryHits(hits)]
##     hit_tx <- ranges(tx)[subjectHits(hits)]
##     read_within <- 
##       elementNROWS(setdiff(hit_pairs, hit_tx)) == 0L
##     tx_within <- 
##       elementNROWS(intersect(hit_tx, hit_splices)) == 0L
##     compatible <- read_within & tx_within
##     compat_hits <- hits[compatible]
##     reads_unique <- tabulate(queryHits(compat_hits), 
##                              queryLength(compat_hits)) == 1L
##     unique <- logical(length(hits))
##     unique[compatible] <- reads_unique[queryHits(compat_hits)]
##     strand_specific <- 
##       all(strand(pairs) != "*")[queryHits(hits)]
##     values(hits) <- DataFrame(strand_specific,
##                               compatible,
##                               unique)
##     list(hits = hits,
##          compatible = compatible,
##          strand_specific = strand_specific)
##   }
##   countIsoformHits <- function(hits, compatible, strand_specific) {
##     countByTx <- function(x) {
##       tabulate(subjectHits(hits)[x], subjectLength(hits))
##     }
##     compatible_strand <- 
##       countByTx(with(values(hits), 
##                      compatible & strand_specific))
##     counts <- DataFrame(compatible_strand,
##                         lapply(values(hits)[-1], countByTx))
##     counts
##   }

##   summarizeSplices <- function(reads) {
##     splices <- values(reads)$splices
##     splices_flat <- unlist(splices, use.names = FALSE)
##     if(length(splices_flat)){    
##     splice_table <- table(gr2key(splices_flat))
##     splice_summary <- 
##       key2gr(names(splice_table), 
##              score = as.integer(splice_table),
##              novel = !names(splice_table) %in% tx_keys,
##              seqlengths = seqlengths(splices))
##   }else{
##     splice_summary <- GRanges()
##   }
##     splice_summary
##   }

##   getUniqueReads <- function(reads, hits) {
##     sel <- values(hits)$unique & 
##     subjectHits(hits) %in% c(1, 4)
##     reads[unique(queryHits(hits)[sel])]
##   }

##   message("Parsing gene structure from txdb...")
##   if(!missing(id)){
##   aldoa_gr <- exons(txdb, columns = c("tx_id", "gene_id"),
##                           filter = list(gene_id = id))
##   aldoa_gr <- keepSeqlevels(aldoa_gr, unique(as.character(seqnames(aldoa_gr))))
##   ## FIXME later
##   nms <- as.character(names(seqlengths(aldoa_gr)))
##   nms.new <- paste(txdb.chr.pre, nms, sep = "")
##   names(nms.new) <- nms
##   aldoa_gr <- renameSeqlevels(aldoa_gr, nms.new)
##   aldoa_range <- range(aldoa_gr)
##   wh <- aldoa_range  
##   aldoa_vals <- values(aldoa_gr)
##   tx <- multisplit(aldoa_gr, aldoa_vals$tx_id)
##   tx_to_val <- match(names(tx), unlist(aldoa_vals$tx_id))
##   values(tx)$gene_id <- 
##     rep(unlist(aldoa_vals$gene_id), 
##         elementNROWS(aldoa_vals$tx_id))[tx_to_val]
##   values(tx)$tx_id <- names(tx)
## }else if(!missing(which) & is(which, "GRanges")){
##     isActiveSeq(txdb)[seqlevels(txdb)] <- FALSE
##     seqnms <- as.character(unique(seqnames(which)))
##     seqnms <- gsub(txdb.chr.pre, "", seqnms)
##     isActiveSeq(txdb)[seqnms] <- TRUE
##     ## aldoa_gr <- exons(txdb, columns = c("tx_id", "gene_id"))
##     ## aldoa_gr <- keepSeqlevels(aldoa_gr, unique(as.character(seqnames(aldoa_gr))))
##     ## nms <- as.character(names(seqlengths(aldoa_gr)))
##     ## nms.new <- paste(txdb.chr.pre, nms, sep = "")
##     ## names(nms.new) <- nms
##     ## aldoa_gr <- renameSeqlevels(aldoa_gr, nms.new)
##     ## aldoa_gr <- subsetByOverlaps(aldoa_gr, which)
##     ## aldoa_range <- range(aldoa_gr)
##     aldoa_range <- which
##     exons_grl <- exonsBy(txdb)
##     gr.l <- stack(exons_grl, ".sample")
##     gr.l <- keepSeqlevels(gr.l, unique(as.character(seqnames(gr.l))))
##     nms <- as.character(names(seqlengths(gr.l)))
##     nms.new <- paste(txdb.chr.pre, nms, sep = "")
##     names(nms.new) <- nms
##     gr.l <- renameSeqlevels(gr.l, nms.new)
##       exons_grl <- split(gr.l, values(gr.l)$.sample)
##      ans <- subsetByOverlaps(exons_grl, which)
##     values(ans)$tx_id <- names(ans)
##     tx_gr <- transcripts(txdb, columns = c("tx_id", "gene_id"))
##     values(ans)$gene_id <- 
##       drop(values(tx_gr)$gene_id)[match(names(ans), 
##                                         values(tx_gr)$tx_id)]
##     tx <- ans
##     ## wh <- aldoa_range
##     ## aldoa_vals <- values(aldoa_gr)
##     ## tx <- multisplit(aldoa_gr, aldoa_vals$tx_id)
##     ## tx_to_val <- match(names(tx), unlist(aldoa_vals$tx_id))
##     ## values(tx)$gene_id <- 
##     ##   rep(unlist(aldoa_vals$gene_id), 
##     ##       elementNROWS(aldoa_vals$tx_id))[tx_to_val]
##     ## values(tx)$tx_id <- names(tx)
## }

##   message("Parsing bam files")
##   bamFiles <- Rsamtools::BamFileList(files)

##   message("Analysing...")
##   introns <- elementGaps(tx)
##   introns_flat <- unlist(introns, use.names = FALSE)
##   tx_keys <- gr2key(introns_flat)

##   readReadRanges <- function(bam) {
##     param <- ScanBamParam(tag = "XS", which = aldoa_range)
##     ga <- readGAlignments(path(bam), 
##                           use.names = TRUE, 
##                           param = param)
##     reads <- grglist(ga)
##     metadata(reads)$bamfile <- bam
##     splices <- elementGaps(reads)
##     values(splices)$XS <- values(reads)$XS
##     pairs <- pairReadRanges(reads)
##     pairs <- strandFromXS(pairs)
##     splices <- pairReadRanges(splices)
##     splices <- strandFromXS(splices)
##     values(pairs)$splices <- splices
##     pairs
##   }


##   N <- length(bamFiles)
##   nms <- names(bamFiles)
##   lst_hits <- lst_counts <- lst_splices <- lst_rr <- list()
##   message("parsing hits/counts/splcies")
##   for(i in 1:N){
##     bf <- bamFiles[[i]]
##     rr <- readReadRanges(bf)
##     lst_rr <- c(lst_rr, rr)
##     hits.lst <- findIsoformOverlaps(rr)
##     lst_hits <- c(lst_hits, hits.lst$hits)
##     lst_counts <- c(lst_counts, countIsoformHits(hits.lst$hits, hits.lst$compatible, hits.lst$strand_specific))
##     lst_splices <- c(lst_splices, summarizeSplices(rr))
##   }
##   names(lst_hits) <- nms
##   names(lst_counts) <- nms
##   names(lst_splices) <- nms
##   names(lst_rr) <- nms
##   ## normal <- readReadRanges(bamFiles)
##   ## tumor <- readReadRanges(bamFiles$tumor)
  
##   ## normal_hits <- findIsoformOverlaps(normal)
##   ## normal_counts <- countIsoformHits(normal_hits)
##   ## normal_splices <- summarizeSplices(normal)

##   ## tumor_hits <- findIsoformOverlaps(tumor)
##   ## tumor_counts <- countIsoformHits(tumor_hits)
##   ## tumor_splices <- summarizeSplices(tumor)


## ###################################################
## ### code chunk number 39: combine-samples
## ###################################################
##   assays <- do.call(mapply, c(list(cbind), lst_counts, list(SIMPLIFY = FALSE)))
##   ## assays <- mapply(cbind, normal_counts, SIMPLIFY = FALSE)
##   ## assays <- mapply(cbind, normal_counts, tumor_counts, 
##   ##                  SIMPLIFY = FALSE)
##   colData <- DataFrame(tumorStatus =  names(bamFiles))
##   rownames(colData) <- colData$tumorStatus
##   se <- SummarizedExperiment(assays, tx, colData)

## ###################################################
## ### code chunk number 40: order-se
## ###################################################
##   uc <- assay(se, "unique")
##   uc_ord <- order(rowSums(uc), decreasing = TRUE)
##   uc_top <- uc[head(uc_ord, 2),]
##   ## fisher.test(uc_top)$estimate

## ###################################################
## ### code chunk number 41: get-unique-reads
## ###################################################
##   lst_uniq <- lapply(1:N, function(i){
##     getUniqueReads(lst_rr[[i]], lst_hits[[i]])
##   })
##   names(lst_uniq) <- nms
##   ## normal_uniq <- getUniqueReads(normal, normal_hits)
##   ## tumor_uniq <- getUniqueReads(tumor, tumor_hits)
##   both_uniq <- do.call(mstack, lapply(lst_uniq, unlist))

## ###################################################
## ### code chunk number 42: combine-splices
## ###################################################
##   lst_uniq_splices <- lapply(lst_uniq, summarizeSplices)
##   names(lst_uniq_splices) <- nms
##   uniq_splices <- do.call(mstack, lst_uniq_splices)
##   all_splices <- do.call(mstack, lst_splices)
##   novel_splices <- all_splices[values(all_splices)$novel]
##   ## novel_splices <-  all_splices[values(all_splices)$novel &
##   ##                               values(all_splices)$score == 9]
##   ## novel_splices <-  all_splices[values(all_splices)$novel &
##   ##                               values(all_splices)$score == 9]
##   ## uniq_novel_splices <- c(uniq_splices, novel_splices)
##   ## uniq_novel_splices <- c(uniq_splices, all_splices)
##   uniq_novel_splices <- c(uniq_splices, novel_splices)  
##   both_uniq <- keepSeqlevels(both_uniq, unique(as.character(seqnames(both_uniq))))
##   .wt <- max(width(uniq_novel_splices))/max(coverage(both_uniq)) * weight
##   .wt <- as.numeric(.wt)
##   aes.res <- do.call(aes,list(size = substitute(score), 
##                               height = substitute(width/.wt, list(.wt = .wt)),
##                               color = substitute(novel)))

##   message("Constructing splicing graphics....")
##   p.novel <- do.call(geom_arch, c(list(data = uniq_novel_splices,
##                                     ylab = "coverage",
##                                        rect.height = 0), list(aes.res)))
##   p.s <- ggplot() + p.novel + do.call(stat_coverage, list(data = both_uniq, facets = name~.))
##   message("Constructing gene model....")
##   if(length(txdb.chr.pre))
##     wh <- GRanges(gsub(txdb.chr.pre, "", as.character(seqnames(wh))),
##                   ranges(wh))
##   ## browser()
##   tx_un <- stack(tx, ".sample")
##   tx_cur <- keepSeqlevels(tx_un, unique(as.character(seqnames(tx_un))))
##   tx_cur <- tx_cur[, setdiff(colnames(values(tx_cur)), c("tx_id", "gene_id"))]
##   tx_cur <- split(tx_cur, values(tx_cur)$.sample)
##   tx_track <- do.call(autoplot, list(object = tx_cur, geom = "alignment", ylab = ""))
##   ## tx_track <- autoplot(tx_16, geom = "alignment", ylab = "")
##   ## tx_track <- do.call(autoplot, list(object = txdb, which = wh))
##   if(missing(xlim))
##     xlim <- c(start(wh), end(wh))
##   tracks(p.s, tx_track, xlim  = xlim, heights = c(3, 1))
## }

setGeneric("mold", function(data, ...) standardGeneric("mold"))

setMethod("mold", c("eSet"), function(data){
  object <- data
  pdata <- phenoData(object)
  df <- as(object, "data.frame")
  df$sampleNames <- row.names(df)
  df.m <- melt(df, id.vars =  c(varLabels(pdata), "sampleNames"))
  df.m
})

setMethod("mold", c("GRanges"), function(data){
  names(data) <- NULL
  df <- as.data.frame(data)
  df$midpoint <- (df$start+df$end)/2
  df
})

setMethod("mold", c("GRangesList"), function(data, indName = "grl_name"){
  gr <- biovizBase:::flatGrl(data, indName)
  df <- mold(gr)
  df
})

setMethod("mold", c("IRanges"), function(data){
  df <- as.data.frame(data)
  df$midpoint <- (df$start+df$end)/2
  df
})

setMethod("mold", c("Seqinfo"), function(data){
  df <- .transformSeqinfo(data)
  df <- mold(df)  
  df <- cbind(df, as.data.frame(data))
  df
})


setMethod("mold", c("matrix"), function(data){
  x <- seq_len(ncol(data))
  y <- seq_len(nrow(data))
  cnms <- colnames(data)
  rnms <- rownames(data)
  df <- expand.grid(x = x, y = y)
  df$value <- as.vector(t(data))
  df$row <- df$y
  df$col <- df$x  
  df$colnames <- cnms[df$x]
  df$rownames <- rnms[df$y]
  df
})


setMethod("mold", c("ExpressionSet"), function(data){
  df <- mold(exprs(data))
  pd <- pData(data)
  idx <- seq_len(ncol(exprs(data)))
  res <- cbind(df, pd[df$x, ])
  res
})

setMethod("mold", c("RangedSummarizedExperiment"), function(data, assay.id = 1){
  ays <- assays(data)
  stopifnot(length(assay.id) == 1 || length(assay.id) <= length(ays))
  if(length(ays) > 1)
    message("Assay index: ", assay.id, " used")
  cd <- as.data.frame(colData(data))
  rd <- as.data.frame(rowRanges(data))
  df <- mold(ays[[assay.id]])
  cd.e <- cd[df$x, ]
  rd.e <- rd[df$y, ]
  res <- cbind(df, cd.e, rd.e)
  res
})

## Remove this method once the transition from SummarizedExperiment to
## RangedSummarizedExperiment is complete. [H. Pages - May 12, 2015]
setMethod("mold", c("SummarizedExperiment"), function(data, assay.id = 1){
  ays <- assays(data)
  stopifnot(length(assay.id) == 1 || length(assay.id) <= length(ays))
  if(length(ays) > 1)
    message("Assay index: ", assay.id, " used")
  cd <- as.data.frame(colData(data))
  rd <- as.data.frame(rowRanges(data))
  df <- mold(ays[[assay.id]])
  cd.e <- cd[df$x, ]
  rd.e <- rd[df$y, ]
  res <- cbind(df, cd.e, rd.e)
  res
})

## setMethod("mold", c("VCF"), function(data,
##                                                    type = c("geno", "info", "fixed"),
##                                                    assay.id = 1){
##   type <- match.arg(type)  
##   hdr <- metadata(object)[["header"]]
##   nms <- rownames(geno(hdr))
##   res <- switch(type,
##                 geno = {
##                   nms <- rownames(geno(hdr))
##                   if(is.numeric(assay.id)){
##                     nm <- nms[assay.id]  
##                   }else if(is.character(assay.id) && assay.id %in% nms){
##                     nm <- assay.id
##                   }else{
##                     stop("wrong assay.id specified, only numeric value which indicates
##                           index of data set under each type or name for that data set.")
##                   }
##                   res <- geno(object)[[nm]]

##                 },
##                 info = {
                  
##                 },
##                 fixed = {
                  
##                 })
  
## })


setMethod("mold", c("Views"), function(data){
  df.r <- as.data.frame(ranges(data))
  nms <- names(data)
  if(!is.null(nms))
    names(data) <- NULL
  mx <- suppressWarnings(as.matrix(data))
  df <- mold(mx)
  df$start <- rep(df.r$start, each = ncol(mx))
  df$end <- rep(df.r$end, each = ncol(mx))
  df$width <- rep(df.r$width, each = ncol(mx))
  df$midpoint <- (df$start + df$end)/2
  if(!is.null(nms))
    df$rowname <- rep(nms, each = ncol(mx))
  df$row <- factor(df$y)
  df$col <- factor(df$x)  
  df
})

## setMethod("mold", c("BamFile"), function(data){

## })

## setMethod("mold", c("GAlignments"), function(data){

## })

## setMethod("mold", c("TxDb"), function(data){
  
## })

setMethod("mold", c("Rle"), function(data){
  df <- as.data.frame(data)
  df$x <- as.numeric(rownames(df))
  df$y <- 1
  df$row <- factor(df$y)
  df$col <- factor(df$x)
  df
})

setMethod("mold", c("RleList"), function(data){
  data <- xRleList
  nms <- names(data)
  if(is.null(nms))
    nms <- seq_len(length(data))
  lst <- lapply(1:length(data), function(i){
    x <- data[[i]]
    df <- mold(x)
    df$group <- nms[i]
    df
  })
  res <- do.call(rbind, lst)
  res
})


setMethod("mold", "VRanges", function(data){
  res <- as.data.frame(data, row.names=NULL)
  res$midpoint <- (res$start + res$end)/2
  res
})

containLetters <- function(obj, all = FALSE){
    obj <- as.character(obj)
    obj <- tolower(obj)
    obj <- unlist(strsplit(obj,""))
    if(!all){
        res <- any(obj%in%letters)
    }else{
        res <- all(obj%in%letters)
    }
    res
}

GCcontent <- function(obj, ..., view.width, as.prob = TRUE){
    require(BSgenome)
    seqs <- getSeq(obj, ..., as.character = FALSE)
    if(missing(view.width)){
        res <- letterFrequency(seqs, letters="CG", as.prob = as.prob)
    }else{
        res <- lapply(seqs, function(x) letterFrequencyInSlidingView(x, view.width,
                                                                     letters="CG", as.prob = as.prob))
        if(length(res) == 1)
            res <- unlist(res)
    }
    res
}


centroidPos <- function(obj){
    if(!isIdeogram(obj))
        stop("Please pass an ideogram, please check function getIdeogram")
    obj.s <- split(obj, as.character(seqnames(obj)))
    lst <- lapply(obj.s,function(x){
        arms <- substr(values(x)$name, 1,1)
        idx <- which(arms == "q")[1]
        data.frame(x = start(x)[idx], y = 5, seqnames = unique(as.character(seqnames(x))))
    })
    df <- do.call(rbind, lst)
    df
}

## ## utils to generate pair-end
pspanGR <- function(file, region, sameChr = TRUE, isize.cutoff = 170){
    ## FIXME: move unmated?
    bam <- scanBam(file, param=ScanBamParam(which = region),
                   flag = scanBamFlag(hasUnmappedMate = FALSE))
    bam <- bam[[1]]
    bamrd <- GRanges(bam$rname, IRanges(bam$pos, width = bam$qwidth),
                     strand = bam$strand,
                     mseqname = bam$mrnm,
                     mstart = bam$mpos,
                     isize = bam$isize)
    ## why negative?sometime
    bamrd <- bamrd[abs(bam$isize) >= isize.cutoff]
    if(sameChr){
        idx <- as.character(seqnames(bamrd)) == values(bamrd)$mseqname
        bamrd <- bamrd[idx]
    }
    if(length(bamrd)){
        p1 <- GRanges(seqnames(bamrd),
                      ranges(bamrd))
        p2 <- GRanges(values(bamrd)$mseqname,
                      IRanges(values(bamrd)$mstart, width = 75))
        pspan <- punion(p1, p2, fill.gap = TRUE)
        pgaps <- pgap(ranges(p1), ranges(p2))
        return(list(pspan = pspan, pgaps = pgaps, p1 = p1, p2 = p2))
    }else{
        return(NULL)
    }
}


genSymbols <- function(org){
    ## TODO, add gene id
    if(!is(org, "OrgDb")){
        stop("org must be a OrgDb object")
    }
    pkg.name <- paste("package", org$packageName, sep = ":")
    sym.name <- grep("SYMBOL$", ls(pkg.name), value = TRUE)
    chr.name <- grep("CHR$", ls(pkg.name), value = TRUE)
    chrloc.name <- grep("CHRLOC$", ls(pkg.name), value = TRUE)
    chrlocend.name <- grep("CHRLOCEND$", ls(pkg.name), value = TRUE)
    en.name <- grep("ENSEMBL$", ls(pkg.name), value = TRUE)
    ## a little hack
    sym.obj <- eval(as.name(sym.name))
    chr.obj <- eval(as.name(chr.name))
    chrloc.obj <- eval(as.name(chrloc.name))
    chrlocend.obj <- eval(as.name(chrlocend.name))
    en.obj <- eval(as.name(en.name))
    symbol_id <- mappedRkeys(sym.obj)
    message("get genen id...")
    gene_id <- mget(symbol_id, revmap(sym.obj), ifnotfound=NA)
    gene_id <- unlist2(gene_id)
    gene_id <- gene_id[!is.na(gene_id)]
    message("get chromosome information...")
    chr <- toTable(chr.obj[gene_id])
    chrloc <- toTable(chrloc.obj[gene_id])
    chrlocend <- toTable(chrlocend.obj[gene_id])
    chrs <- chrloc
    chrs$end_location <- chrlocend$end_location
    res <- chrs
    if(sum(duplicated(gene_id)))
        stop("Duplicated gene_id")# no duplicate
    idx <- match(res$gene_id, gene_id)
    res$symbol <- names(gene_id[idx])
    if(sum(with(res, start_location > 0 & end_location < 0)))
        stop("Wierd pattern found")
    if(sum(with(res, start_location < 0 & end_location > 0)))
        stop("Wierd pattern found")
    message("get strand information...")
    res$sense <- with(res, start_location>=0&end_location>=0)
    res.sense <- res[res$sense,]
    res.antisense <- res[!res$sense,]
    res$strand <- strand(!res$sense)
    message("get EMSEMBL ID...")
    ensembl_id <- unlist(mget(gene_id, en.obj))
    idx <- match(res$gene_id, names(ensembl_id))
    res$ensembl_id <- ensembl_id[idx]
    message("generating a GRanges object for genen symbols...")
    genesymbol <- with(res,GRanges(seqnames = paste("chr",Chromosome,sep = ""),
                                   IRanges(abs(start_location), abs(end_location)),
                                   strand = strand,
                                   symbol = symbol,
                                   ensembl_id = ensembl_id))
    names(genesymbol) <- values(genesymbol)$symbol
    message("Done")
    genesymbol
}

setGeneric("getGaps", function(obj, ...) standardGeneric("getGaps"))
setMethod("getGaps", "GRanges", function(obj, group.name = NULL, facets = NULL){
    if(!length(obj))
        return(GRanges())
    if(!length(facets))
        facets <- as.formula(~seqnames)
    allvars.extra <- colnames(mcols(obj))
    if(!is.null(group.name)){
        if(!group.name %in% colnames(values(obj)))
            stop(group.name, " is not in obj")
        grl <- splitByFacets(obj, facets)
        grl <- endoapply(grl, function(dt){
            res <- split(dt, values(dt)[,group.name])
            gps <- gaps(ranges(res))
            idx <- elementNROWS(gps) > 0
            res.sub <- res[idx,]
            stds <- unlist(lapply(res.sub, function(x) as.character(strand(x))[1]))
            gps.sub <- gps[idx,]
            dfs <- values(unlist(res.sub))[cumsum(elementNROWS(res.sub)),c(allvars.extra, "stepping"),
                                           drop = FALSE]

            ir <- unlist(gps.sub)
            if(length(ir)){
                gr <- GRanges(unique(seqnames(dt)), ir)
                values(gr) <- dfs[togroup(PartitioningByWidth(gps.sub)),,
                                  drop = FALSE]
                strand(gr) <- stds[togroup(PartitioningByWidth(gps.sub))]
            }else{
                gr <- GRanges()
            }
            gr
        })
        res <- unlist(grl)
    }else{

    }
    if(length(res)){
        values(res)$type <- "gaps"
        res <- resize(res, width = width(res) + 2, fix = "center")
    }else{
        GRanges()
    }
    res
})



getIdeoGR <- function(gr){
    if(!is(gr, "GenomicRanges"))
        stop("require GenomicRanges")
    if(all(is.na(seqlengths(gr)))){
        warning("geom(ideogram) need valid seqlengths information for accurate mapping,
                 now use reduced information as ideogram... ")
        res <- range(reduce(gr, ignore = TRUE))
        start(res) <- 1
        res
    }else{
        res <- as(seqinfo(gr), "GenomicRanges")
    }
    res
}

## get major/mionr scale, y value, major text?
getScale <- function(gr, unit = NULL, n = 100, type = c("M", "B", "sci")){
    type <- match.arg(type)
    if(is.null(unit)){
        unit <- sum(as.numeric(width(gr)))/n
        unit <- 10^floor(log10(unit))
    }
    ## not like normal scale
    res <- split(gr, seqnames(gr))
    grl <- endoapply(res, function(gr){
        st <- 1
        ed <- end(gr)
        major.pos <- seq(st, ed, by = 5*unit)
        minor.pos <- seq(st, ed, by = unit)
        minor.pos <- setdiff(minor.pos, major.pos)
        if(type == "M"){
            texts <- c(paste(as.character(round(major.pos/1e6, digits = 1)), "M",
                             sep = ""),
                       rep("", length(minor.pos)))
        }
        if(type == "B"){
            texts <- c(paste(as.character(round(major.pos/1e9, digits = 1)), "B",
                             sep = ""),
                       rep("", length(minor.pos)))
        }
        if(type == "sci"){
            texts <- c(as.character(format(major.pos, scientific = TRUE, digit = 1)),
                       rep("", length(minor.pos)))
        }
        GRanges(seqnames(gr),
                IRanges(start = c(major.pos, minor.pos),
                        width = 1),
                type = c(rep("major", length(major.pos)), rep("minor", length(minor.pos))),
                scale.y = c(rep(3, length(major.pos)),
                    rep(1, length(minor.pos))),
                text.major = texts)
    })
    res <- unlist(grl)
    seqlengths(res) <- seqlengths(gr)
    res
}

getFormalNames <- function(..., remove.dots = TRUE){
    res <- lapply(list(...), function(fun){
        if(is.function(fun)){
            res <- names(formals(fun))
            if(remove.dots)
                res <- res[ res != "..."]
            res
        }else{
            stop("arguments passed to getFormalNames must be functions")
        }
    })
    res <- unlist(res)
    res <- res[!duplicated(res)]
    res
}


subsetArgsByFormals <- function(args, ..., remove.dots = TRUE){
    .formals <- getFormalNames(..., remove.dots = remove.dots)
    res <- args[names(args) %in% .formals]
    res
}

flatGrl <- function(object, indName = "grl_name"){
    idx <- togroup(PartitioningByWidth(object))
    gr <- stack(object, indName)
    values(gr) <-   cbind(values(gr), values(object)[idx,,drop = FALSE])
    gr
}


## test heterogeneity
is_homo <- function(grl){
    all(unlist(lapply(grl, function(gr){
        length(unique(as.character(seqnames(gr)))) == 1
    })))
}


genGenesymbolTable <- function(db, keys, keytype = "SYMBOL",
                               columns = c("CHR", "CHRLOC", "CHRLOCEND", "SYMBOL"), unique = TRUE){

    res <- select(db, keys = keys, keytype = keytype, columns = columns)
    chr <- res[, "CHR"]
    s <- substr(res[, "CHRLOC"], 1, 1)
    s[s != "-"] <- "+"

    st <- as.numeric(substr(res[, "CHRLOC"], 2, nchar(res[, "CHRLOC"])))
    ed <- as.numeric(substr(res[, "CHRLOCEND"], 2, nchar(res[, "CHRLOCEND"])))
    sym <- res[, "SYMBOL"]

    res.gr <- GRanges(chr, IRanges(start = st, end = ed), strand = s, symbol = sym)
    if(unique){
        res.gr.l <- split(res.gr, sym)
        syms <- sapply(res.gr.l, function(x) unique(x$symbol))
        res.gr.l <- range(res.gr.l)
        res.gr <- unlist(res.gr.l)
        names(res.gr) <- syms
    }
    res.gr
}


## setGeneric("fetch", function(obj, ...) standardGeneric("fetch"))
fetch <- function(obj, which, ..., gene.id,
                  truncate.gaps = FALSE,
                  truncate.fun = NULL,
                  ratio = 0.0025,
                  resize.extra = 10,
                  include.level = TRUE,
                  use.names = TRUE,
                  columns = c("tx_id", "tx_name","gene_id"),
                  type = c("all", "single", "exons.reduce", "exons.all",
                      "exons.geneid",  "gapped.pair", "raw", "all")){
    .txdb.type <- c("all", "single", "exons.reduce",
                    "exons.all", "exons.geneid")
    .bamfile.type <- c("gapped.pair", "raw", "all")

    ## type <- match.arg(type)
    if(length(type) >1){
        if(is(obj,"TxDb"))
            type <- "all"
        if(is(obj,"BamFile"))
            type <- "gapped.pair"
    }
    if(is(obj, "TxDb")){
        if(!(type %in% .txdb.type))
            stop("type for TxDb must be ", .txdb.type)
        if(is.list(which)){
            message("Parsing exons based on which(list) arguments")
            temp <- exons(obj, columns = columns, filter = which)
            which <- range(temp)
        }
        ## 1st set all the sequences to be inactive:
        isActiveSeq(obj)[seqlevels(obj)] <- FALSE
        ## Then set only "chr9" to be active:
        seqnms <- as.character(unique(seqnames(which)))
        isActiveSeq(obj)[seqnms] <- TRUE
        ## You can see which are active like this:
        if(type == "exons.geneid"){
            message("return exons within one gene...")
            ## this require gene id, only exons
            exons.gene <- exonsBy(obj, by = "gene")
            if(missing(gene.id))
                stop("Please specify the gene id")
            res <- exons.gene[[as.character(gene.id)]]
        }
        if(type == "exons.reduce"){
            message("Parsing exons...")
            exons <- exonsByOverlaps(obj, which, ...)
            exons <- reduce(exons)
            values(exons)$type <- "exon"
            res <- exons
        }
        if(type == "exons.all"){
            message("Parsing exons...")
            exons <- exonsByOverlaps(obj, which, ...)
            values(exons)$type <- "exon"
            res <- exons
        }
        if(type %in% c("all","single")){
            message("Parsing exons...")
            exons <- exonsBy(obj, "tx")
            message("Parsing cds...")
            cdss <- cdsBy(obj, "tx")
            message("Parsing transcripts...")
            tx <- transcripts(obj, columns = columns)
            tx <- subsetByOverlaps(tx, which)
            message("Aggregating...")
            txids <- values(tx)$tx_id
            lst <- lapply(txids, function(id){
                id <- as.character(id)
                tx_nm <- values(tx)[values(tx)$tx_id == id, "tx_name"]
                gene_id <- values(tx)[values(tx)$tx_id == id, "gene_id"]
                gene_id <- paste(unlist(gene_id), sep = ",")
                if(!length(gene_id))
                    gene_id <- NA
                exons.cur <- exons[[id]]
                values(exons.cur) <- data.frame(tx_id = id,
                                                tx_name = tx_nm,
                                                gene_id = gene_id,
                                                type = "exon")

                cds.cur <- cdss[[id]]
                seqs <- unique(as.character(seqnames(exons.cur)))
                exon_union <- reduce(exons.cur)
                ir.g <- IRanges(gaps(ranges(exons.cur)))
                if(length(ir.g)){
                    introns.cur <- GRanges(seqs,ir.g)
                    values(introns.cur) <- data.frame(tx_id = id,
                                                      tx_name = tx_nm,
                                                      gene_id = gene_id,
                                                      type = "intron")

                }else{
                    introns.cur <- GRanges()
                }
                if(!is.null(cds.cur)){
                    values(cds.cur) <- data.frame(tx_id = id,
                                                  tx_name = tx_nm,
                                                  gene_id = gene_id,
                                                  type = "cds")

                    cds_union <- reduce(cds.cur)
                    utrs <- setdiff(exon_union, cds_union)
                    if(length(utrs))
                        values(utrs) <- data.frame(tx_id = id,
                                                   tx_name = tx_nm,
                                                   gene_id = gene_id,
                                                   type = "utr")
                    else
                        utrs <- GRanges()
                    ir.g2 <- gaps(reduce(c(ranges(cds_union), ranges(utrs))))
                    if(length(ir.g2)){
                        gaps.cur <- GRanges(seqs,ir.g2)
                        values(gaps.cur) <- data.frame(tx_id = id,
                                                       tx_name = tx_nm,
                                                       gene_id = gene_id,
                                                       type = "gap")
                    }else{
                        gaps.cur <- GRanges()
                    }
                    gr <- c(exons.cur, introns.cur, cds.cur, gaps.cur, utrs)
                }else{
                    utrs <- exons.cur
                    values(utrs)$type <- factor("utr")
                    gaps.cur <- introns.cur
                    if(length(gaps.cur)){
                        values(gaps.cur)$type <- factor("gap")
                    }
                    gr <- c(exons.cur, introns.cur, utrs, gaps.cur)
                }
                gr
            })
            res <- do.call("c", lst)
            isActiveSeq(obj)[seqlevels(obj)] <- TRUE
            if(length(res))
                res <- keepSeqlevels(res, unique(as.character(seqnames(res))))
            else
                res <- GRanges()
        }
        if(type == "single"){
            if(length(res)){
                cds.s <- reduce(res[values(res)$type == "cds"])
                values(cds.s)$type <- factor("cds")
                exon.s <- reduce(res[values(res)$type == "exon"])
                values(exon.s)$type <- factor("exon")
                utr.s <- setdiff(exon.s, cds.s)
                values(utr.s)$type <- factor("utr")
                gap.s <- gaps(cds.s, start = min(start(cds.s)),
                              end = max(end(cds.s)))
                values(gap.s)$type <- factor("gap")
                res <- c(cds.s, utr.s, gap.s)
            }
        }
        message("Done")
    }
    if(is(obj, "GappedAlignments")){
        ## require(Rsamtools)
        ## require(Rsamtools)
        if(!missing(which))
            obj <- subsetByOverlaps(obj, which)
        if(!length(obj))
            stop("No entry in the GappedAlignments data")
        if(is.null(names(obj)))
            stop("Missing qname, please make use.names = TRUE, when reading GappedAlignments")
        grg <- grglist(obj)
        grg.u <- stack(grg, ".grl.name")
        message("extracting information...")
        values(grg.u)$junction <- rep(ifelse(elementNROWS(grg)>1, TRUE, FALSE),
                                      times = elementNROWS(grg))
        names(grg.u) <- NULL
        res <- grg.u
    }
    if(is(obj, "BamFile")){
        if(!(type %in% .bamfile.type))
            stop("type for TxDb must be ", .bamfile.type)
        ## require(Rsamtools)
        if(type == "gapped.pair"){
            message("Read GappedAlignments from BamFile...")
            ga <- readGAlignments(obj, param = ScanBamParam(which = which),
                                       use.names = use.names, ...)
            res <- fetch(ga)
        }

        if(type == "raw"){
            message("Read Raw from BamFile by calling scanBam...")
            res <- scanBamGRanges(obj, which, ...)
        }

        if(type == "all"){
            message("Read Raw from BamFile by calling scanBam...")
            res.mb <- scanBamGRanges(obj, which,
                                     flag = scanBamFlag(isFirstMateRead = TRUE))
            message("Read GappedAlignments from BamFile...")
            ga <- readGAlignments(obj, param = ScanBamParam(which = which),
                                       use.names = use.names, ...)
            res.gp <- fetch(ga)
            message("Combine...")
            nms <- values(res.mb)$qname
            ## fow now just, using isize
            isize <- values(res.mb)$issize
            names(isize) <- nms
            values(res.gp)$isize <- isize[values(res.gp)$qname]
            res <- res.gp
        }
    }
    if(truncate.gaps){
        if(is.null(truncate.fun)){
            if("gap" %in% unique(values(res)$type))
                idx <- values(res)$type %in% c("utr", "cds")
            res.s <- reduce(res[idx], ignore.strand = TRUE)
            truncate.fun <- shrinkageFun(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                         maxGap(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                                ratio = ratio))
        }
        res <- truncate.fun(res)
    }
    res
}

rectifySeqlevelsStyle <- function(x, y) {
    seqlevelsStyle(x) <- seqlevelsStyle(y)
    x
}

.transformSeqinfo <- function(obj){
    ss <- seqlengths(obj)
    res <- GRanges(seqnames(obj), IRanges(start = 1, end = ss))
    res <- keepSeqlevels(res, names(ss))
    seqlengths(res) <- ss
    res
}

setGeneric("crunch", function(obj, ...) standardGeneric("crunch"))
setMethod("crunch", "TxDb", function(obj, which,
                                     columns = c("tx_id", "tx_name","gene_id"),
                                     type = c("all", "reduce"),
                                     truncate.gaps = FALSE,
                                     truncate.fun = NULL,
                                     ratio = 0.0025){

    type <- match.arg(type)
    if(is.list(which)){
        message("Parsing exons based on which(list) arguments")
        temp <- exons(obj, columns = columns, filter = which)
        which <- range(temp)
    }

    seqnms <- as.character(unique(seqnames(which)))
    if(seqnms %in% seqlevels(obj)){
        seqlevels(obj, pruning.mode="coarse") <- seqnms
    }else{
        stop(seqnms, " is not matched with seqlevels of your object, please rename your 'which' arguments ")
    }
    ## seqlevels(obj, pruning.mode="coarse") <- seqnms
    on.exit(restoreSeqlevels(obj))  # needed only because TxDb are reference
                                    # objects (unlike R objects in general that
                                    # have a copy-on-change semantics)

    ## system.time(cdss <- cdsBy(obj, "tx")) #2.552s

    message("Parsing transcripts...")
    ## tx <- transcripts(obj, columns = columns)
    ## subsetByOverlaps(tx, which)

    tx <- transcriptsByOverlaps(obj, which, columns = columns)

    if(!length(tx)){
        message("No transcripts found at this region.")
        return(GRanges())
    }else{

        message("Parsing exons...")
        ## system.time(exons <- exonsBy(obj, "tx")) #takes 3.5s
        exons <- exonsByOverlaps(obj, which, columns = c("exon_id", "tx_id")) # 1.6
        txids <- unlist(exons$tx_id)
        idx <- togroup(PartitioningByWidth(exons$tx_id))

        exons <- exons[idx]
        exons <- exons[,1]
        exons$tx_id <- txids
        exons <- split(exons, exons$tx_id)

        message("Parsing cds...")
        cdss <- cdsByOverlaps(obj, which, columns = c("exon_id", "tx_id"))
        txids <- unlist(cdss$tx_id)
        idx <- togroup(PartitioningByWidth(cdss$tx_id))
        cdss <- cdss[idx]
        cdss <- cdss[,1]
        cdss$tx_id <- txids
        cdss <- split(cdss, cdss$tx_id)


        message("Parsing utrs...")
        txids <- as.character(values(tx)$tx_id)

        ## new method operate on GRangesList levels, and it is much faster to aggregate
        gids <- values(tx)$gene_id
        gids <- unlist(lapply(gids, function(x) do.call(paste0, list(as.list(x), collapse = ","))))

        tx_nm <- values(tx)$tx_name

        ## match table
        mt <- data.frame(txids, gids, tx_nm)
        rownames(mt) <- txids
        colnames(mt) <- c("tx_id", "gene_id", "tx_name")


        ## exons
        message("------exons...")
        gr.exons <- unlist(exons)
        .nms <- as.character(gr.exons$tx_id)
        .gid.nms <- mt[.nms, "gene_id"]
        .tx.nms <- mt[.nms, "tx_name"]
        values(gr.exons) <- NULL
        values(gr.exons) <- data.frame(tx_id = .nms, tx_name = .tx.nms,
                                       gene_id = .gid.nms, type = "exon")
        values(gr.exons)$type <-     as.character(values(gr.exons)$type)
        names(gr.exons) <- NULL

        ## cds
        message("------cdss...")
        gr.cdss <- unlist(cdss)
        .nms <- as.character(gr.cdss$tx_id)
        .gid.nms <- mt[.nms, "gene_id"]
        .tx.nms <- mt[.nms, "tx_name"]
        if(length(gr.cdss)){
            values(gr.cdss) <- NULL
            values(gr.cdss) <- data.frame(tx_id = .nms, tx_name = .tx.nms,
                                          gene_id = .gid.nms, type = "cds")
            values(gr.cdss)$type <-     as.character(values(gr.cdss)$type)
            names(gr.cdss) <- NULL
        }else{
            gr.cdss <- GRanges()
        }

        ## intron
        message("------introns...")
        irl.introns <- gaps(ranges(exons))
        ir.introns <- unlist(irl.introns)
        if(length(ir.introns)){
            .nms <- names(ir.introns)
            .gid.nms <- mt[.nms, "gene_id"]
            .tx.nms <- mt[.nms, "tx_name"]
            gr.introns <- GRanges(seqnms, ir.introns, tx_id = .nms,
                                  tx_name = .tx.nms, gene_id = .gid.nms,
                                  type = "gap")
            names(gr.introns) <- NULL
            values(gr.introns)$type <-  as.character(values(gr.introns)$type)
        }else{
            gr.introns <- GRanges()
        }
        ## utrs
        message("------utr...")
        if(length(exons) && length(cdss)){
            txnms <- intersect(names(exons), names(cdss))
            irl.utrs <- setdiff(ranges(exons[txnms]), ranges(cdss[txnms]))
            ir.utrs <- unlist(irl.utrs)
            if(length(ir.utrs)){
                .nms <- names(ir.utrs)
                .gid.nms <- mt[.nms, "gene_id"]
                .tx.nms <- mt[.nms, "tx_name"]
                gr.utrs <- GRanges(seqnms, ir.utrs, tx_id = .nms,
                                   tx_name = .tx.nms, gene_id = .gid.nms,
                                   type = "utr")
                names(gr.utrs) <- NULL
                values(gr.utrs)$type <-     as.character(values(gr.utrs)$type)
            }else{
                gr.utrs <- GRanges()
            }
        }else{
            gr.utrs <- GRanges()
        }
        ## combine
        res <- c(gr.exons, gr.cdss, gr.introns, gr.utrs)

        return(reduceNtruncate(res, type=type, truncate.gaps=truncate.gaps,
                               truncate.fun=truncate.fun, ratio=ratio))
    }
})

setMethod("crunch", "GAlignments", function(obj, which,
                                            truncate.gaps = FALSE,
                                            truncate.fun = NULL,
                                            ratio = 0.0025){
    if(!missing(which))
        obj <- subsetByOverlaps(obj, which)
    if(!length(obj))
        stop("No entry in the GAlignments data")
    if(is.null(names(obj)))
        stop("Missing qname, please make use.names = TRUE, when reading GAlignments")
    grg <- grglist(obj)
    grg.u <- stack(grg, ".grl.name")
    message("extracting information...")
    values(grg.u)$junction <- rep(ifelse(elementNROWS(grg)>1, TRUE, FALSE),
                                  times = elementNROWS(grg))
    names(grg.u) <- NULL
    res <- grg.u
    if(truncate.gaps){
        if(is.null(truncate.fun)){
            if("gap" %in% unique(values(res)$type))
                idx <- values(res)$type %in% c("utr", "cds")
            res.s <- reduce(res[idx], ignore.strand = TRUE)
            truncate.fun <- shrinkageFun(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                         maxGap(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                                ratio = ratio))
        }
        res <- truncate.fun(res)
    }
    res
})

setMethod("crunch", "BamFile", function(obj, which, ...,
                                        type = c("gapped.pair", "raw", "all"),
                                        truncate.gaps = FALSE,
                                        truncate.fun = NULL,
                                        ratio = 0.0025){

    type <- match.arg(type)
    ## require(Rsamtools)
    if(type == "gapped.pair"){
        message("Read GAlignments from BamFile...")
        ga <- readGAlignments(obj, param = ScanBamParam(which = which),
                                   use.names = TRUE, ...)

        res <- crunch(ga)
    }

    if(type == "raw"){
        message("Read Raw from BamFile by calling scanBam...")
        res <- scanBamGRanges(obj, which, ...)
    }

    if(type == "all"){
        message("Read Raw from BamFile by calling scanBam...")
        res.mb <- scanBamGRanges(obj, which,
                                 flag = scanBamFlag(isFirstMateRead = TRUE))
        message("Read GAlignments from BamFile...")
        ga <- readGAlignments(obj, param = ScanBamParam(which = which),
                                   use.names = TRUE, ...)

        res.gp <- crunch(ga)
        message("Combine...")
        nms <- values(res.mb)$qname
        ## fow now just, using isize
        isize <- values(res.mb)$issize
        names(isize) <- nms
        values(res.gp)$isize <- isize[values(res.gp)$qname]
        res <- res.gp
    }
    if(truncate.gaps){
        if(is.null(truncate.fun)){
            if("gap" %in% unique(values(res)$type))
                idx <- values(res)$type %in% c("utr", "cds")
            res.s <- reduce(res[idx], ignore.strand = TRUE)
            truncate.fun <- shrinkageFun(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                         maxGap(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                                ratio = ratio))
        }
        res <- truncate.fun(res)
    }
    res
})


scanBamGRanges <- function(file, which, what = c("rname", "strand", "pos", "qwidth"),
                           ...){
    ##  check
    .names <- c("rname", "strand", "pos", "qwidth")
    if(!all(.names %in% what)){
        what <- unique(c(what, .names))
    }
    dfnm <- setdiff(what, .names)
    param <- ScanBamParam(which = which, what = what,...)
    message("scanBam...")
    bam <- scanBam(file, param = param)
    message("Coerce list to GRanges")
    .names <- c("rname", "strand", "pos", "qwidth")
    res <- lapply(1:length(bam), function(i){
        bm <- bam[[i]]
        df <- as(bm, "DataFrame")[,dfnm]
        if(length(bm$rname))
            GRanges(bm$rname, IRanges(start = bm$pos, width = bm$qwidth),
                    strand = bm$strand, df)
        else
            GRanges()
    })
    bam.gr <- do.call("c", res)
}


####============================================================
##  crunch method from biovizBase
##
##  which can be a GRanges object or an object extending BasicFilter, or a
##  list of such filter objects.
####------------------------------------------------------------
setMethod("crunch", "EnsDb", function(obj, which,
                                      columns = c("tx_id",
                                                  "gene_name",
                                                  "gene_id"),
                                      type = c("all", "reduce"),
                                      truncate.gaps = FALSE,
                                      truncate.fun = NULL,
                                      ratio = 0.0025){
    type <- match.arg(type)
    ## If which is a GRanges convert that to a GRangesFilter, otherwise just
    ## pass on the "which" parameter as argument "filter" to the ensembldb
    ## methods.
    if(is(which, "GRanges")) {
        if(length(which) == 0) {
            message("No transcripts found at this region.")
            return(GRanges())
        }
        if(length(which) > 1)
            stop("'which' has to be a single GRanges object.")
        if(!all(is.na(genome(which)))){
            if(!all(genome(which) %in% genome(obj)))
                stop("Genome versions do not fit! Argument 'which' has ",
                     paste(genome(which), collapse=","), " argument 'obj' ",
                     paste(genome(obj), collapse=","), "!")
        }
        ## Check if we've got the seqnames.
        if(!all(seqnames(which) %in% seqlevels(obj)))
            stop(paste(seqlevels(which), collapse=", "),
                 " do(es) not match any seqlevel in argument 'obj'!")
        exFilter <- GRangesFilter(which, type = "any")
    } else exFilter <- which
    ## Check input argument 'columns':
    notAvailable <- !(columns %in% listColumns(obj))
    if(any(notAvailable)){
        if(all(notAvailable))
            stop("None of the columns specified by arguments 'columns' are available!",
                 " Allowed values are:", paste(listColumns(obj), collapse=", "), ".")
        ## Reducing to those which are allowed...
        warning("Columns ", paste(columns[notAvailable], collapse=", "), " are not",
                " available in the database and have been removed.")
        columns <- columns[!notAvailable]
    }
    ## Approach:
    ## 1) Get all exons in that region and retrieve also the
    ##    tx_coding_seq_start. We're fetching the data just once and
    ##    calculating everything we need from that, i.e. cds, utr and introns.
    message("Fetching data...", appendLF=FALSE)
    requiredCols <- c("tx_cds_seq_start", "tx_cds_seq_end", "exon_id", "tx_id")
    ## Forcing tx_id on the columns:
    columns <- unique(c(columns, "tx_id"))
    ## In order to solve also the "overlapping" condition I have first to fetch
    ## the transcripts in the region and their exons. That way we get, for a
    ## GRangesFilter or GRanges all transcripts that have an exon or an intron
    ## at the specified region.
    txInRegion <- transcripts(obj, filter = exFilter)
    if(length(txInRegion) == 0){
        message("No transcripts found at this region.")
        return(GRanges())
    }
    regExons <- exons(obj, filter = TxIdFilter(unique(txInRegion$tx_id)),
                      columns = unique(c(requiredCols, columns)))
    ## Simple sanitizing: check if what we got is on the same chromosome!
    if(length(unique(as.character(seqnames(regExons)))) > 1)
        stop("Got features from different chromosomes! Please adjust argument",
             " 'which' in order to fetch only features from a single chromosome.")
    message("OK")
    if(length(regExons) > 0){
        message("Parsing exons...", appendLF=FALSE)
        ## Get an DataFrame that we can use as mcols for the GRanges:
        mDf <- unique(mcols(regExons)[, columns])
        ## Actually, with this I have all I need.
        ## 2) Define introns.
        regExonsList <- split(regExons, regExons$tx_id)
        message("OK\nDefining introns...", appendLF=FALSE)
        ints <- gaps(ranges(regExonsList))
        ints <- unlist(ints)
        if(length(ints) > 0){
            regIntrons <- GRanges(seqnames=unique(seqnames(regExons)), ranges=ints,
                                  strand="*",
                                  mDf[match(names(ints), mDf$tx_id), ])
            regIntrons$type <- "gap"
        }else{
            regIntrons <- GRanges()
        }
        message("OK\nDefining UTRs...", appendLF=FALSE)
        ## 3) Define UTRs.
        codingTx <- regExons[!is.na(regExons$tx_cds_seq_end)]
        if(length(codingTx) > 0){
            ## Define the whole CDS region per tx
            codingReg <- GRanges(seqnames=seqnames(codingTx),
                                 ranges=IRanges(codingTx$tx_cds_seq_start,
                                                codingTx$tx_cds_seq_end),
                                 strand=strand(codingTx),
                                 tx_id=codingTx$tx_id)
            codingTx <- split(codingTx, codingTx$tx_id)
            ## Subset to one CDS region per tx and split
            codingReg <- codingReg[match(names(codingTx), codingReg$tx_id)]
            codingReg <- split(codingReg, codingReg$tx_id)
            regUTRs <- unlist(setdiff(codingTx, codingReg))
            mcols(regUTRs) <- mDf[match(names(regUTRs), mDf$tx_id), ]
            regUTRs$type <- rep("utr", length(regUTRs))
            message("OK\nDefining CDS...", appendLF=FALSE)
            ## 4) Define CDS
            regCDSs <- unlist(intersect(codingTx, codingReg))
            mcols(regCDSs) <- mDf[match(names(regCDSs), mDf$tx_id), ]
            regCDSs$type <- "cds"
        }else{
            regUTRs <- GRanges()
            regCDSs <- GRanges()
        }
        message("OK\n", appendLF=FALSE)
        regExons <- regExons[, columns]
        regExons$type <- "exon"
        ensRes <- c(regExons, regIntrons, regUTRs, regCDSs)
    }else{
        warning("Did not find any transcript at the specified region!")
        return(GRanges())
    }
    return(reduceNtruncate(ensRes, type=type, truncate.gaps=truncate.gaps,
                           truncate.fun=truncate.fun, ratio=ratio))

})

## Just some simple helper function to avoid repeating code for TxDb and EnsDb.
reduceNtruncate <- function(res, type=c("all", "reduce"),
                            truncate.gaps=FALSE, truncate.fun=NULL,
                            ratio = 0.0025){
        message("aggregating...")
        if(!length(res))
            res <- GRanges()
        res$type <- factor(res$type)

        if(type == "reduce"){
            if(length(res)){
                cds.s <- reduce(res[values(res)$type == "cds"])
                values(cds.s)$type <- factor("cds")
                exon.s <- reduce(res[values(res)$type == "exon"])
                values(exon.s)$type <- factor("exon")
                utr.s <- setdiff(exon.s, cds.s)
                values(utr.s)$type <- factor("utr")
                gap.s <- gaps(cds.s, start = min(start(cds.s)),
                              end = max(end(cds.s)))
                values(gap.s)$type <- factor("gap")
                res <- c(cds.s, utr.s, gap.s)
                ## change it to *
                strand(res) <- "*"
            }
        }
        if(truncate.gaps){
            message("truncating ...")
            if(is.null(truncate.fun)){
                if("gap" %in% unique(values(res)$type))
                    idx <- values(res)$type %in% c("utr", "cds")
                res.s <- reduce(res[idx], ignore.strand = TRUE)
                truncate.fun <- shrinkageFun(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                             maxGap(gaps(res.s, min(start(res.s)), max(end(res.s))),
                                                    ratio = ratio))
            }
            res <- truncate.fun(res)
        }
        message("Done")
        return(res)
}

.DNABasesNColor <- function(){
  c(A = "#ABD9E9",
    T = "#2C7BB6",
    G = "#D7191C",
    C = "#FDAE61",
    N = "#FFFFBF")
}

.DNABasesColor <- function(){
  c(A = "#ABD9E9",
    T = "#2C7BB6",
    G = "#D7191C",
    C = "#FDAE61")
}

.DNAAlphabetColor <- function(pal = c("BluetoDarkOrange.18",
                                "DarkRedtoBlue.18")){

  pal <- match.arg(pal)
  cols <- dscale(DNA_ALPHABET, dichromat_pal(pal))
  names(cols) <- DNA_ALPHABET
  cols
}

.RNABasesNColor <- function(){
  c(A = "#ABD9E9",
    U = "#2C7BB6",
    G = "#D7191C",
    C = "#FDAE61",
    N = "#FFFFBF")
}

.RNABasesColor <- function(){
  c(A = "#ABD9E9",
    U = "#2C7BB6",
    G = "#D7191C",
    C = "#FDAE61")
}

## blind.pal.info
.RNAAlphabetColor <- function(){
  ## pal <- match.arg(pal)
  ## cols <- dscale(RNA_ALPHABET, dichromat_pal(pal))
  N <- length(RNA_ALPHABET)
  cols <- colorBlindSafePal("Categorical.12")(N, repeatable = TRUE)
  names(cols) <- RNA_ALPHABET
  cols
}


.IUPACCodeMapColor <- function(){
  N <- length(IUPAC_CODE_MAP)
  ## cols <- dscale(IUPAC_CODE_MAP, dichromat_pal(pal))
  cols <- colorBlindSafePal("Categorical.12")(N, repeatable = TRUE)
  names(cols) <- names(IUPAC_CODE_MAP)
  cols
}

.strandColor <- function(){
  c("+" = "#1B9E77",
    "-" = "#7570B3",
    "*" = "#D95F02")
}

## preset gpos1:100 and other widely used color
.cytobandColor <- function(){
  .getCytobandColor(c("gneg", "gvar", "stalk", "acen", paste0("gpos", 1:100)))
}

## got these codes and hints from Florian Hahne, from Gviz
.getCytobandColor <- function(type){
    type <- as.character(type)
    ocols <-  c(gneg = "grey100",  gpos100 = "grey0", gvar = "grey100", 
                stalk = "brown3", acen = "brown4")
    cols <- c(ocols[c("gneg", "stalk", "acen")],
              gpos=unname(ocols["gpos100"]), gvar=unname(ocols["gpos100"]))
    gpcols <- unique(grep("gpos", type, value=TRUE))
    crmp <- colorRampPalette(c(cols["gneg"], cols["gpos"]))(100)
    posCols <- setNames(crmp[as.integer(gsub("gpos", "", gpcols))], gpcols)
    return(c(cols, posCols))
}


.AminoAcidCodeColor <- function(){
  N <- length(AMINO_ACID_CODE)
  cols <- colorBlindSafePal("Categorical.12")(N, repeatable = TRUE)
  names(cols) <- names(AMINO_ACID_CODE)
  cols
}


.AAAlphabetColor <- function(){
  N <- length(AA_ALPHABET)
  cols <- colorBlindSafePal("Categorical.12")(N, repeatable = TRUE)
  names(cols) <- AA_ALPHABET
  cols
}

getBioColor <- function(type = c("DNA_BASES_N",
                          "DNA_BASES",
                          "DNA_ALPHABET",
                          "RNA_BASES_N",
                          "RNA_BASES",
                          "RNA_ALPHABET",
                          "IUPAC_CODE_MAP",
                          "AMINO_ACID_CODE",
                          "AA_ALPHABET",
                          "STRAND",
                          "CYTOBAND"),
                        source = c("option",
                          "default")){

  type <- match.arg(type)
  source <- match.arg(source)

  if(source == "default"){
    cols <- switch(type,
                   DNA_BASES_N = .DNABasesNColor(),
                   DNA_BASES = .DNABasesColor(),
                   DNA_ALPHABET = .DNAAlphabetColor(),
                   RNA_BASES_N = .RNABasesNColor(),
                   RNA_BASES = .RNABasesColor(),
                   RNA_ALPHABET = .RNAAlphabetColor(),
                   IUPAC_CODE_MAP = .IUPACCodeMapColor(),
                   AMINO_ACID_CODE = .AminoAcidCodeColor(),
                   AA_ALPHABET = .AAAlphabetColor(),
                   STRAND = .strandColor(),
                   CYTOBAND = .cytobandColor())
  }

  if(source == "option"){
    colname <- switch(type,
                      DNA_BASES_N = "DNABasesNColor",
                      DNA_BASES = "DNABasesColor",
                      DNA_ALPHABET = "DNAAlphabetColor",
                      RNA_BASES_N = "RNABasesNColor",
                      RNA_BASES = "RNABasesColor",
                      RNA_ALPHABET = "RNAAlphabetColor",
                      IUPAC_CODE_MAP = "IUPACCodeMapColor",
                      AMINO_ACID_CODE = "AminoAcidCodeColor",                     
                      AA_ALPHABET = "AAAlphabetColor",
                      STRAND = "strandColor",
                      CYTOBAND = "cytobandColor")
    cols <- getOption("biovizBase")[[colname]]
    ## res <- cols[base]
  }
  cols
}

plotColorLegend <- function(colors, labels, title){
  if(missing(labels))
    labels <- names(colors)
  n <- length(colors)
  plot(c(0, 1), c(1, -n), type = "n", xlab="", ylab="", axes = FALSE)
  if(!missing(title))
    text(0.5, 1, title)
  rect(0.5, -(1:n), 1, -(1:n)+0.5+0.25, col = colors)
  text(0, -(1:n) + 0.5, labels)
}




## this should include as many as I cann
colorBlindSafePal <- function(palette){
  namelist <- rownames(blind.pal.info)
  if(missing(palette)){
    return(blind.pal.info)
  }
  if(is.numeric(palette) && (palette < nrow(blind.pal.info)))
    name <- rownames(blind.pal.info)[palette]
  else{
    if(is.character(palette) && (palette %in% namelist))
      name <- palette
    else
      stop("Invalid palette name, please check colorBlindSafePal().")
  }
  pkg <- blind.pal.info[name,"pkg"]
  maxcolors <- as.numeric(blind.pal.info[name,"maxcolors"])
  getPalColor <- function(n, pal, pkg){
    cols <- switch(pkg,
                   RColorBrewer = {
                     brewer.pal(n, pal)
                   },
                   dichromat = {
                     colorschemes[[pal]][seq(n)]
                   })
  }
  function(n, repeatable = FALSE){
    if (n < 3) {
      warning("minimal value for n is 3,
    returning requested palette with 3 different levels\n")
      cols <- getPalColor(3, name, pkg)
    }

    if (n > maxcolors) {
      if(!repeatable){
        warning(paste("n too large, allowed maximum for palette", 
                      name, "is", maxcolors, 
                      "\nReturning the palette you asked for with that many colors\n"))
        cols <- getPalColor(maxcolors, name, pkg)
      }else{
        ## repeatable colors
        rpt <- n %/% maxcolors
        res <- n %% maxcolors
        full.cols <- getPalColor(maxcolors, name, pkg)
        cols <- c(rep(full.cols, rpt), full.cols[-maxcolors][1:res])      
      }
    }else{
      cols <- getPalColor(n, name, pkg)
    }
    cols
  }
}

genBrewerBlindPalInfo <- function(){
  res <- RColorBrewer::brewer.pal.info
  idx.seq <- which(rownames(res)
                   %in% c("Blues","BuPu","GnBu","Greens",
                          "Greys","Oranges","OrRd","PuBu","PuBuGn",
                          "PuRd","Purples","Reds","YlGn","YlGnBu",
                          "YlOrBr","YlOrRd"))
  idx.div <- which(rownames(res)
                   %in% c("PiYG","PRGn","PuOr","RdBu","RdYlBu"))
  idx.qual <- which(rownames(res) %in% c("Dark2","Paired","Set2"))
  res <- res[c(idx.seq, idx.div, idx.qual),]
  res[rownames(res) %in% c("Dark2", "Set2"), 1] <- 3
  res[rownames(res) %in% c("Paired"), 1] <- 4
  res[, c("maxcolors", "category"), drop=FALSE]
}


## return current pallete which is color-blind safe, if missing n,
## return list of supported pal, if not, use index to return a set
## color. should have qual/div/seq
.colorschemes.num <-function(){
  lst <- gregexpr("[0-9]",names(colorschemes))
  unlist(lapply(1:length(lst), function(i){
  pos <- lst[[i]]
  if(length(pos) == 1)
    pos <- c(pos, pos)
  as.numeric(substr(names(colorschemes)[i], pos[1], pos[2]))
}))}

## make a similar color pal.info like brewer.pal.info
genDichromatPalInfo <- function(){
  dichromat.pal.info <- data.frame(maxcolors = .colorschemes.num(),
                                   category = "div", stringsAsFactors = FALSE)
  rownames(dichromat.pal.info) <- names(colorschemes)
  idx.seq <-  rownames(dichromat.pal.info) %in%
  c("LightBluetoDarkBlue.10", "LightBluetoDarkBlue.7", "SteppedSequential.5")   
  idx.qual <- rownames(dichromat.pal.info) %in% "Categorical.12"
  dichromat.pal.info[idx.seq, 2] <- "seq"
  dichromat.pal.info[idx.qual, 2] <- "qual"
  dichromat.pal.info <- dichromat.pal.info[order(dichromat.pal.info[,2]),]
  dichromat.pal.info                   
}



genBlindPalInfo <- function(){
  blind.pal.info <- rbind(brewer.pal.blind.info, dichromat.pal.blind.info)
  id <- seq(nrow(blind.pal.info))
  pkg <- c(rep("RColorBrewer", nrow(brewer.pal.blind.info)),
           rep("dichromat", nrow(dichromat.pal.blind.info)))
  blind.pal.info$pkg <- pkg
  idx <- order(blind.pal.info$category, blind.pal.info$maxcolors)
  blind.pal.info <- blind.pal.info[idx,]
  blind.pal.info$pal.id <- id
  blind.pal.info
}

brewer.pal.blind.info <- genBrewerBlindPalInfo()
dichromat.pal.blind.info <- genDichromatPalInfo()
blind.pal.info <- genBlindPalInfo()

## show color
show_col2 <- function(colours) {
  n <- length(colours)
  ncol <- ceiling(sqrt(n))
  nrow <- ceiling(n / ncol)
  
  colors <- c(colours, rep(NA, nrow * ncol - length(colours)))
  colors <- matrix(colors, ncol = ncol, byrow = TRUE)

  colours <- c(colours, rep(NA, nrow * ncol - length(colours)))
  nms <- matrix(names(colours), ncol = ncol, byrow = TRUE)
  
  old <- par(pty = "s", mar = c(0, 0, 0, 0))
  on.exit(par(old))
  size <- max(dim(colors))
  plot(c(0, size), c(0, -size), type = "n", xlab="", ylab="", axes = FALSE)
  rect(col(colors) - 1, -row(colors) + 1, col(colors), -row(colors),
    col = colors)
  text(col(colors) - 0.5, -row(colors) + 0.5, nms)
}

showColor <- function(colors, label = c("color", "name")){
  label <- match.arg(label)
  colfun <- switch(label,
                color = scales::show_col,
                name = show_col2)
  colfun(colors)
}

library(biovizBase)
## ------------------------------------------------------------
## getBioColor
## ------------------------------------------------------------
opts <- getOption("biovizBase")
opts$DNABasesNColor[1] <- "red"
options(biovizBase = opts)
## get from option(default)
getBioColor("DNA_BASES_N")
## get default fixed color
getBioColor("DNA_BASES_N", source = "default")
seqs <- c("A", "C", "T", "G", "G", "G", "C")
## get colors for a sequence.
getBioColor("DNA_BASES_N")[seqs]
getBioColor("DNA_BASES")
getBioColor("DNA_ALPHABET")
getBioColor("RNA_BASES_N")
getBioColor("RNA_BASES")
getBioColor("RNA_ALPHABET")
getBioColor("IUPAC_CODE_MAP")
getBioColor("AMINO_ACID_CODE")
getBioColor("AA_ALPHABET")
getBioColor("STRAND")
getBioColor("CYTOBAND")

## done
## ------------------------------------------------------------
## ColorBlindPal
## ------------------------------------------------------------
## brewer subse of only color blind palette
brewer.pal.blind.info
genBrewerBlindPalInfo()
## dichromat info
dichromat.pal.blind.info
genDichromatPalInfo()
## all color blind palette, adding id/pkg.
blind.pal.info
## with no parameters, just return blind.pal.info
colorBlindSafePal()
mypal <- colorBlindSafePal(20)
## or pass character name
mypal <- colorBlindSafePal("Set2")
mypal12 <- colorBlindSafePal(22)
showColor(mypal(12, repeatable = FALSE)) # warning
show_col(mypal(11, repeatable = TRUE))  # no warning, and repeat
show_col(mypal12(12))
showColor(getBioColor("CYTOBAND"), "name")
showColor(getBioColor("DNA_BASES"), "name")
plotColorLegend(getBioColor("DNA_BASES"))

mypalFun <- colorBlindSafePal(21)   
par(mfrow = c(1, 3))
showColor(mypalFun(4))
library(dichromat)
showColor(dichromat(mypalFun(4), "deutan"))
showColor(dichromat(mypalFun(4), "protan"))

getOption("biovizBase")$cytobandColor

par(mfrow = c(1, 3))
cols <- getBioColor("DNA_BASES_N", "default")
showColor(cols, "name")
cols.deu <- dichromat(cols, "deutan")
names(cols.deu) <- names(cols)
cols.pro <- dichromat(cols, "protan")
names(cols.pro) <- names(cols)
showColor(cols.deu, "name")
showColor(cols.pro, "name")


library(GenomicRanges)
set.seed(1)
N <- 500
gr <- GRanges(seqnames =
              sample(c("chr1", "chr2", "chr3", "chrX", "chrY"),
                     size = N, replace = TRUE),
              IRanges(
                      start = sample(1:300, size = N, replace = TRUE),
                      width = sample(70:75, size = N,replace = TRUE)),
              strand = sample(c("+", "-", "*"), size = N,
                replace = TRUE),
              value = rnorm(N, 10, 3), score = rnorm(N, 100, 30),
              group = sample(c("Normal", "Tumor"),
                size = N, replace = TRUE),
              pair = sample(letters, size = N,
                replace = TRUE))



head(addSteppings(gr))
head(addSteppings(gr, group.name = "pair"))
gr.close <- GRanges(c("chr1", "chr1"), IRanges(c(10, 20), width = 9))
addSteppings(gr.close)
addSteppings(gr.close, extend.size = 5)


gr.temp <- GRanges("chr1", IRanges(start = c(100, 250),
                                 end = c(200, 300)))
maxGap(gaps(gr.temp, start = min(start(gr.temp))))
maxGap(gaps(gr.temp, start = min(start(gr.temp))), ratio = 0.5)



gr1 <- GRanges("chr1", IRanges(start = c(100, 300, 600),
                               end = c(200, 400, 800)))
shrink.fun1 <- shrinkageFun(gaps(gr1), max.gap = maxGap(gaps(gr1), 0.15))
shrink.fun2 <- shrinkageFun(gaps(gr1), max.gap = 0)
library(ggbio)
p1 <- qplot(gr1)
p2 <- qplot(shrink.fun1(gr1))
p3 <- qplot(shrink.fun2(gr1))
tracks(p1, p2, p3)

gr2 <- GRanges("chr1", IRanges(start = c(100, 350, 550),
                               end = c(220, 500, 900)))
gaps.gr <- intersect(gaps(gr1, start = min(start(gr1))),
                     gaps(gr2, start = min(start(gr2))))
shrink.fun <- shrinkageFun(gaps.gr, max.gap = maxGap(gaps.gr))
p1 <- qplot(gr1)
p2 <- qplot(gr2)
p3 <- qplot(shrink.fun(gr1))
p4 <- qplot(shrink.fun(gr2))
tracks(p1, p2, p3, p4)

library(Rsamtools)
data(genesymbol)
library(BSgenome.Hsapiens.UCSC.hg19)    
bamfile <- system.file("extdata", "SRR027894subRBM17.bam", package="biovizBase")
test <- pileupAsGRanges(bamfile, region = genesymbol["RBM17"])
test.match <- pileupGRangesAsVariantTable(test, Hsapiens)

## library(rtracklayer)
library(biovizBase)
hg19IdeogramCyto <- getIdeogram("hg19", cytoband = TRUE)
hg19Ideogram <- getIdeogram("hg19", cytoband = FALSE)
unknowIdeogram <- getIdeogram()

head(ucscGenomes()$db)

data(hg19IdeogramCyto)
head(hg19IdeogramCyto)
data(hg19Ideogram)
head(hg19Ideogram)

## to be removed
library(biovizBase)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
data(genesymbol, package = "biovizBase")
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
txdb@organism
wh <- genesymbol["BRCA1"]
genome(txdb)
str(txdb)
obj <- txdb
## decrease from 18s to 13s, then to 4.7s
system.time(temp <- crunch(txdb, which = genesymbol["BRCA1"], type = "all"))
system.time(temp <- crunch(txdb, which = genesymbol["BRCA1"], type = "single"))

library("org.Hs.eg.db")
hdb <- org.Hs.eg.db
k <- keys(hdb, keytype = "SYMBOL")
res <- genGenesymbolTable(hdb, keys = c("BRCA1", "BRCA2"))
res <- genGenesymbolTable(hdb, keys = c("BRCA1", "BRCA2"), unique  = FALSE)

## test new OrganismDbi
library(Homo.sapiens)

## utils for creating proper scale breaks and labels for certain coordinate
setGeneric("getXScale", function(obj, ...) standardGeneric("getXScale"))
setMethod("getXScale", "GRanges", function(obj, type = c("default",
                                                  "all", "left", "right")){
  type <- match.arg(type)
  if(is_coord_truncate_gaps(obj)){
    message("using coord:truncate_gaps to parse x scale")
    res <- switch(type,
                  left = {
                    list(breaks = start(obj),
                         labels = start(values(obj)$.ori))
                  },
                  right = {
                    list(breaks = end(obj),
                         labels = end(values(obj)$.ori))
                  },
                  all = {
                    list(breaks = c(start(obj), end(obj)),
                         labels = c(start(values(obj)$.ori),
                           end(values(obj)$.ori)))
                  },
                  default = {
                    breaks = c(min(start(obj)), max(end(obj)))
                    labels = c(min(start(values(obj)$.ori)),
                      max(end(values(obj)$.ori)))
                    list(breaks = breaks, labels = labels)
                  })
  }else if(is_coord_genome(obj)){
    message("using coord:genome to parse x scale")
    res <- switch(type,
                  default = {
                      list(breaks = metadata(obj)$breaks,
                           labels = metadata(obj)$labels)
                  })
  }else{
    stop("coord other than truncate_gaps are not suported for getXScale yet")
  }
  res
})

##' This function summarize reads from bam files for nucleotides on
##' single base unit in a given region, this allows the downstream
##' mismatch summary analysis.
##'
##' It's a wrapper around \code{applyPileup} function in Rsamtools
##' package, more detailed control could be found under manual of
##' ApplyPileupsParam function in Rsamtools. \code{pileupAsGRanges} function
##' return a GRanges object which including summary of nucleotides,
##' depth, bam file path. This object could be read directly into
##' \code{pileupGRangesAsVariantTable} function for mismatch
##' summary.
##' @title Summarize reads for certain region
##' @param bams A character which specify the bam file path.
##' @param regions A GRanges object specifying the region to be
##' summarized. This passed to \code{which} arguments in
##' \code{ApplyPileupsParam}.
##' @param DNABases Nucleotide type you want to summarize in the result
##' and in specified order. It must be one or more of A,C,G,T,N.
##' @param ... Extra parameters passed to \code{ApplyPileupsParam}.
##' @return A GRanges object, each row is one single base unit. and
##' elementMetadata contains summary about this position about all
##' nucleotides specified by DNABases. and \code{depth} for total
##' reads, \code{bam} for file path.
##' @author  Michael Lawrence, Tengfei Yin
pileupAsGRanges <- function(bams, regions,
                            DNABases = c("A", "C", "G", "T", "N"),
                            ...) {
  pileupFiles <- PileupFiles(bams)
  applyPileupsParam <- ApplyPileupsParam(which = regions, ...)
  ## what if it's multiple bam files
  bamNames <- names(bams)
  if (is.null(bamNames))
    bamNames <- bams
  pileupFun <- function(x) {
    grl <- lapply(seq_len(length(bamNames)), function(i) {
      seq <- x$seq[DNABases,i,]
      sn <- unique(names(x$seqnames))
      GRanges(seqnames = Rle(names(x$seqnames), x$seqnames),
              ranges = IRanges(x$pos, width = rep(1L, length(x$pos))),
              strand = Rle("+", length(x$pos)),
              t(seq),
              depth = colSums(seq), bam = Rle(bamNames[i], length(x$pos)))
    })
    do.call(c, grl[!is.null(grl)])
  }
  gr <- do.call(c, applyPileups(pileupFiles, pileupFun,
                                param = applyPileupsParam))
  seqinfo(gr) <- seqinfo(regions)
  gr
}

##' Compare to reference genome and compute mismatch summary for
##' certain region of reads.
##'
##' User need to make sure to pass the right reference genome to this
##' function to get the right summary. This function drop the position
##' has no reads and only keep the region with coverage in the
##' summary. The result could be used to show stacked barchart for
##' mismatch summary.
##' @title Mismatch summary
##' @param gr A GRanges object, with nucleotides summary, each base
##' take one column in elementMetadata or user can simply passed the
##' returned result from \code{pileupAsGRanges} function to this
##' function.
##' @param genome BSgenome object, need to be the reference genome.
##' @param DNABases Nucleotide types contained in passed GRanges
##' object. Default is A/C/G/T/N, it tries to match the column names
##' in elementMetadata to those default nucleotides. And treat the
##' matched column as base names.
##' @return A GRanges object. Containing the following elementMetadata
##' \describe{
##'  \itemize{
##'  \item {[ref] Nucleotide in reference genome.}
##'  \item {[read] Nucleotide contained in the reads at particular position,
##'  if multiple nucleotide, either matched or unmatched are found, they will be
##' summarized in different rows.}
##'  \item {[count] Count for read column.}
##'  \item{[match] Logical value, whether matched to reference genome or not}
##'  \item{[bam] Character indicate bam file path.}
##' }
##' }
##' @author Michael Lawrence, Tengfei Yin
pileupGRangesAsVariantTable <- function(gr, genome,
                                        DNABases = c("A", "C", "G", "T", "N")) {
  require(BSgenome)
  colnms <- colnames(values(gr))
  DNAbases <- colnms[colnms %in% DNABases]
  gr <- rectifySeqlevelsStyle(gr, genome)
  refBases <- getSeq(genome, gr, as.character = TRUE)
  counts <- as.matrix(as.data.frame(values(gr)[,DNABases]))
  refCounts <- counts[cbind(seq(nrow(counts)), match(refBases, DNABases))]
  variantsForBase <- function(base) {
    baseCounts <- counts[,base, drop=TRUE]
    keep <- baseCounts > 0
    if(sum(keep)){
      count <- baseCounts[keep]
      gr <- GRanges(seqnames(gr)[keep], ranges(gr)[keep], strand(gr)[keep],
                    ref = refBases[keep], read = base,
                    count = baseCounts[keep],
                    values(gr)[keep, setdiff(colnames(values(gr)), DNABases)])
      values(gr)$match <- values(gr)$ref == values(gr)$read
      gr
    }else{
      NULL
    }
  }
  lst <- lapply(colnames(counts), variantsForBase)
  idx <- !sapply(lst, is.null)
  res <- do.call("c", lst[idx])  ## why this is slow
  if(length(res))
    return(res[order(as.numeric(start(res)))])
  else
    GRanges()
}

##----------------------------------------------------------------##
##                         ideogram
##----------------------------------------------------------------##
getIdeogram <- function(genome,subchr = NULL,cytobands=TRUE){
  .gnm <- genome
  lst <- lapply(.gnm, function(genome){
    print(genome)
  ## to remove the "heavy dependency" we put require here.
  require(rtracklayer)
  if(!(exists("session")&&extends(class(session),"BrowserSession")))
    session <- rtracklayer::browserSession()
  if(missing(genome)){
    choices <- rtracklayer::ucscGenomes()[,1]
    res <- menu(choices,title="Please specify genome")
    genome <- as.character(choices[res])
  }
  if(cytobands){
    message("Loading ideogram...")
    tryres <- try(query <-
                  rtracklayer::ucscTableQuery(session,"cytoBand",genome))
    if(!inherits(tryres, "try-error")){
      rtracklayer::tableName(query) <- "cytoBand"
      df <- rtracklayer::getTable(query)
      gr <- GRanges(seqnames=df$chrom,
                    IRanges(start=df$chromStart,end=df$chromEnd))
      values(gr) <- df[,c("name","gieStain")]
      message("Loading ranges...")
   
      gr.r <- rtracklayer::GRangesForUCSCGenome(genome)
      suppressWarnings(seqlengths(gr) <- seqlengths(gr.r)[names(seqlengths(gr))])
      gr <- trim(gr)
      
    }else{
      message("cytoBand informatin is not available, only get ranges.")
      message("Loading ranges...")
      gr <- rtracklayer::GRangesForUCSCGenome(genome)
      message("Done")      
    }
  }else{
    message("Loading...")
    gr <- rtracklayer::GRangesForUCSCGenome(genome)
    message("Done")
  }
  if(length(subchr))
    gr <- gr[seqnames(gr) == subchr]

  gr <- sort(gr)
  gr
  })
  names(lst) <- .gnm
  if(length(lst) == 1)
    res <- lst[[1]]
  else 
    res <- lst
  res
}

isIdeogram <- function(obj){
  is(obj, "GRanges") && all(c("gieStain", "name") %in% colnames(values(obj)))
}

isSimpleIdeogram <- function(obj){
  is(obj, "GRanges") && all(elementNROWS(split(obj, as.character(seqnames(obj)))) == 1)
}

setMethod("addStepping",c("GenomicRanges"),function(obj, group.name, extend.size = 0,
                                                    fix = "center",
                                                     group.selfish = TRUE){
  if(!missing(group.name)){
    if(! (group.name %in% colnames(values(obj))))
      stop("group.name must be one of the elementMetadta column")
  }else{
    group.name <- NULL
  }
  ## levels only make sense on different seqnames
  ## so split them first
  obj.lst <- split(obj,as.character(seqnames(obj)))
  lv <- endoapply(obj.lst,function(x){
    if(!is.null(group.name)){
      if(!group.selfish){
        x.n <- split(x, values(x)[,group.name])
        irs <- unlist(range(ranges(x.n)))
        irs.new <- resize(irs, fix = fix, width = width(irs) + extend.size)
        irs.new <- sort(irs.new)
        .lvs <- disjointBins(irs.new)
        values(x)$stepping <- .lvs[as.character(values(x)[,group.name])]
        x
      }else{
      values(x)$stepping <- as.numeric(as.factor(values(x)[,group.name]))
      x
    }
    }else{
      irs <- ranges(x)
      values(x)$stepping <- as.numeric(disjointBins(resize(irs, fix = "center",
                                  width = width(irs) + extend.size)))
      x
  }})
  res <- unlist(lv)
  res
})

getBioColor("DNA_BASES_N")
getBioColor("DNA_BASES")
getBioColor("DNA_ALPHABET")
getBioColor("RNA_BASES_N")
getBioColor("RNA_BASES")
getBioColor("RNA_ALPHABET")
getBioColor("IUPAC_CODE_MAP")
getBioColor("AA_ALPHABET")
getBioColor("STRAND")
getBioColor("CYTOBAND")

getBioColor("DNA_BASES_N", source = "default")
getBioColor("DNA_BASES", source = "default")
getBioColor("DNA_ALPHABET", source = "default")
getBioColor("RNA_BASES_N", source = "default")
getBioColor("RNA_BASES", source = "default")
getBioColor("RNA_ALPHABET", source = "default")
getBioColor("IUPAC_CODE_MAP", source = "default")
getBioColor("AA_ALPHABET", source = "default")
getBioColor("STRAND", source = "default")
getBioColor("CYTOBAND", source = "default")

## rules
## 1. add coord into metadata
## 2. never revise oringal
## 3. treak with labels
setGeneric("transformToGenome", function(data, ...) standardGeneric("transformToGenome"))
setMethod("transformToGenome", "GRanges", function(data, space.skip = 0.1, chr.weight = NULL){
  seqs.l <- seqlengths(data)
  if(all(!is.na(seqs.l))){
    chr.l <- seqs.l
    seqs.suml <- sum(as.numeric(seqs.l))
  }else{
    ## if no seqlengths are found, use data range
    chr.l <- max(end(split(data, seqnames(data))))
    seqs.suml <- sum(as.numeric(chr.l))
  }
  chr.l0 <- chr.l
  N <- length(chr.l)
  .start <- start(data)
  .end <- end(data)
  ## proportion specified for certain chromosomes
  if(!is.null(chr.weight)){
    if(any(!is.numeric(chr.weight)))
      stop("chr.weight must be a vector of numeric values.")
    if(sum(chr.weight) > 1)
      stop("sum of chr.weight must be equal or less than 1.")
    if(length(chr.weight) <= N){
      if(!all(names(chr.weight) %in% names(chr.l)))
        stop("chr.weight names must be a subset of seqinfo chromosome names")
      if(length(chr.weight) < N){
        .chrs <- setdiff(names(chr.l), names(chr.weight))
        .w <- chr.l[.chrs]/sum(as.numeric(chr.l[.chrs])) * (1 - sum(chr.weight))
        names(.w) <- .chrs
        chr.weight <- c(chr.weight, .w)
      }
    }else{
      stop("length of chr.weight cannot exceed length of seqinfo")
    }
    chr.l <- seqs.suml * chr.weight
    .start <- .start/chr.l0[as.character(seqnames(data))] * chr.l[as.character(seqnames(data))]
    .end <- .end/chr.l0[as.character(seqnames(data))] * chr.l[as.character(seqnames(data))]
  }  
  chr.l.back <- chr.l
  space.skip <- space.skip * seqs.suml
  skps <- space.skip * ((1:length(seqs.l))-1)
  names(skps) <- names(seqlengths(data))
  nms <- names(chr.l)
  chr.l <- cumsum(as.numeric(chr.l))
  chr.l2 <- c(0, chr.l[-length(chr.l)])
  .breaks <- chr.l2 + chr.l.back/2  + skps
  names(.breaks) <- nms
  names(chr.l2) <- nms
  sts.new <- .start + skps[as.character(seqnames(data))] +
    chr.l2[as.character(seqnames(data))] 
  ed.new <- .end + skps[as.character(seqnames(data))] +
    chr.l2[as.character(seqnames(data))] 
  max.chr <- rev(names(chr.l2))[1]
  x.max <- chr.l.back[max.chr] + skps[max.chr] + chr.l2[max.chr] + space.skip
  names(x.max) <- "genome"
  metadata(data)$breaks <- .breaks
  metadata(data)$labels <- names(.breaks)
  values(data)$.start <- round(sts.new)
  values(data)$.end <- round(ed.new)
  metadata(data)$space.skip <- space.skip
  metadata(data)$coord <- "genome"
  metadata(data)$x.max <- x.max
  data
})


setMethod("transformToGenome", "GRangesList", function(data, space.skip = 0.1, chr.weight = NULL){
  obj <- stack(data, ".group")
  obj <- transformToGenome(obj, space.skip = space.skip, chr.weight = NULL)
  obj
})
  
is_coord_genome <- function(data){
  if(!is.null(metadata(data)$coord)){  
   res <- metadata(data)$coord == "genome"
  }else{
    res <- FALSE
  }
  res
}

rescaleGr <- function(gr, maxSize = 1e8){
  if(!is_coord_genome(gr))
    stop("must pass a GRanges of coord genome")
  x.max <- metadata(gr)$x.max
  if(x.max > maxSize){
    values(gr)$.rescale.idx <- 1:length(gr)    
    gr <- gr[order(values(gr)$.start),]    
    .breaks <- metadata(gr)$breaks
    metadata(gr)$rescale <- TRUE
    .v <- rescale(c(1, .breaks, values(gr)$.start, values(gr)$.end), to = c(1, maxSize))
    .breaks <- .v[2:(1 + length(.breaks))]
    .v <- .v[-(1:(1+length(.breaks)))]
    N <- length(.v)
    res <- GRanges(seqnames(gr), IRanges(start  = .v[1:(N/2)],
                                     end = .v[(N/2+1):N]))
    values(res) <- values(gr)
    ## res <- sort(res)
    metadata(res) <- metadata(gr)
    metadata(res)$breaks <- .breaks
    res <- res[order(values(res)$.rescale.idx)]
  }else{
    res <- GRanges(seqnames(gr), IRanges(start  = values(gr)$.start,
                                     end = values(gr)$.end))
    values(res) <- values(gr)
    metadata(res) <- metadata(gr)
  }
  res
}

## then need a transformation to circlular view
## data is a GRanges dataect
transformToCircle <- function(data, x = NULL, y = NULL, ylim = NULL,
                      radius = 10, trackWidth =10, direction = c("clockwise", "anticlockwise"),
                      mul = 0.05){

  if(!".midpoint" %in% colnames(values(data)))
    values(data)$.midpoint <- (values(data)$.start + values(data)$.end)/2
  if(!length(y)){
    values(data)$.circle.equal.y <- 1
    y <- as.character(".circle.equal.y")
    ## stop("y is missing")
  }else if(!is.character(y) && !is.numeric(y)){
    stop("y must be character or numeric")
  }
  if(!length(x)){
    x <- ".midpoint"
  }else{
    if(x == "midpoint" & (!"midpoint" %in% colnames(values(data))))
      x <- ".midpoint"
  }

  if(length(ylim)){
    idx <- values(data)[,y] <= max(ylim) & values(data)[,y] >= min(ylim)
    data <- data[idx]
  }
  
  temp.x <- values(data)[,x] 
  
  if(is.character(y))
    temp.y <- values(data)[,y]
  if(is.numeric(y)){
    temp.y <- y
  }
  

  if(length(ylim))
    temp.y.r <- expand_range(ylim, mul = mul)
  else
    temp.y.r <- expand_range(range(temp.y), mul = mul)
  temp.y <- (temp.y - min(temp.y.r))/diff(temp.y.r) * trackWidth + radius
  
  ## always from 1 to max
  direction <- match.arg(direction)
  if(direction == "clockwise")
    dir <- 1
  else
    dir <- -1
  ## need to use max of "ideo"
  if(is_coord_genome(data)){
    x.max <- metadata(data)$x.max
  }else{
    x.max <- max(end(data))
  }
  angle.unit <- pi* 2/ x.max
  x.new <- temp.y * sin(angle.unit * (temp.x - 1) * dir)
  y.new <- temp.y * cos(angle.unit * (temp.x - 1)* dir)
  values(data)$.circle.x <- x.new
  values(data)$.circle.y <- y.new
  values(data)$.circle.angle <- angle.unit * (temp.x-1) * dir
  data
}

## interpolate segment or rectangle
## rectangle only use stepping as y

transformToRectInCircle <- function(data, y = NULL, space.skip = 0.1,
                                    trackWidth = 10, radius = 10,
                                    direction = c("clockwise", "anticlockwise"),
                                    n = 100, mul = 0.05,
                                    chr.weight = NULL){
  if(is_coord_genome(data))
    x.max <- values(data)$x.max
  else
    x.max <- max(end(data))
  x.unit <- x.max/n
  direction <- match.arg(direction)
  data.back <- data

    ## need to consider the space
  if(length(y)){
    temp.y <- as.character(values(data)[,as.character(y)])
    if(is.character(temp.y)){
      temp.y <- as.numeric(as.factor((temp.y)))
    }
    if(!all(check.integer(temp.y)))
      stop("geom(bar) require specified 'y', must be integer which indicates the levels")
    values(data)$stepping <- temp.y
  }else{
      values(data)$stepping <- disjointBins(data, ignore.strand = TRUE)
  }
  names(data) <- NULL
  df <- as.data.frame(data)

  lst <- lapply(1:nrow(df), function(i){
    if(df[i,"width"] > 1){
      .n <- round(df[i, "width"]/x.unit)
      if(.n< 5) .n <- 5
      inter.fun <- function(x, y) approx(x, y, n = .n)
      res.x <- as.integer(inter.fun(c(df[i,"start"], df[i,"end"]), c(0, 0))$x)
    }else{
      res.x <- c(df[i, "start"], df[i, "start"])
    }
    res <- data.frame(.biovizBase.new.x = c(res.x, rev(res.x)))
    N <- nrow(res)
    res$.biovizBase.group <- i
    res$.biovizBase.level <- c(rep(df[i, "stepping"] - 0.4, N/2),
                               rep(df[i, "stepping"] + 0.4, N/2))
    res <- suppressWarnings(cbind(res, df[i,]))
    res$.int.id <- seq_len(nrow(res))
    res
  })
  res <- do.call("rbind", lst)
  .y <- ".biovizBase.level"
  res.gr <- GRanges(res$seqnames, IRanges(start = res$.biovizBase.new.x,
                                          width = 1),
                    strand = res$strand)
  values(res.gr) <- subset(res, select = -c(start, end, width, strand,seqnames))
  seqlengths(res.gr) <- seqlengths(data)
  res <- transformToGenome(res.gr, space.skip = space.skip, chr.weight = chr.weight)
  res <- transformToCircle(res, y = .y, radius = radius, trackWidth = trackWidth,
                   direction = direction, mul = mul)
  res
}

transformToBarInCircle <- function(data, y = NULL, space.skip = 0.1, trackWidth = 10, radius = 10,
                     direction = c("clockwise", "anticlockwise"),
                                   n = 100, mul = 0.05, chr.weight = NULL){
  
  if(is_coord_genome(data))
    x.max <- values(data)$x.max
  else
    x.max <- max(end(data))
  x.unit <- x.max/n
  direction <- match.arg(direction)
  data.back <- data
    ## need to consider the space
  if(length(y)){
    values(data)$stepping <- values(data)[, as.character(y)]
  }else{
    if("score" %in% colnames(values(data))){
      message("use 'score' for default y, or you can specify it in aes()")
     values(data)$stepping <- values(data)$score
    }else{
      stop("please provide aes(y = ), or use 'score' as column")
    }
  }
  names(data) <- NULL
  df <- as.data.frame(data)
  lst <- lapply(1:nrow(df), function(i){
    if(df[i,"width"] > 1){
      .n <- round(df[i, "width"]/x.unit)
      if(.n< 5) .n <- 5
      inter.fun <- function(x, y) approx(x, y, n = .n)
      res.x <- as.integer(inter.fun(c(df[i,"start"], df[i,"end"]), c(0, 0))$x)
    }else{
      res.x <- c(df[i, "start"], df[i, "start"])
    }
    res <- data.frame(.biovizBase.new.x = c(res.x, rev(res.x)))
    N <- nrow(res)
    res$.biovizBase.group <- i
    res$.biovizBase.new <- c(rep(0, N/2),
                             rep(df[i, "stepping"], N/2))
    res <- suppressWarnings(cbind(res, df[i,]))
    res$.int.id <- seq_len(nrow(res))
    res
  })
  res <- do.call("rbind", lst)
  .y <- ".biovizBase.new"
  res.gr <- GRanges(res$seqnames, IRanges(start = res$.biovizBase.new.x,
                                          width = 1),
                    strand = res$strand)
  values(res.gr) <- subset(res, select = -c(start, end, width, strand,seqnames))
  seqlengths(res.gr) <- seqlengths(data)
  res <- transformToGenome(res.gr, space.skip = space.skip, chr.weight = chr.weight)
  res <- transformToCircle(res, y = .y, radius = radius, trackWidth = trackWidth,
                   direction = direction, mul = mul)
  res
}

## ok, segment allow user to use a flexible y
transformToSegInCircle <- function(data, y = NULL, space.skip = 0.1, trackWidth = 10, radius = 10,
                      direction = c("clockwise", "anticlockwise"),
                      n = 100, chr.weight = NULL){
  if(is_coord_genome(data))
    x.max <- values(data)$x.max
  else
    x.max <- max(end(data))
  x.unit <- x.max/n

  direction <- match.arg(direction)
  ## do the linear interpolatoin first

  if(!length(y)){
    values(data)$stepping <- disjointBins(ranges(data))
  }
  names(data) <- NULL
  df <- as.data.frame(data)
  lst <- lapply(1:nrow(df), function(i){
    if(df[i,"width"] > 1){
      .n <- round(df[i, "width"]/x.unit)
      if(.n< 5) .n <- 5
      inter.fun <- function(x, y) approx(x, y, n = .n)
      res.x <- as.integer(inter.fun(c(df[i,"start"], df[i,"end"]), c(0, 0))$x)
    }else{
      res.x <- c(df[i, "start"], df[i, "start"])
    }
    res <- data.frame(.biovizBase.new.x = res.x)
    res$.biovizBase.group <- i
    if(!length(y)){          
      res$.biovizBase.level <- df[i, "stepping"]
    }
    N <- nrow(res)
    df.extra <- do.call("rbind", lapply(1:N, function(k) df[i,]))
    res <- cbind(res, df.extra)
  })
  res <- do.call("rbind", lst)
  if(!length(y))
    .y <- ".biovizBase.level"
  else
    .y <- as.character(y)

  res.gr <- GRanges(res$seqnames, IRanges(start = res$.biovizBase.new.x,
                                          width = 1),
                    strand = res$strand)
  values(res.gr) <- subset(res, select = -c(start, end, width, strand,seqnames))
  seqlengths(res.gr) <- seqlengths(data)
  res <- transformToGenome(res.gr, space.skip, chr.weight = chr.weight)
  res <- transformToCircle(res, y = .y, radius = radius, trackWidth =trackWidth,
                   direction = direction)
  res
}


transformToSegInCircle2 <- function(data, y = NULL, space.skip = 0.1,
                                    trackWidth = 10, radius = 10,
                                    direction = c("clockwise", "anticlockwise"),
                                    n = 100, mul = 0.05, chr.weight = NULL){

  if(is_coord_genome(data))
    x.max <- values(data)$x.max
  else
    x.max <- max(end(data))
  x.unit <- x.max/n
  direction <- match.arg(direction)
  data.back <- data

    ## need to consider the space
  if(length(y)){
    temp.y <- as.character(values(data)[,as.character(y)])
    if(is.character(temp.y)){
      temp.y <- as.numeric(as.factor((temp.y)))
    }
    if(!all(check.integer(temp.y)))
      stop("geom(bar) require specified 'y', must be integer which indicates the levels")
    values(data)$stepping <- temp.y
  }else{
      values(data)$stepping <- disjointBins(data, ignore.strand = TRUE)
  }
  names(data) <- NULL
  df <- as.data.frame(data)

  lst <- lapply(1:nrow(df), function(i){
    if(df[i,"width"] > 1){
      res.x <- c((df[i, "start"] + df[i, "end"])/2, (df[i, "start"] + df[i, "end"])/2)      
    }else{
      res.x <- c(df[i, "start"], df[i, "start"])
    }
    res <- data.frame(.biovizBase.new.x = res.x)
    N <- nrow(res)
    res$.biovizBase.group <- i
    res$.biovizBase.level <- c(df[i, "stepping"] - 0.4,
                               df[i, "stepping"] + 0.4)
    res <- suppressWarnings(cbind(res, df[i,]))
    res$.int.id <- seq_len(nrow(res))
    res
  })
  res <- do.call("rbind", lst)
  .y <- ".biovizBase.level"
  res.gr <- GRanges(res$seqnames, IRanges(start = res$.biovizBase.new.x,
                                          width = 1),
                    strand = res$strand)
  values(res.gr) <- subset(res, select = -c(start, end, width, strand,seqnames))
  seqlengths(res.gr) <- seqlengths(data)
  res <- transformToGenome(res.gr, space.skip = space.skip, chr.weight = chr.weight)
  res <- transformToCircle(res, y = .y, radius = radius, trackWidth = trackWidth,
                   direction = direction, mul = mul)
  res  
}

## for a special GRanges
transformToLinkInCircle <- function(data, linked.to, space.skip = 0.1, trackWidth = 10,
                                    radius = 10, link.fun = function(x, y, n = 100)
                                    bezier(x, y, evaluation = n),
                                    direction = c("clockwise", "anticlockwise"),
                                    chr.weight = NULL){

  direction <- match.arg(direction)
  if(missing(linked.to))
    stop("linked.to must be provided and be a GRanges")
  ## trace id
  N <- length(data)
  values(data)$.biovizBase.idx <- 1:N
  values(values(data)[, linked.to])$.biovizBase.idx <- 1:N
  obj <- transformToGenome(data, space.skip, chr.weight = chr.weight)
  obj <- transformToCircle(obj, y = 0, radius = radius, trackWidth =trackWidth,
                   direction = direction)
  obj <- obj[order(values(obj)$.biovizBase.idx)]
  names(obj) <- NULL
  df <- as.data.frame(obj)
  linktodata <- values(data)[,linked.to]
  ## missing y
  linktodata <- transformToGenome(linktodata, space.skip, chr.weight = chr.weight)
  ## keep order
  linktodata <- transformToCircle(linktodata, radius = radius,
                          y = 0,
                          trackWidth =trackWidth,
                          direction = direction)
  linktodata <- linktodata[order(values(linktodata)$.biovizBase.idx)]  
  linkdf <- as.data.frame(linktodata)
  extra.df <- subset(df, select = -c(.circle.x, .circle.y))
  linkdf2 <- data.frame(from.x = df$.circle.x,
                        from.y = df$.circle.y,
                        to.x = linkdf$.circle.x,
                        to.y = linkdf$.circle.y)
  linkdf2 <- cbind(linkdf2, extra.df)
  lst <- lapply(1:nrow(linkdf2), function(i){
    ndf <- link.fun(c(linkdf2[i, "from.x"],0,linkdf2[i,"to.x"]),
                    c(linkdf2[i, "from.y"],0,linkdf2[i,"to.y"]))
    ndf <- as.data.frame(do.call("cbind", ndf))
    N <- nrow(ndf)
    ndf$.biovizBase.group <- i
    ## need to past other meta data
    extra.df <- subset(linkdf2[i,], select = -c(from.x, from.y, to.x, to.y))
    extra.df2 <- do.call("rbind", lapply(1:N, function(i) extra.df))
    ndf <- cbind(ndf, extra.df2)
  })
  res <- do.call("rbind", lst)
  res
}


## ddplyFun <- function(data, .fun, ..., window = 1e5){
##   grl <- split(data, seqnames(data))
##   idx <- elementNROWS(grl) > 0
##   grl <- endoapply(grl[idx], function(gr){
##     ## let's make it more works for multiple seqnames
##     bks <- seq(1, max(end(gr)), by = window)
##     N <- length(bks)
##     seqn <- unique(seqnames(gr))
##     if(length(seq) > 1)
##       stop("Multiple seqnames found")
##     if(!N%%2){
##       qgr <- GRanges(seqn, IRanges(start = bks[1:(N-1)],
##                                    end = bks[2:N]))
##     }else{
##       bks <- c(bks, max(end(gr)))
##       N <- length(bks)
##       qgr <- GRanges(seqn, IRanges(start = bks[1:(N-1)],
##                                    end = bks[2:N]))
##     }
##     if(max(end(qgr)) < max(end(gr))){
##       qgr <- c(qgr, GRanges(seqn, IRanges(max(end(qgr))+ 1,
##                                           max(end(gr)))))
##     }
##     of <- findOverlaps(gr, qgr, select = "first")
##     idx <- !is.na(of)
##     df <- as.data.frame(gr[idx])
##     df$of <- of[idx]
##     resdf <- plyr::ddply(df, .(of), .fun, ...)
##     idx <- as.numeric(resdf$of)
##     ## idx <- idx[!is.na(idx)]
##     res <- qgr[idx]
##     values(res) <- as.data.frame(resdf[,2])
##     res
##   })
##   res <- unlist(grl[!is.null(grl)])
##   names(res) <- NULL
##   sort(res)
## }

setGeneric("transformToDf", function(data, ...) standardGeneric("transformToDf"))

## setMethod("transformToDf", c("eSet"), function(model, data){
##   object <- model
##   pdata <- phenoData(object)
##   df <- as(object, "data.frame")
##   df$sampleNames <- row.names(df)
##   df.m <- melt(df, id.vars =  c(varLabels(pdata), "sampleNames"))
##   df.m
## })

setMethod("transformToDf", c("GRanges"), function(data){
  vals <- values(data)
  if(length(vals)){
    idx <- !unlist(lapply(vals@listData, function(x) is(x, "List") & !is(x, "DNAStringSet")))
    if(sum(!idx))
      warning(colnames(vals)[!idx], " column has been dropped for the reason that the corecion of class List is not supported")
    df <- as.data.frame(data[,idx])
  }else{
    df <- as.data.frame(data)
  }
  df$midpoint <- (df$start+df$end)/2
  df
})



transformDfToGr <- function(data, seqnames = NULL, start = NULL, end = NULL,
                            width = NULL, strand = NULL,
                            to.seqnames = NULL, to.start = NULL, to.end = NULL,
                            to.width = NULL, to.strand = NULL,
                            linked.to = "to.gr"){
  
  ## this losing seqinfo....
  colnms <- colnames(data)
  if(is.null(seqnames)){
    if("seqnames" %in% colnms)
      seqnames <- "seqnames"
    else
      stop("Please sepicify which column represent the seqnames")
  }

  seqnames <- data[, seqnames]
  if(length(c(start, end, width)) <2){
    stop("Pleaase sepeicfy two of start/end/width")
  }

  if(is.null(end)){
    if("end" %in% colnms)
      end <- data[,"end"]
  }else{
    end <- data[, end]
  }

  if(is.null(start)){
    if("start" %in% colnms)
      start <- data[,"start"]
  }else{
    start <- data[, start]
  }

  if(is.null(width)){
    if("width" %in% colnms)
      width <- data[,"width"]
  }else{
    width <- data[, width]
  }

  gr <- GRanges(as.character(seqnames), IRanges(start = start, width = width, end = end))
  if(is.null(strand)){
    if("strand" %in% colnms){
      strand <- data[, "strand"]
      strand(gr) <- strand
    }}else{
      strand(gr) <- data[, strand]
    }

  values(gr) <- data[,!colnames(data) %in% c(start, end, width, seqnames, strand)]
  ## linked ranges
  if(!is.null(to.seqnames)){
    message("try to creating a linked GRanges as column to.gr")
    if(length(c(to.start, to.end, to.width)) > 1){
      if(is.null(to.end)){
        if("to.end" %in% colnms)
          to.end <- data[,"to.end"]
      }else{
        to.end <- data[, to.end]
      }

      if(is.null(to.start)){
        if("to.start" %in% colnms)
          to.start <- data[,"to.start"]
      }else{
        to.start <- data[, to.start]
      }

      if(is.null(width)){
        if("to.width" %in% colnms)
          to.width <- data[,"to.width"]
      }else{
        to.width <- data[, to.width]
      }
      to.gr <- GRanges(seqnames = as.character(data[,to.seqnames]),
                       IRanges(start = to.start,
                               end = to.end,
                               width = to.width))
      if(!is.null(to.strand))
        strand(to.gr) <- data[,to.strand]
      values(gr)[,linked.to] <- to.gr
    }
  }
  gr
  
}

check.integer <- function(x){
  sapply(x, function(x)
    !length(grep("[^[:digit:]]", as.character(x))))
}


## retrun a new GRanges, with new coord
transformGRangesForEvenSpace <- function(gr){
  st <- start(range(gr))
  ed <- end(range(gr))
  wd <- width(range(gr))
  N <- length(gr)
  wid <- wd/N
  x.new <- st + wid/2 + (1:N - 1) * wid
  values(gr)$x.new <- x.new
  metadata(gr)$coord <- "even"
  gr
}

transformToEven <- function(data){
  st <- start(range(data))
  ed <- end(range(data))
  wd <- width(range(data))
  N <- length(data)
  wid <- wd/N
  x.new <- st + wid/2 + (1:N - 1) * wid
  gr <- GRanges(start = x.new, width = wid)
  values(gr) <- values(data)
  ## values(gr)$x.new <- x.new
  values(gr)$.ori <- data
  metadata(gr)$coord <- "even"
  data
}

## setGeneric("transform")
## setMethod("transform", "GRanges", function(_data, space.skip = 0.1,
##                                            maxSize = 1e9,
##                                            max.gap = 1L,
##                                            method = c("genome",
##                                              "truncate_gap",
##                                              "even")){
##   method <- match.arg(method)
##   obj <- switch(method,
##                 genome = {
##                   transformToGenome(_data, space.skip = space.skip)
##                 },
##                 truncate_gaps = {
                  
##                 },
##                 even = {
##                   transformToEven(_data, space.skip = space.skip)
##                 })
## }

## autoplot(gr, layout = "circle", geom = "ideo", rect.inter.n = 10, space.skip = 0.2,
##          aes(fill = .ori.seqnames))
## autoplot(gr, coord = "genome")
## autoplot(gr, layout = "circle",space.skip = 0.2, geom = "bar", grid = TRUE)
## gr2 <- transformToGenome(gr, space.skip = 0)
## gr2
## metadata(gr2)$coord <- NULL
## autoplot(gr2)

setGeneric("transformToArch", function(data, ...) standardGeneric("transformToArch"))
setMethod("transformToArch", "GRanges", function(data, width = 1){
  grl <- split(data, 1:length(data))
  grl <- endoapply(grl, function(gr){
    sts <- c(start(gr)-width, end(gr))
    res <- GRanges(seqnames(gr), IRanges(sts, width = width))
    values(res) <- values(gr)[c(1, 1),]
    seqlengths(res) <- seqlengths(gr)
    res
  })
  grl
})

setMethod("transformToArch", "GRangesList", function(data, width = 1){
  coord <- metadata(data)$coord
  res <- unlist(endoapply(data, function(gr){
    gps <- gaps(gr, start = min(start(gr)), end = max(end(gr)))
    gps <- gps[strand(gps) == "*"]
    values(gps) <- values(gr[1,])
    gps
  }))
  metadata(res)$coord <- coord
  res
})

## ## coord
## library(GenomicRanges)
## gr1 <- GRanges("chr1", IRanges(start = 1:10, width = 5))
## gr2 <- GRanges("chr1", IRanges(start = 100:110, width = 5))
## gr3 <- GRanges("chr1", IRanges(start = 200:210, width = 5))
## grl <- GenomicRangesList(gr1, gr2, gr3)


## setGeneric("transformToOrigin", function(data, ...) standardGeneric("transformToOrigin"))
## setMethod("transformToOrigin", "GRanges", function(data, ...){
##   ori <- metadata(gr)$origin
##   if(is.null(ori))
##     ori <- 1
##   values(gr)$.ori <- data
##   metadata(gr)$coord <- "even"
## })

## setMethod("transformToOrigin", "GenomicRangesList", function(data, ...){
  
##   values(gr)$.ori <- data
##   metadata(gr)$coord <- "even"
## })

##' biovizBase is a package which provides utilities and color scheme
##' for higher level graphic package which aim to visualize biological
##' data especially genetic data.
##'
##' This package provides default color scheme for nucleotide, strand,
##' amino acid, try to pass colorblind checking as possible as we can.
##' And also provide giemsa stain result color scheme used to show
##' cytoband. This package also provides utilites to manipulate and
##' summarize raw data to get them ready to be visualized.
##' @name biovizBase-package
##' @aliases biovizBase
##' @docType package
NULL

##' Gene symbols with position
##'
##' This data set provides genen symbols in human with position
##' and starnd information, stored as GRanges object.
##' @name genesymbol
##' @docType data
##' @usage data(genesymbol)
##' @format GRanges: length of 16914 
##' @keywords datasets
##' @examples
##' data(genesymbol)
##' head(genesymbol)
NULL

##' Hg19 ideogram without cytoband information
##'
##' This data set provides hg19 genome information wihout cytoband
##' information.
##' @name hg19Ideogram
##' @docType data
##' @usage data(hg19Ideogram)
##' @format GRanges: length of 24
##' @keywords datasets
##' @examples
##' data(hg19Ideogram)
##' hg19Ideogram
NULL

##' Hg19 ideogram with cytoband information
##'
##' This data set provides hg19 genome information with cytoband
##' information.
##' @name hg19IdeogramCyto
##' @docType data
##' @usage data(hg19IdeogramCyto)
##' @format GRanges: length of 862
##' @keywords datasets
##' @examples
##' data(hg19IdeogramCyto)
##' hg19IdeogramCyto
NULL

##' Subset of RNA editing sites in hg19
##'
##' This data set provides a subset(500 sites only) of hg19 RNA editing sites, and
##' originally from DARNED \url{http://darned.ucc.ie/} for the hg19
##' assembly. 
##' @name darned_hg19_subset500
##' @docType data
##' @usage data(darned_hg19_subset500)
##' @format GRanges: length of 500
##' @keywords datasets
##' @examples
##' data(darned_hg19_subset500)
##' darned_hg19_subset500
NULL

library(biovizBase)
library(GenomicRanges)
## with no 
hg19IdeogramCyto <- getIdeogram("hg19", cytobands = TRUE)
seqinfo(hg19Ideogram)
hg19Ideogram <- getIdeogram("hg19", cytobands = FALSE)
## then for genesymbol
library("org.Hs.eg.db")
ls("package:org.Hs.eg.db")
available.dbschemas()

symbol_id <- mappedRkeys(org.Hs.egSYMBOL)
head(symbol_id)
length(symbol_id)


gene_id <- mget(symbol_id, revmap(org.Hs.egSYMBOL), ifnotfound=NA)
gene_id <- unlist2(gene_id)
gene_id <- gene_id[!is.na(gene_id)]
chr <- toTable(org.Hs.egCHR[gene_id])
chrloc <- toTable(org.Hs.egCHRLOC[gene_id])
chrlocend <- toTable(org.Hs.egCHRLOCEND[gene_id])
identical(chrloc$gene_id, chrlocend$gene_id)
chrs <- chrloc
head(chrlocend)
chrs$end_location <- chrlocend$end_location
head(chrs)
res <- chrs
sum(duplicated(gene_id))# no duplicate
idx <- match(res$gene_id, gene_id)
res$symbol <- names(gene_id[idx])
sum(with(res, start_location > 0 & end_location < 0))# good
sum(with(res, start_location < 0 & end_location > 0))# good
## merge(merge(data.frame(gene_id=gene_id, symbol=names(gene_id)), chrloc),
## chrlocend)
## df <- merge(merge(merge(data.frame(gene_id = gene_id,
##                                    symbol = names(gene_id)), chr), chrloc), chrlocend)
## do I need to subset it? No.
## add extra column to indicate sense or antisense.
##  FOXD4L2,
res$sense <- with(res, start_location>=0&end_location>=0)
head(res)
res.sense <- res[res$sense,]
sum(with(res.sense, start_location > end_location))
res.antisense <- res[!res$sense,]
sum(with(res.antisense, start_location < end_location))
head(res)
## need to make sure you get this right
res$strand <- strand(!res$sense)
head(res)
sum(with(res, start_location == end_location))
## df2 <- subset(df, start_location>=0&end_location>=0)
## strand <- ifelse(df2$start_location>df2$end_location, "-", "+")
## strand[df2$start_location == df2$end_location] <- "*"
## id <- strand == "-"
## tmp <- df2[id,"start_location"]
## df2[id,"start_location"] <- df2[id,"end_location"]
## df2[id,"end_location"] <- tmp
## df2$strand <- strand
## df2 <- df2[!duplicated(df2),]
## for ensembl
ensembl_id <- unlist(mget(gene_id, org.Hs.egENSEMBL))
head(ensembl_id)
idx <- match(res$gene_id, names(ensembl_id))
res$ensembl_id <- ensembl_id[idx]
head(res)
library(GenomicRanges)
genesymbol <- with(res,GRanges(seqnames = paste("chr",Chromosome,sep = ""),
                              IRanges(abs(start_location), abs(end_location)),
                               strand = strand,
                              symbol = symbol,
                               ensembl_id = ensembl_id))
names(genesymbol) <- values(genesymbol)$symbol
save(genesymbol, file = "~/Datas/rdas/genesymbol.rda")
save(genesymbol, file = "~/Codes/gitrepos/biovizBase/data/genesymbol.rda")
save(hg19IdeogramCyto, file = "~/Codes/gitrepos/biovizBase/data/hg19IdeogramCyto.rda")
save(hg19Ideogram, file = "~/Codes/gitrepos/biovizBase/data/hg19Ideogram.rda")

id <- unlist2(mget("FOXD4L2", revmap(org.Hs.egSYMBOL)))
id <- revmap(org.Hs.egSYMBOL)[["FOXD4L2"]]
org.Hs.egCHRLOC[[id]]
org.Hs.egCHRLOCEND[[id]]

## new data generation method, with seqlengths
ideogramCyto <- getIdeogram(c("hg19", "hg18", "mm10", "mm9"))
ideogram <- getIdeogram(c("hg19", "hg18", "mm10", "mm9"), cytoband = FALSE)
ideo <- ideogram
ideoCyto <- ideogramCyto
save(ideo,  file = "../../../../bioc-devel/biovizBase/data/ideo.rda")
save(ideoCyto, file = "/Users/tengfei/Code/svnrepos/bioc-devel/biovizBase/data/ideoCyto.rda")
hg19 <- ideoCyto$hg19
ideoCyto$hg18 <- keepSeqlevels(ideoCyto$hg18, paste0("chr", c(1:22, "X", "Y")))
ideoCyto$hg19 <- keepSeqlevels(ideoCyto$hg19, paste0("chr", c(1:22, "X", "Y")))
ideoCyto$mm10 <- keepSeqlevels(ideoCyto$mm10, paste0("chr", c(1:19, "X", "Y")))
ideoCyto$mm9 <- keepSeqlevels(ideoCyto$mm9, paste0("chr", c(1:19, "X", "Y")))

## for circular view
crc1 <- system.file("extdata", "crc1-missense.csv", package = "biovizBase")
crc1 <- read.csv(crc1)
library(GenomicRanges)
mut.gr <- with(crc1,GRanges(Chromosome, IRanges(Start_position, End_position),
                            strand = Strand))
values(mut.gr) <- subset(crc1, select = -c(Start_position, End_position, Chromosome))
data("hg19Ideogram", package = "biovizBase")
seqs <- seqlengths(hg19Ideogram)
## subset_chr
chr.sub <- paste("chr", 1:22, sep = "")
## levels tweak
seqlevels(mut.gr) <- c(chr.sub, "chrX")
mut.gr <- keepSeqlevels(mut.gr, chr.sub)
seqs.sub <- seqs[chr.sub]
## remove wrong position
bidx <- end(mut.gr) <= seqs.sub[match(as.character(seqnames(mut.gr)),
                                      names(seqs.sub))]
mut.gr <- mut.gr[which(bidx)]
## assign_seqlengths
seqlengths(mut.gr) <- seqs.sub
## reanme to shorter names
new.names <- as.character(1:22)
names(new.names) <- paste("chr", new.names, sep = "")
new.names
mut.gr.new <- renameSeqlevels(mut.gr, new.names)
head(mut.gr.new)

hg19Ideo <- hg19Ideogram
hg19Ideo <- keepSeqlevels(hg19Ideogram, chr.sub)
hg19Ideo <- rena

rearr  <- read.csv(system.file("extdata", "crc-rearrangment.csv", package = "biovizBase"))
## start position
gr1 <- with(rearr, GRanges(chr1, IRanges(pos1, width = 1)))
## end position
gr2 <- with(rearr, GRanges(chr2, IRanges(pos2, width = 1)))
## add extra column
nms <- colnames(rearr)
.extra.nms <- setdiff(nms, c("chr1", "chr2", "pos1", "pos2"))
values(gr1) <- rearr[,.extra.nms]
## remove out-of-limits data
seqs <- as.character(seqnames(gr1))
.mx <- seqlengths(hg19Ideo)[seqs]
idx1 <- start(gr1) > .mx
seqs <- as.character(seqnames(gr2))
.mx <- seqlengths(hg19Ideo)[seqs]
idx2 <- start(gr2) > .mx
idx <- !idx1 & !idx2
gr1 <- gr1[idx]
seqlengths(gr1) <- seqlengths(hg19Ideo)
gr2 <- gr2[idx]
seqlengths(gr2) <- seqlengths(hg19Ideo)

values(gr1)$to.gr <- gr2
## rename to gr
gr <- gr1
values(gr)$rearrangements <- ifelse(as.character(seqnames(gr))
                                    == as.character(seqnames((values(gr)$to.gr))),
                                    "intrachromosomal", "interchromosomal")
crc.gr <- gr
crc.gr
mut.gr <- mut.gr.new
hg19sub <- hg19Ideo
save(crc.gr, mut.gr, hg19sub, file = "/Users/tengfei/Code/svnrepos/bioc-devel/biovizBase/data/CRC.rda")


snp <- read.table("/Users/tengfei/Downloads/plink.assoc.txt", header = TRUE)
N <- 2000
set.seed(123)
snp <- snp[sample(1:nrow(snp), size = N, replace = FALSE),]
rownames(snp) <- NULL
head(snp)
write.table(snp, 
            file = "/Users/tengfei/Code/svnrepos/bioc-devel/biovizBase/inst/extdata/plink.assoc.sub.txt", row.names = FALSE)
## https://github.com/stephenturner/qqman

setGeneric("addStepping",function(obj,...) standardGeneric("addStepping"))

setGeneric("getFragLength", function(obj, ...) standardGeneric("getFragLength"))

setGeneric("shrinkageFun", function(obj,...) standardGeneric("shrinkageFun"))

setGeneric("maxGap", function(obj, ...) standardGeneric("maxGap"))

setGeneric("spliceSummary", function(obj, model, ...) standardGeneric("spliceSummary"))

### =========================================================================
### Coverage utilities
### -------------------------------------------------------------------------

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Estimation of coverage
###

setGeneric("estimateCoverage",
           function(x, ...) standardGeneric("estimateCoverage"))

setMethod("estimateCoverage", "BamFile", function(x, maxBinSize = 2^14) {
  ## bai_path <- paste(index(x), ".bai", sep = "")
  bai_path <- index(x)
  bai <- file(bai_path, "rb")
  on.exit(close(bai))
  bytes <- readBin(bai, "raw", n = file.info(bai_path)[,"size"])
  bam_si <- seqinfo(x)

  bin_offsets <- .Call(scan_bam_bin_offsets, bytes)
  names(bin_offsets) <- seqlevels(bam_si)

  ## CONSTANTS
  UBSIZE <- 2^16 # hopefully is not going to change
  CBSIZE <- 19100 # determined empirically
  BINSIZES <- 2 ^ seq(29, 14, -3)
  TOTALSIZE <- 2 ^ 29
  NBIN <- sum(TOTALSIZE / BINSIZES)
  METABIN <- NBIN + 1

  makeBinRanges <- function() {
    unlist(List(lapply(BINSIZES[BINSIZES <= maxBinSize], function(x) {
      chunks <- breakInChunks(TOTALSIZE, chunksize=x)
      IRanges(start(chunks), end(chunks))
    })))
  }

  bin_ranges <- makeBinRanges()

  bin_offsets <- lapply(bin_offsets, function(x) {
    x[,x[1,] != METABIN] # drop the samtools metadata bin
  })

  off_scores <- lapply(seqlevels(bam_si), function(sn) {
    off <- t(bin_offsets[[sn]])
    colnames(off) <- c("bin", "start.coffset", "end.coffset",
                       "start.uoffset", "end.uoffset")
    coffdiff <- off[,"end.coffset"] - off[,"start.coffset"]
    off_diff <- ifelse(coffdiff > 0,
                       UBSIZE - off[,"start.uoffset"] +
                       UBSIZE * pmax(0, round(coffdiff / CBSIZE - 1)) +
                       off[,"end.uoffset"],
                       off[,"end.uoffset"] - off[,"start.uoffset"])
    ## sum bins with multiple chunks
    off_diff <- rowsum(as.numeric(off_diff), off[,"bin"])
    score <- numeric(NBIN)
    score[as.integer(rownames(off_diff)) + 1L] <- as.vector(off_diff)
    score[rep(BINSIZES, TOTALSIZE / BINSIZES) <= maxBinSize]
  })

  ## for performance, important to construct GRanges outside of loop
  sn_bin <- rep(seqlevels(bam_si), each = length(bin_ranges))
  bin_ranges <- restrict(rep(bin_ranges, length(bam_si)),
                         end = seqlengths(bam_si)[sn_bin],
                         keep.all.ranges = TRUE)
  gr <- GRanges(sn_bin, bin_ranges, score = unlist(off_scores),
                seqlengths = seqlengths(bam_si))
  gr[width(gr) > 0]
})

setMethod("shrinkageFun", "GenomicRanges", function(obj, max.gap = 1L){
  obj <- obj[order(as.numeric(end(obj)))]
  shrinkage <- pmax(0, width(obj) - max.gap)
  shrinksum <- c(0L, cumsum(shrinkage))
  function(x) {
    res <- shift(x, -shrinksum[findInterval(start(x), end(obj)) + 1L])
    metadata(res)$coord <- "truncate_gaps"
    metadata(res)$max.gap <- max.gap
    metadata(res)$gaps <- obj
    values(res)$.ori <- x
    res
  }  
})

setMethod("shrinkageFun", "IRanges", function(obj, max.gap = 1L){
  obj <- obj[order(as.numeric(end(obj)))]
  shrinkage <- pmax(0, width(obj) - max.gap)
  shrinksum <- c(0L, cumsum(shrinkage))
  function(x) {
    res <- shift(x, -shrinksum[findInterval(start(x), end(obj)) + 1L])
    metadata(res)$coord <- "truncate_gaps"    
    metadata(res)$max.gap <- max.gap
    metadata(res)$gaps <- obj    
    values(res)$.ori <- x    
    res
  }  
})

is_coord_truncate_gaps<- function(obj){
  if(!is.null(metadata(obj)$coord))
    res <- metadata(obj)$coord == "truncate_gaps"
  else
    res <- FALSE
  res
}

parseArgsForAes <- function(args){
    res <- if ("mapping" %in% names(args)) {
               args[["mapping"]]
           } else {
               Find(function(x) {
                   is(x, "uneval")
               }, args)
           }

    if (is.null(res))
        res <- structure(list(), class="uneval")
    
  idx <- ggplot2_is_calculated_aes(res)
  res[idx] <- ggplot2_strip_dots(res[idx])
  res
}

parseArgsForNonAes <- function(args){
    Filter(function(x) !is(eval(x, parent.frame()), "uneval"), args)
}

## to reduce burden to import too much packages
## I redfined/move functions in some other packages to here
.all_aesthetics <- c("adj", "alpha", "angle", "bg", "cex", "col",  "color", "colour", "fg",
                     "fill", "group", "hjust",  "label", "linetype", "lower", "lty",
                     "lwd",  "max", "middle", "min", "order", "pch", "radius", "sample",
                     "shape" , "size",  "srt",  "upper", "vjust", "weight", "width",
                     "x" ,  "xend", "xmax", "xmin", "xintercept",  "y",
                     "yend", "ymax", "ymin", "yintercept",  "z")

ggplot2_is.constant <- function(x){
    sapply(x, function(x) "I" %in% all.names(asOneSidedFormula(x)))
}

## for ggplot2
ggplot2_is_calculated_aes <- function(aesthetics){
  match <- "\\.\\.([a-zA-z._]+)\\.\\."
  stats <- rep(FALSE, length(aesthetics))
  grepl(match, sapply(aesthetics, quo_text))
}


ggplot2_strip_dots <- function(aesthetics){
  match <- "\\.\\.([a-zA-z._]+)\\.\\."
  strings <- lapply(aesthetics, quo_text)
  strings <- lapply(strings, gsub, pattern = match, replacement = "\\1")
  lapply(strings, function(x) parse(text = x)[[1]])
}

## Simple test function to test the crunch method on EnsDb
library(EnsDb.Hsapiens.v75)
edb <- EnsDb.Hsapiens.v75

test_crunch_ensdb <- function(){
    zbtb <- genes(edb, filter = ~ genename == "ZBTB16")

    Zens <- crunch(edb, zbtb)
    checkTrue(all(Zens$gene_name == "ZBTB16"))
    ## Expect a single gene.
    checkEquals(length(unique(Zens$gene_id)), 1)
    ## crunch without which but using a filter.
    Test <- crunch(edb, ~ genename == "ZBTB16")
    checkEquals(Zens, Test)

    ## Testing exceptions:
    checkException(crunch(edb, which = "bla"))

    strand(zbtb) <- "*"
    ## Expect two genes.
    Test <- crunch(edb, zbtb)
    checkEquals(length(unique(Test$gene_id)), 3)

    ## Testing columns
    wantCols <- c("gene_biotype", "tx_biotype")
    Test <- crunch(edb, zbtb, columns = wantCols)
    checkTrue(all(wantCols %in% colnames(mcols(Test))))

    ## Now let's try a tricky one...
    Test <- crunch(edb, which = GenenameFilter("ALDOA"))
    gr <- GRanges(seqnames = 16, IRanges(30768000, 30770000), strand = "*")
    Test <- crunch(edb, which = gr)
}

%\VignetteIndexEntry{An Introduction to biovizBase}
%\VignetteDepends{}
%\VignetteKeywords{visualization utilities}
%\VignettePackage{biovizBase}
\documentclass[10pt]{article}
% \SweaveOpts{width = 5, height = 4.5}
% \usepackage{times}
\usepackage{hyperref}
\usepackage{verbatim}

\textwidth=6.5in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=-.1in
\evensidemargin=-.1in
\headheight=-.3in

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}
\newcommand{\Rfunarg}[1]{{\texttt{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}
\newcommand{\Rcode}[1]{{\texttt{#1}}}

\newcommand{\software}[1]{\textsf{#1}}
\newcommand{\R}{\software{R}}
\newcommand{\IRanges}{\Rpackage{IRanges}}
\newcommand{\biovizBase}{\Rpackage{biovizBase}}
\newcommand{\ggbio}{\Rpackage{ggbio}}
\newcommand{\visnab}{\Rpackage{visnab}}

\title{An Introduction to \biovizBase{}}
\author{Tengfei Yin, Michael Lawrence, Dianne Cook}
\date{\today}

\begin{document}
\maketitle
\newpage
\tableofcontents
\newpage

<<options,echo=FALSE>>=
options(width=72)
@

\section{Introduction}

The \biovizBase{} package is designed to provide a set of utilities and
color schemes serving as the basis for visualizing biological data,
especially genomic data. Two other packages are currently built on this
package, a static version of graphics is provided by the package \ggbio{},
and an interactive version of graphics is provided by \visnab{}(Currently not released).

In this vignette, we will introduce those color schemes and different
utilities functions using simple examples and data sets. Utilities
includes functions that precess the raw data, validate names,
add attributes, and generate summaries such as fragment length,
GC content, and mismatch information.

\section{Color Schemes}
The \biovizBase{} package aims to provide a set of default color schemes
for biological data, based on the following principles.
\begin{itemize}
\item Make biological sense.  Data is displayed in a way that is
  similar to observed results under the microscope. (Example: giemsa
  stain results)
\item Generate aesthetically pleasing colors based on well-defined
  color sets like \emph{color brewer}
  \footnote{\url{http://colorbrewer2.org/}}. Produce the appropriate
  color for \emph{sequential, diverging, and qualitative} color
  schemes.
\item Accommodate colorblind vision by creating color pallets that
  pass the color blind check on the \emph{Vischeck} website
  \footnote{\url{http://www.vischeck.com/}} or use palette from
  package \Rpackage{dichromat} or use color-blind safe color palette
  checked by \emph{ColorBrewer}
  website\footnote{\url{http://colorbrewer2.org/}}. There are three
  types of colorblind checking strategy defined on these website.
  \begin{description}
     \item[Deuteranope] a form of red/green color deficit; 
     \item[Protanope] another form of red/green color deficit;
     \item[Tritanope] a blue/yellow deficit- very rare.
  \end{description}
\end{itemize}

Our color scheme try to pass color-blind checking points to make sure
all the users can tell the difference between groups of data
displayed.  To make the implementation easy, we most time just use
\Rpackage{dichromat} to check this, \Rpackage{dichromat} collapses
red-green color distinctions to approximate the effect of the two
common forms of red/green color blindness, protanopia and
deuteranopia.  Or we could simply implement proved color-blind safe
palette from \Rpackage{dichromat} or \Rpackage{RColorBrewer}.

All color schemes have a general color generating function and a
default color generating function.  They are automatically stored in
\Rfunction{options} as default when loading the package. Other
packages built on \biovizBase{} can use the default color scheme,
ensuring consistent color themes across all static and interactive
graphics.  Users may also change the default color in the
\Rfunction{options} to personalize the global color scheme to fit
their needs.


@ 
<<color-scheme-option>>=
library(biovizBase)
## library(scales)

@ %def 

\subsection{Colorblind Safe Palette}
For graphics, it's important to make sure most people can tell the
difference between colors on the plots, even for people with deficient
or anomalous red-green vision. 

We will add more and more colorblind safe palette gradually, now we
only supported palettes from two packages, \Rpackage{dichromat} or
\Rpackage{RColorBrewer}. However, \Rpackage{RColorBrewer} doesn't
provide information about colorblind palette. So we need to check
manually on \textit{ColorBrewer} website, and add this information
with the palette information. For \Rpackage{dichromat} package, it
doesn't have a palette information like \Rcode{brewer.pal.info}, which
contains three different types, \textbf{qual, div, seq} representing
quality, divergent and sequential respectively, and also missing max
colors information, so we integrate all these information and generate
three palette information.

\begin{itemize}
\item \Robject{brewer.pal.blind.info} provides only colorblind safe
  palette subset.
\item \Robject{dichromat.pal.blind.info} provides colorblind safe
  palette with category information and max color allowed.
\item \Robject{blind.pal.info} integrate first two, provides a general
  palette information with extra column like pal.id, which used for
  function \Rfunction{colorBlindSafePal} as index for arguments
  \Rfunarg{palette} or maxcolors for allowed number of
  color. \textit{pkg} providing information about which package it is
  defined.
\end{itemize}
@ 
<<color-pal-info>>=
head(blind.pal.info)
@ %def 

Then we defined a color generating function
\Rfunction{colorBlindSafePal}, this function reading in a palette
argument which could be a index number or names for palette defined in
\Robject{blind.pal.info}. And return a color generating function, a
\Rfunarg{repeatable} argument will control, for number over max color
numbers required, does it simply repeat it or just providing limited
number of colors.

@ 
<<colorBlindPal>>=
## with no arguments, return blind.pal.info
head(colorBlindSafePal())
## 
mypalFun <- colorBlindSafePal("Set2")
## mypalFun(12, repeatable = FALSE) #only three
mypalFun(11, repeatable = TRUE)  #repeat
@ %def 

To Collapses red-green color distinctions to approximate the effect of
the two common forms of red- green color blindness, protanopia and
deuteranopia, we can use function \Rfunction{dichromat} from package
\Rpackage{dichromat}, this save us the time to 

\begin{figure}[h!t!p!b]
  \centering
@ 
<<color-dichromat, fig = TRUE>>=
## for palette "Paried"
mypalFun <- colorBlindSafePal(21)   
par(mfrow = c(1, 3))
showColor(mypalFun(4))
library(dichromat)
showColor(dichromat(mypalFun(4), "deutan"))
showColor(dichromat(mypalFun(4), "protan"))
@ %def   
\caption{Checking colors with two common type of color blindness. The
  first one is normal perception, second one for deuteranopia and last
  one for protanopia. Since we are using selected color palettes in
  this package, it should be fine with those types of blindness.}
  \label{fig:dichromat}
\end{figure}

We only show this as an examples and won't compare all other color
schemes in the following sections. Please notice that
\begin{itemize}
\item If the categorical data contains many levels like amino acid,
  people cannot easily tell the difference anyway, we did the trick to
  simply repeat the colors. This might be useful for many other cases
  like grand linear view for chromosomes, since if the viewed orders of
  chromosomes is fixed it's OK to use repeated colors since they are
  not going to be layout as neighbors anyway.
\item For schemes like cytobands, we try to follow the biological
  sense, in this case, we don't really check the color blindness.
\end{itemize}



\subsection{Cytobands Color}
Chemically staining the metaphase chromosomes results in a alternating
dark and light banding pattern, which could provide information about
abnormalities for chromosomes. Cytogenetic bands could also provide
potential predictions of chromosomal structural characteristics, such
as repeat structure content, CpG island density, gene density, and GC
content.

\biovizBase{} package provides utilities to get ideograms from the UCSC
genome browser, as a wrapper around some functionality from
\Rpackage{rtracklayer}. It gets the table for \emph{cytoBand} and stores
the table for certain species as a \Robject{GRanges} object.

We found a color setting scheme in package \Rpackage{geneplotter},
and we implemented it in biovisBase.

The function .cytobandColor will return a default color set. You could
also get it from \Rfunction{options} after you load \biovizBase{}
package.

And we recommended function \Rfunction{getBioColor} to get the color
vector you want, and names of the color is biological categorical
data. This function hides interval color genenerators and also the
complexity of getting color from options. You could specify whether
you want to get colors by default or from options, in this way, you
can temporarily edit colors in options and could change or all the
graphics. This give graphics a uniform color scheme.

@ 
<<cytoband-color>>=
getOption("biovizBase")$cytobandColor
getBioColor("CYTOBAND")
## differece source from default or options.
opts <- getOption("biovizBase")
opts$DNABasesNColor[1] <- "red"
options(biovizBase = opts)
## get from option(default)
getBioColor("DNA_BASES_N")
## get default fixed color
getBioColor("DNA_BASES_N", source = "default")
seqs <- c("A", "C", "T", "G", "G", "G", "C")
## get colors for a sequence.
getBioColor("DNA_BASES_N")[seqs]
@ %def 

You can check the color scheme by calling the
\Rfunction{plotColorLegend} function.  or the \Rfunction{showColor}.


\begin{figure}[h!t!p]
  \centering
@ 
<<cytoband-color-legend, fig = TRUE>>=
cols <- getBioColor("CYTOBAND")
plotColorLegend(cols, title  = "cytoband")
@ %def 
  \caption{Legend for cytoband color}
  \label{fig:cytoband}
\end{figure}


\subsection{Strand Color}
In the \Robject{GRanges} object, we have \Robject{strand} which
contains three levels, \textbf{+, -, *}. We are using a qualitative
color set from \emph{Color Brewer} and check with \Rpackage{dichromat}
as Figure\ref{fig:strand} shows, and we can see that this color set
passes all three types of colorblind test. Therefore it should be a
safe color set to use to color strand.

\begin{figure}[h!t!p]
  \centering
  @ 
<<strand-color, fig = TRUE>>=
par(mfrow = c(1, 3))
cols <- getBioColor("STRAND")
showColor(cols)
showColor(dichromat(cols, "deutan"))
showColor(dichromat(cols, "protan"))
@ %def 
  \caption{Colorblind vision check for color of strand}
  \label{fig:strand}
\end{figure}

\subsection{Nucleotides Color}
We start with the five most used nucleotides, \textbf{A,T,C,G,N}, most
genome browsers have their own color scheme to represent nucleotides, We
chose our color scheme based on the principles introduced
above. Since in genetics, \emph{GC-content} usually has special
biological significance because GC pair is bound by three hydrogen
bonds instead of two like AT pairs. So it has higher thermostability
which could result in different significance, like higher annealing
temperature in PCR. So we hope to choose warm colors for \textbf{G,C}
and cold colors for \textbf{A,T}, and a color in between to represent
\textbf{N}.  They are chosen from a diverging color set of \emph{color
  brewer}. So we should be able to easily tell the GC enriched region.
Figure \ref{fig:ne} shows the results from \Rpackage{dichromat},
and we can see this color set passes all two types of the colorblind
test. It should be a safe color set to use to color the five most used
nucleotides.

@ 
<<base-color>>=
getBioColor("DNA_BASES_N")

@ %def 

\begin{figure}[h!t!p]
  \centering
@ 
<<ne-dichromat, fig = TRUE>>=
par(mfrow = c(1, 3))
cols <- getBioColor("DNA_BASES_N", "default")
showColor(cols, "name")
cols.deu <- dichromat(cols, "deutan")
names(cols.deu) <- names(cols)
cols.pro <- dichromat(cols, "protan")
names(cols.pro) <- names(cols)
showColor(cols.deu, "name")
showColor(cols.pro, "name")
@ %def 

  \caption{Colorblind vision check for color of nucleotide}
  \label{fig:ne}
\end{figure}

\subsection{Amino Acid Color and Other Schemes}
We also include some other color schemes created based on existing
object in package \Rpackage{Biostrings} and other customized color
scheme.  Please notice that the object name is not the same as the
name in the options.  On the left of \textbf{=}, it's name of object,
most of them are defined in \Rpackage{Biostrings} and on the right,
it's the name in options.

\begin{verbatim}
DNA_BASES_N = "DNABasesNColor"
DNA_BASES = "DNABasesColor"
DNA_ALPHABET = "DNAAlphabetColor"
RNA_BASES_N = "RNABasesNColor"
RNA_BASES = "RNABasesColor"
RNA_ALPHABET = "RNAAlphabetColor"
IUPAC_CODE_MAP = "IUPACCodeMapColor"
AMINO_ACID_CODE = "AminoAcidCodeColor"                     
AA_ALPHABET = "AAAlphabetColor"
STRAND = "strandColor"
CYTOBAND = "cytobandColor"
\end{verbatim}

They all could be retrieved by calling function
\Rfunction{getBioColor}.


\subsection{Future Schemes}
Current color schemes are most generated based on known object in
\R{}, which has a clear definition and classification. But we do have
more interesting events or biological significance need to be color
coded. Like most genome browser, they try to color code many events,
for instance, color the insertion size which is larger/smaller than
the estimated size; for paired RNA-seq data, we may color the paired
reads mapped to a different chromosome. 

We may include more color coded events in this package in next
release.

\section{Utilities}
\biovizBase{} serves as a basis for the visualization of biological data,
especially for genomic data. \Rpackage{IRanges} and
\Rpackage{GenomicRanges} are the two most important infrastructure
packages to manipulate genomic data. They already have lots of useful
and fast utilities for processing genomic data. Some other package
such as \Rpackage{rtracklayer}, \Rpackage{Rsamtools},
\Rpackage{ShortRead}, \Rpackage{GenomicFeatures} provide common I/O
for certain types of biological data and utilities for processing
those raw data. Most of our utilities to be introduced in this section
only manipulate the data in a simple and different way to get them ready
for visualization. Most cases are only useful for visualization work,
like adding brush color attributes to a \Robject{GRanges} object. Some
of the other utilities are responsible for summarizing certain types of
raw data, getting it ready to be visualized. Some of those utilities may be moved to
a separate package later.

\subsection{GRanges Related Manipulation}
\biovizBase{} mainly focuses on visualizing the genomic data, so we
have some utilities for manipulating \Robject{GRanges} object. We are
going to introduce these functions in the flow wing sub-sections.
Overall, we hope to reduce people's work through these common
utilities.

% \subsubsection{Chromosome Name Manipulation}
% We don't have required canonical chromosome name for
% \Robject{seqnames} in \Robject{GRanges}. Most times, we have chromosome
% name with prefix \textbf{chr} in lower case. However, we could come
% across all kinds of other names with prefix \textbf{Chr}, or
% without any prefix at all. This complicates our integration of data
% since we need to check with \Robject{seqnames} before we are able to aggregate
% them or before we can visualize them. A consistent name scheme is very
% important in most graphic packages like \Rpackage{ggplot2}.  For
% example, you are going to fail if you try to overlay two data in
% the same plot which follows a different name schema. Sometimes we
% also need to layout our graphics for genomes in a nice ordered way,
% which requires some sorting/ordering function to re-order our
% \Robject{GRanges} object.


% Function \Rfunction{sortChr} works for \Robject{GenomicRanges}, or
% characters or factor, sort it based on seqnames.  The default is first
% showing numeric seqnames in increasing order, then showing mixed
% letters names in alphabetical. Function \Rfunction{orderChr} return
% an index instead of a \Robject{GenomicRanges} object. The user could also
% provide a model which is a vector of seqnames to sort by.
% @ 
% <<sort-chr>>=
% sort(gr)
% head(sortChr(gr, prefix = "chr"))
% head(orderChr(gr, prefix = "chr"))
% head(sortChr(gr, model = c("chrX", "chr3", "chr1", "chrY", "chr2")))
% sortChr(c("chrX", "chr3", "chr1", "chrY", "chr2"))
% orderChr(c("chrX", "chr3", "chr1", "chrY", "chr2"))
% sortChr(factor(c("chrX", "chr3", "chr1", "chrY", "chr2")))
% orderChr(factor(c("chrX", "chr3", "chr1", "chrY", "chr2")))
% @ %def 


\subsubsection{Adding Disjoint Levels}
@ 
<<sample-gr>>=
library(GenomicRanges)
set.seed(1)
N <- 500
gr <- GRanges(seqnames =
              sample(c("chr1", "chr2", "chr3", "chrX", "chrY"),
                     size = N, replace = TRUE),
              IRanges(
                      start = sample(1:300, size = N, replace = TRUE),
                      width = sample(70:75, size = N,replace = TRUE)),
              strand = sample(c("+", "-", "*"), size = N,
                replace = TRUE),
              value = rnorm(N, 10, 3), score = rnorm(N, 100, 30),
              group = sample(c("Normal", "Tumor"),
                size = N, replace = TRUE),
              pair = sample(letters, size = N,
                replace = TRUE))
@ %def 

This is a tricky question.  For example, for pair-end RNA-seq data, we
may want to put the reads with the same \emph{qname} on the same
level, with nothing falling in between. For better visualization of
the data, we may hope that adding invisible extensions to the reads
will prevent closely neighbored reads from showing up on the same
level.

\Rfunction{addStepping} function takes a
\Robject{GenomicRanges} object and will add an extra column
called \textbf{.levels} to the object.  This function is essentially a
wrapper around a function \Rfunction{disjointBins} but allows a more
flexible way to assign levels to each entry. For example, if the
arguments \Rfunarg{group.name} is specified to one of the column in
elementMetadata, the function will make sure
\begin{itemize}
\item Grouped intervals are in the same levels( if they are not
  overlapped each other).
\item No entry is following between the grouped intervals.
\item If extend.size is provided, it buffers the intervals and then
  computes the disjoint levels, thus ensuring that two closely positioned
  intervals will be assigned to different levels, a good practice for
  visualization.
\end{itemize}

For now, this function is only useful for visualization purposes.

@ 
<<addStepping-gr>>=
head(addStepping(gr))
head(addStepping(gr, group.name = "pair"))
gr.close <- GRanges(c("chr1", "chr1"), IRanges(c(10, 20), width = 9))
addStepping(gr.close)
addStepping(gr.close, extend.size = 5)
@ %def 

\subsection{Shrink the Gaps}
Sometime, in a gene centric view, we hope to truncate or shrink the
gaps to better visualize the short reads or annotation data. It's
\textbf{DANGEROUS} to shrink the gaps, since it only make sense in
visualization. And even in the visualization the x-scale will be
discontinued, and labels became somehow meaningless. \textbf{Make
  sure} you are not using the shrunk version of data when performing
the down stream analysis.

This is a tricky question too, we hope to provide a flexible way to
shrink the gaps.  When we have multiple tracks, users would be 
responsible to shrink all the tracks based on the common gaps,
otherwise there will be mis-aligned tracks.

\Rfunction{maxGap} computes a suitable estimated gap based on passed
\Robject{GenomicRanges}
@ 
<<maxGap>>=
gr.temp <- GRanges("chr1", IRanges(start = c(100, 250),
                                 end = c(200, 300)))
maxGap(gaps(gr.temp, start = min(start(gr.temp))))
maxGap(gaps(gr.temp, start = min(start(gr.temp))), ratio = 0.5)
@ %def 

\Rfunction{shrinkageFun} function will read in a
\Robject{GenomicRanges} object which represents the gaps, and returns
a function which alters a different \Robject{GenomicRanges} object, to
shrink that object based on previously specified gaps shrinking
information.  You could use this function to treat multiple
tracks(e.g. \Robject{GRanges}) to make sure they are shrunk based on
the common gaps and the same ratio.

Be careful in the following situations.
\begin{itemize}
\item When use the same shrinkage function to shrink multiple tracks,
  make sure the gaps passed to \Rfunction{shrinkageFun} function is
  the common gaps across all tracks, otherwise, it doesn't make sense
  to cut a overlapped gap within one of the tracks.
\item The default max gap is not 0, just for visualization purpose.  If
  for estimation purpose, you might want to make sure you cut all
  the gaps.
\end{itemize}

And notice, after shrinking, the x-axis labes only provide approximate
position as shown in Figure \ref{fig:shrink-single} and
\ref{fig:shrink-two}, because it's clipped. It's just for
visualization purpose.
@ 
<<shrink-single>>=
gr1 <- GRanges("chr1", IRanges(start = c(100, 300, 600),
                               end = c(200, 400, 800)))
shrink.fun1 <- shrinkageFun(gaps(gr1), max.gap = maxGap(gaps(gr1), 0.15))
shrink.fun2 <- shrinkageFun(gaps(gr1), max.gap = 0)
head(shrink.fun1(gr1))
head(shrink.fun2(gr1))
@ %def   
\begin{figure}[h!t!p!b]
  \centering
  \includegraphics[width = 0.8\textwidth]{intro-shrink-single.pdf}
\caption{Shrink single GRanges. The first track is original GRanges,
  the second one use a ratio which shrink the GRanges a little bit, and
  default is to remove all gaps shown as the third track }
  \label{fig:shrink-single}
\end{figure}


@ 
<<shrinkageFun>>=
gr2 <- GRanges("chr1", IRanges(start = c(100, 350, 550),
                               end = c(220, 500, 900)))
gaps.gr <- intersect(gaps(gr1, start = min(start(gr1))),
                     gaps(gr2, start = min(start(gr2))))
shrink.fun <- shrinkageFun(gaps.gr, max.gap = maxGap(gaps.gr))
head(shrink.fun(gr1))
head(shrink.fun(gr2))
@ %def   

\begin{figure}[h!t!p!b]
  \centering
\includegraphics[width = 0.8\textwidth]{intro-shrinkageFun.pdf}
\caption{shrinkageFun demonstration for multiple GRanges, the top two
  tracks are the original tracks, please note how we clipped common
  gaps for those two tracks and shown as bottom two tracks.}
  \label{fig:shrink-two}
\end{figure}


\subsection{GC content}
As mentioned before, GC content is an interesting variable which may
be related to various biological questions. So we need a way to
compute GC content in a certain region of a reference genome.

\Rfunction{GCcontent} function is a wrapper around \Rfunction{getSeq}
function in \Rpackage{BSgenome} package and
\Rfunction{letterFrequency} in \Rpackage{Biostrings} package. It reads
a \Robject{BSgenome} object and returns count/probability for
\textbf{GC} content in specified region.

@ 
<<gc-content, eval = FALSE>>=
library(BSgenome.Hsapiens.UCSC.hg19)
GCcontent(Hsapiens, GRanges("chr1", IRanges(1e6, 1e6 + 1000)))
GCcontent(Hsapiens, GRanges("chr1", IRanges(1e6, 1e6 + 1000)), view.width = 300)
@ %def 


\subsection{Mismatch Summary}
Compared to short-read alignment visualization, it's more useful to
just show the summary of nucleotides of short reads per base and
compare with the reference genome.  We need a way to show the
mismatched nucleotides, coverage at each position and proportion of
mismatched nucleotides, and use the default color to indicate the type
of nucleotide.

\Rfunction{pileupAsGRanges} function summarizes reads from bam files
for nucleotides on single base units in a given region, which allows
the downstream mismatch summary analysis. It's a wrapper around
\Rfunction{applyPileup} function in Rsamtools package and more
detailed control could be found under manual of ApplyPileupsParam function
in Rsamtools. \Rfunction{pileupAsGRanges} function returns a GRanges
object which includes a summary of nucleotides, depth, and bam file
path. This object could be read directly into the
\Rfunction{pileupGRangesAsVariantTable} function for a mismatch
summary.

This function returns a \Robject{GRanges} object with extra
\Robject{elementMetadata}, counts for \textbf{A,C,T,G,N} and
\textbf{depth} for coverage.  \textbf{bam} indicates the bam file
path. Each row is single base unit.

\Rfunction{pileupGRangesAsVariantTable} performs comparisons to the
reference genome(a \Robject{BSgenome} object) and computes the
mismatch summary for a certain region of reads.  User need to make
sure to pass the right reference genome to this function to get the
right summary. This function drops the positions that have no reads
and only keeps the regions with coverage in the summary. The result
could be used to show stacked barchart for the mismatch summary.

This function returns a \Robject{GRanges} with the following
elementMetadata information. 

\begin{description}
\item[ref] Reference base.
\item[read] Sequenced read at that position. Each type of
  \textbf{A,C,T,G,N} summarize counts at one position, if no counts
  detected, will not show it.
\item[count] Count for each nucleotide.
\item[depth] Coverage at that position.
\item[match] A logical value, indicate it's matched or not.
\item[bam] Indicate bam file path.
\end{description}

Sample raw data is from SRA(Short Read Archive), Accession: SRR027894
and subset the gene at chr10:6118023-6137427, which within gene
RBM17. contains junction reads.

@ 
<<PileupAsGRanges, eval = FALSE>>=
library(Rsamtools)
data(genesymbol)
library(BSgenome.Hsapiens.UCSC.hg19)    
bamfile <- system.file("extdata", "SRR027894subRBM17.bam", package="biovizBase")
test <- pileupAsGRanges(bamfile, region = genesymbol["RBM17"])
test.match <- pileupGRangesAsVariantTable(test, Hsapiens)
head(test[,-7])
head(test.match[,-5])
@ %def 

% \subsection{Fragment Length Estimation}
% \emph{Fragment length} could be defined in different
% ways. Here, the \Rfunction{getFragLength} function simply uses the shrinkage
% function (introduced in previous section) to cut all gaps existing in the
% provided model and recompute the length for \Robject{GAlignments}
% object instead of using its width directly. 

% @ 
% <<getFragLength, eval = FALSE>>=
% bamfile <- system.file("extdata", "SRR027894subRBM17.bam", package="biovizBase")
% library(TxDb.Hsapiens.UCSC.hg19.knownGene)
% txdb <- Hsapiens_UCSC_hg19_knownGene_TxDb
% exons <- exonsBy(txdb, by = "tx")
% model <- subsetByOverlaps(exons, genesymbol["RBM17"])
% ## model <- exonsByOverlaps(txdb, genesymbol[16910])
% frg <- getFragLength(file, model)
% @ %def 

% \subsection{Splicing Summary}
% One of the information we could find from the RNA-seq data is the
% abundance of different alternative splicing models. There are two ways
% to visualizing the slicing summary, one is summary for full model,
% which could be represented as \Robject{GRangesList}, for instance, a
% \Robject{GRangesList} grouped by transcirpt id, and each element is
% \Robject{GRanges} object contains all the exons. Another one is single
% model which could be represented as \Robject{GRanges}, for instance, a
% \Robject{GRanges} object which just contains all the exons not grouped
% by transcript. So we need a way to compute counts support each model
% in the full model or splicing junction in the single model.

% There are different ways to summarize abundance based on RNA-seq, I
% started with a simple approach which just counting reads acrossing the
% junction. The algorithm will first filter out reads which are not
% junction reads from the raw data, then mapping junction reads to
% models.
% \begin{itemize}
% \item For full model, if one junction reads mapped to one model which
%   covers two consecutive exons, then this model plus one for
%   supporting evidence. When \Rfunarg{weighted} turned on to TRUE, if a
%   junction reads mapped to multiple model, we will simply plus 1/cases
%   for all the supported model. 
% \item For single model, we just summarize the splicing of all junction
%   reads for all the exons. Using notion like \textbf{1-2, 3-4} to
%   indicate the junction, here the number means the index of given
%   model, we assume each row in the model is one exon. When
%   \Rfunarg{model\_id} set to certain column, for example,
%   'exon\_id'. Then we use exon\_id for our notion of our summary.
% \end{itemize}

% The \Rfunction{spliceSummary} is a S4 method which dispatch on the first
% arguments, and will return a vector which is total counts for each
% model or exon junctions. And names of the vector is transcript name or
% notion like \textbf{1-3} to indicate the isoform.

% \textbf{Note:} Currently we don't support new model generation, which
% means if junction reads cannot mapped to any known model or exons
% which passed to argument \Rfunarg{model} of \Rfunction{spliceSummary}
% function, we don't give summary about these reads. But in the future,
% we would support more flexible summary for this case.

% @ 
% <<spliceSummary, eval = FALSE>>=
% splice.sum1 <- spliceSummary(bamfile, model)
% splice.sum2 <- spliceSummary(bamfile, model, weighted = FALSE)
% exonsall <- exons(txdb)
% model.single <- subsetByOverlaps(exonsall, genesymbol["RBM17"])
% splice.sum3 <- spliceSummary(bamfile, model.single)
% splice.sum4 <- spliceSummary(bamfile, model.single, model_id = "exon_id")
% @ %def 
% \begin{verbatim}
% > splice.sum1
% 38191 38192 38193 
%    57    57     0 
% > splice.sum2
% 38191 38192 38193 
%   114   114     0 
% > splice.sum3
% 1-2 1-3 
%   1 114 
% > splice.sum4
% 140875-140876 140875-140887 
%           114             1 
% \end{verbatim}

% \subsection{Aggregating from Raw Data}
% Those utilities used inside this package internally, useful
% but not exported to users until later, may be dropped or
% changed to more general existing method defined in another package, like
% package \Rpackage{stream}.

\subsection{Get an Ideogram}
\Rfunction{getIdeogram} function is a wrapper of some functionality
from \Rpackage{rtracklayer} to get certain table like cytoBand.  A
full table schema can be found
\href{http://genome.ucsc.edu/cgi-bin/hgTables}{here} at \emph{UCSC
  genome browser}. Please click \emph{describe table schema}.

This function requires a network connection and will parse the data on
the fly.  The first argument of \Rfunction{getIdeogram} is
\Rfunarg{species}.  If \Rcode{missing}, the function will give you a
choice hint, so you will not have to remember the name for the
database you want, or you can simply get the database name for a
different genome using the \Rfunction{ucscGenomes} function in
\Rpackage{Rtracklayer}. The second argument \Rfunarg{subchr} is used
to subset the result by chromosome name. The third argument cytoband
controls if you want to get the gieStain information/band information
or not, which is useful for the visualization of the whole genome or
single chromosome.  You can see some examples in \Rpackage{ggbio}.

@ 
<<ucsc-genome, eval = FALSE>>=
library(rtracklayer)
hg19IdeogramCyto <- getIdeogram("hg19", cytoband = TRUE)
hg19Ideogram <- getIdeogram("hg19", cytoband = FALSE)
unknowIdeogram <- getIdeogram()
@ %def 
\begin{verbatim}
Please specify genome 

 1: hg19      2: hg18      3: hg17      4: hg16      5: felCat4
 6: felCat3   7: galGal3   8: galGal2   9: panTro3  10: panTro2
11: panTro1  12: bosTau4  13: bosTau3  14: bosTau2  15: canFam2
16: canFam1  17: loxAfr3  18: fr2      19: fr1      20: cavPor3
21: equCab2  22: equCab1  23: petMar1  24: anoCar2  25: anoCar1
26: calJac3  27: calJac1  28: oryLat2  29: mm9      30: mm8    
31: mm7      32: monDom5  33: monDom4  34: monDom1  35: ponAbe2
36: ailMel1  37: susScr2  38: ornAna1  39: oryCun2  40: rn4    
41: rn3      42: rheMac2  43: oviAri1  44: gasAcu1  45: tetNig2
46: tetNig1  47: xenTro2  48: xenTro1  49: taeGut1  50: danRer7
51: danRer6  52: danRer5  53: danRer4  54: danRer3  55: ci2    
56: ci1      57: braFlo1  58: strPur2  59: strPur1  60: apiMel2
61: apiMel1  62: anoGam1  63: droAna2  64: droAna1  65: droEre1
66: droGri1  67: dm3      68: dm2      69: dm1      70: droMoj2
71: droMoj1  72: droPer1  73: dp3      74: dp2      75: droSec1
76: droSim1  77: droVir2  78: droVir1  79: droYak2  80: droYak1
81: caePb2   82: caePb1   83: cb3      84: cb1      85: ce6    
86: ce4      87: ce2      88: caeJap1  89: caeRem3  90: caeRem2
91: priPac1  92: aplCal1  93: sacCer2  94: sacCer1  

Selection: 
\end{verbatim}

Here is the example on how to get the genome names.
@ 
<<ucsc-genome-db, eval = FALSE>>=
head(ucscGenomes()$db)
@ %def 
\begin{verbatim}
[1] hg19    hg18    hg17    hg16    felCat4 felCat3
122 Levels: ailMel1 anoCar1 anoCar2 anoGam1 apiMel1 apiMel2 ... 
\end{verbatim}
@ 

We put the most used hg19 ideogram as our default data set, so you can
simply load it and see what they look like. They are all returned by
the \Rfunction{getIdeogram} function. The one with cytoband
information has two special columns.
\begin{description}
\item[name] Name of cytogenetic band
\item[gieStain] Giemsa stain results
\end{description}


<<default-data>>=
data(hg19IdeogramCyto)
head(hg19IdeogramCyto)
data(hg19Ideogram)
head(hg19Ideogram)
@ %def 

There are two simple functions to test if the ideogram is valid or
not.  \Rfunction{isIdeogram} simply tests if the result came from the
\Rfunction{getIdeogram} function, making sure it's a
\Robject{GenomicRanges} object with an extra
column. \Rfunction{isSimpleIdeogram} only tests if it's
\Robject{GenomicRanges} and does not require cytoband information. But
it double checks to make sure there is only one entry per
chromosome. This is useful to show stacked overview for
genomes. Please check some examples in \Rpackage{ggbio} to draw
stacked overview and single chromosome.

@ 
<<check-ideogram>>=
isIdeogram(hg19IdeogramCyto)
isIdeogram(hg19Ideogram)
isSimpleIdeogram(hg19IdeogramCyto)
isSimpleIdeogram(hg19Ideogram)
@ %def 

\subsection{Other Utilities and Data Sets} 
We are not going to introduce other utilities in this vignette, please
refer to the manual for more details, we have other function to
transform a \Robject{GRanges} to a special format only for graphic
purpose, such as function \Rfunction{transformGRangesForEvenSpace} and
\Rfunction{transformGRangesToDfWithTicks} could be used for grand
linear view or linked view as introduced in package \Rpackage{ggbio}.

We have introduced data sets like \Robject{hg19IdeogramCyto} and
\Robject{hg19Ideogram} in the previous sections. We also have a data
set called \Robject{genesymbol}, which is extracted from human
annotation package and stored as \Robject{GRanges} object, with extra
columns \emph{symbol} and \emph{ensembl\_id}. For fast mapping, we use
\emph{symbol} as row names too.

This could be used for convenient overlapped subset with other
annotation, and has potential use in a auto-complement drop list for
gene search bar like most gene browsers have.

@ 
<<genesymbol>>=
data(genesymbol)
head(genesymbol)
genesymbol["RBM17"]

@ %def 
\section{Bugs Report and Features Request}

Latest code are available on github
\url{https://github.com/tengfei/biovizBase}

Please file bug/request on issue page, this is preferred way. or
email me at yintengfei <at> gmail dot com.

It's a new package and under active development.

Thanks in advance for any feedback.
\section{Acknowledgement}
I wish to thank all those who helped me. Without them, I could not
have started this project.
\begin{description}
\item[Genentech]{Sponsorship and valuable feed back and help for this
    project and my other project.}
  \item[Jennifer Chang]{Feedback on this package}
\end{description}


\section{Session Information}
@ 
<<R-session>>=
sessionInfo()
@ %def 
\end{document}

\name{ideo}
\alias{ideo}
\title{ideogram without cytoband information}
\usage{data(ideo)}
\description{ideogram without cytoband information}
\details{This data set provides hg19, hg18, mm10, mm9 genome information wihout cytoband
information as a lit.}
\docType{data}
\keyword{datasets}
\examples{data(ideo)
ideo}

\name{transformGRangesForEvenSpace}
\alias{transformGRangesForEvenSpace}
\title{Transform GRanges with New Coordinates}
\description{
  For graphics, like linked plot, e.g. generated by
  \code{qplotRangesLinkedToData} function in package ggbio.
  we need to generate a new set of coordinates which is used for even
  spaced statistics track.
}
\usage{
  transformGRangesForEvenSpace(gr)
}
\arguments{
  \item{gr}{
    A GRanges object.
  }
}
\value{
  A GRanges object as passed in, with new column x.new which indicate
  the static track coordinates, in this way, we could map the new
  coordinates with the old one.
}
\details{
  Most used internally for special graphics, like
  \code{qplotRangesLinkedToData} function in package ggbio.
}
\examples{
library(GenomicRanges)
gr <- GRanges("chr1", IRanges(seq(1,100, length.out = 10), width = 5))
library(biovizBase)
transformGRangesForEvenSpace(gr)
}
\author{Tengfei Yin}

\name{shrinkageFun-method}
\alias{shrinkageFun}
\alias{is_coord_truncate_gaps}
\usage{
## For IRanges
\S4method{shrinkageFun}{IRanges}(obj, max.gap = 1L)

## For GenomicRanges
\S4method{shrinkageFun}{GenomicRanges}(obj, max.gap = 1L)

is_coord_truncate_gaps(obj)
}
\title{Shrinkage function}
\description{Create a shrinkage function based on specified gaps and shrinkage rate.}
\details{
  \code{shrinkageFun} function will read in a GenomicRanges object
which represent the gaps, and return a function which works for
another GenomicRanges object, to shrink that object based on
previously specified gaps shrinking information.  You could use
this function to treat multiple tracks(e.g. GRanges) to make sure
they shrunk based on the common gaps and the same ratio.

\code{is_coord_truncate_gaps} is used to check if a \code{GRanges}
object is in "truncate_gaps" coordiantes or not.

}
\value{
A shrinkage function which could shrink a GenomicRanges
object, this function will add coord "truncate_gaps" and max.gap to metadata, ".ori" for oringal
data as extra column
}
\alias{shrinkageFun,GenomicRanges-method}
\alias{shrinkageFun,IRanges-method}
\author{Michael Lawrence, Tengfei Yin}
\arguments{\item{obj}{GenomicRanges object which represent gaps}
\item{max.gap}{Gaps to be kept, it's a fixed value, if this value
is bigger than certain gap interval, then that gap is not going to
be shrunk.}
}
\examples{
library(GenomicRanges)
gr1 <- GRanges("chr1", IRanges(start = c(100, 300, 600),
                               end = c(200, 400, 800)))

shrink.fun1 <- shrinkageFun(gaps(gr1), max.gap = maxGap(gaps(gr1), 0.1))
shrink.fun2 <- shrinkageFun(gaps(gr1, start(gr1), end(gr1)), max.gap = maxGap(gaps(gr1), 0.1))
shrink.fun3 <- shrinkageFun(gaps(gr1), max.gap = 0)
s1 <- shrink.fun1(gr1)
s2 <- shrink.fun2(gr1)
s3 <- shrink.fun3(gr1)
metadata(s1)$coord
values(s1)$.ori
is_coord_truncate_gaps(s1)
}

\name{getScale}
\alias{getScale}
\title{Get scale information from a GRanges}
\description{
  Trying to get scale information from a \code{GRanges} object, used for
  circular view for geom "scale".
}
\usage{
getScale(gr, unit = NULL, n = 100, type = c("M", "B", "sci"))
}
\arguments{
  \item{gr}{
    a \code{GRanges} object.
  }
  \item{unit}{
    A numeric value for scale unit. Default \code{NULL} use argument
    \code{n} to estimate the unit.
  }
  \item{n}{
    Integer value to indicate how many scale ticks to make.
  }
  \item{type}{
    unit types to shown.
  }
}
\value{
 A \code{GRanges} object, with extra column: "type" indicate it's longer
 major ticks or shorter minor ticks. "scale.y"  indicates y height for
 major and minor ticks. Default ratio is 3:1.
}
\author{Tengfei Yin}

\name{isMatchedWithModel}
\alias{isMatchedWithModel}
\alias{isJunctionRead}
\title{Utils for Splicing Summary}
\description{
  Utilities used for summarizing isoforms
}
\usage{
isJunctionRead(cigar)
isMatchedWithModel(model, gr)
}
\arguments{
  \item{cigar}{
    A CIGAR string vector.
  }
  \item{model}{
    A GRanges object.
  }
  \item{gr}{
    A GRanges object.
  }
}
\value{
  Logical vectors.
}
\details{
  \code{isJunctionRead} simply parsing the CIGAR string to see if there
  is "N" in between and return a logical vector of the same length as
  cigar parameters, indicate it's junction read or not.

  \code{isMatchedWithModel} mapping \code{gr} to \code{model}, and counting overlapped
  cases for each row of model, If \code{gr} contains all the read, this
  will return a logical vector of the same length as \code{gr}, and
  indicate if each read is the support for this model. NOTICE: we only
  assume it's a full model, so each \code{model} here is simply one
  isoform. So we only treat the gaped reads which only overlapped with two
  consecutive exons in \code{model} as one support for it.
}
\examples{
library(GenomicAlignments)
bamfile <- system.file("extdata", "SRR027894subRBM17.bam",
  package="biovizBase")
## get index of junction read
which(isJunctionRead(cigar(readGAlignments(bamfile))))
##
model <- GRanges("chr1", IRanges(c(10, 20, 30, 40), width  = 5))
gr <- GRanges("chr1", IRanges(c(10, 10, 12, 22, 33), c(31, 40, 22, 32,
  44)))
isMatchedWithModel(model, gr)
}
\author{Tengfei Yin}

\name{getXScale}
\alias{getXScale}
\alias{getXScale,GRanges-method}
\title{get x scale breaks and labels}
\description{
  get x scale breaks and labels for GRanges with different
  coordintes(currently only "truncate_gaps" and "genome" supported).
}
\usage{
 \S4method{getXScale}{GRanges}(obj, type = c("default", "all", "left", "right"))
}
\arguments{
  \item{obj}{
    a \code{GRanges} object. "coord" in metadata shows proper
    coordinates transformation for this object.
  }
  \item{type}{
    types of labels for transformed data.
  }
}
\value{
  list of breaks and labels.
}
\examples{
library(GenomicRanges)
gr1 <- GRanges("chr1", IRanges(start = c(100, 300, 600),
                               end = c(200, 400, 800)))
shrink.fun1 <- shrinkageFun(gaps(gr1), max.gap = maxGap(gaps(gr1), 0.15))
shrink.fun2 <- shrinkageFun(gaps(gr1), max.gap = 0)
s1 <- shrink.fun1(gr1)
getXScale(s1)
#  coord:genome
set.seed(1)
gr1 <- GRanges("chr1", IRanges(start = as.integer(runif(20, 1, 100)),
width = 5))
gr2 <- GRanges("chr2", IRanges(start = as.integer(runif(20, 1, 100)),
width = 5))
gr <- c(gr1, gr2)
gr.t <- transformToGenome(gr, space.skip = 1)
getXScale(gr.t)
}
\author{Tengfei Yin}

\name{pileupGRangesAsVariantTable}
\alias{pileupGRangesAsVariantTable}
\title{Mismatch summary}
\usage{pileupGRangesAsVariantTable(gr, genome, DNABases=c("A", "C", "G", "T", "N"))
}
\description{Compare to reference genome and compute mismatch summary for
certain region of reads.}
\details{User need to make sure to pass the right reference genome to this
function to get the right summary. This function drop the position
has no reads and only keep the region with coverage in the
summary. The result could be used to show stacked barchart for
mismatch summary.}
\value{A GRanges object. Containing the following elementMetadata
\itemize{
\item {ref}{Nucleotide in reference genome.}
\item {read}{Nucleotide contained in the reads at particular position,
if multiple nucleotide, either matched or unmatched are found, they will be
summarized in different rows.}
\item {count}{Count for read column.}
\item{match}{Logical value, whether matched to reference genome or not}
\item{bam}{Character indicate bam file path.}
}}
\author{Michael Lawrence, Tengfei Yin}
\arguments{\item{gr}{A GRanges object, with nucleotides summary, each base
take one column in elementMetadata or user can simply passed the
returned result from \code{pileupAsGRanges} function to this
function.}
\item{genome}{BSgenome object, need to be the reference genome.}
\item{DNABases}{Nucleotide types contained in passed GRanges
object. Default is A/C/G/T/N, it tries to match the column names
in elementMetadata to those default nucleotides. And treat the
matched column as base names.}
}
\examples{
\dontrun{
library(Rsamtools)
data(genesymbol)
library(BSgenome.Hsapiens.UCSC.hg19)    
bamfile <- system.file("extdata", "SRR027894subRBM17.bam", package="biovizBase")
test <- pileupAsGRanges(bamfile, region = genesymbol["RBM17"])
test.match <- pileupGRangesAsVariantTable(test, Hsapiens)
head(test[,-7])
head(test.match[,-5])
}
}

\name{parseArgsForAes}
\alias{parseArgsForAes}
\alias{parseArgsForNonAes}
\title{Utils for parsing (un)evaluated arguments}
\description{
  Utilities for parsing (un)evaluated arguments
}
\usage{
parseArgsForAes(args)
parseArgsForNonAes(args)
}
\arguments{
  \item{args}{
    arguments list.
  }
}
\value{
  For \code{parseArgsForAes} return a unevaluated arguments.

  For \code{parseArgsNonForAes} return a evaluated/quoted arguments.
}
\examples{
args <- alist(a = color, b = "b")
# parseArgsForAes(args)
}
\author{Tengfei Yin}

\name{transformToGenome}
\alias{transformToGenome}
\alias{transformToGenome,GRanges-method}
\alias{transformToGenome,GRangesList-method}
\alias{transformToArch}
\alias{transformToArch,GRanges-method}
\alias{transformToCircle}
\alias{transformToRectInCircle}
\alias{transformToBarInCircle}
\alias{transformToSegInCircle}
\alias{transformToLinkInCircle}
\alias{transformDfToGr}
\alias{transformToDf}
\alias{transformToDf-method}
\alias{transformToDf,GRanges-method}
\alias{is_coord_genome}
\title{Transform GRanges to different coordinates and layout}
\description{
  Used for coordiante genome transformation, other transformation in
  circular view.
}
\usage{
\S4method{transformToGenome}{GRanges}(data, space.skip = 0.1, chr.weight
= NULL)
\S4method{transformToGenome}{GRangesList}(data, space.skip = 0.1,
chr.weight = NULL)

\S4method{transformToArch}{GRanges}(data, width = 1)
transformToCircle(data, x = NULL, y = NULL, ylim = NULL, 
                  radius = 10, trackWidth =10,
                  direction = c("clockwise", "anticlockwise"),
                   mul = 0.05)


transformToRectInCircle(data, y = NULL, space.skip = 0.1, trackWidth = 10, radius = 10,
                      direction = c("clockwise", "anticlockwise"),
                      n = 100, mul = 0.05, chr.weight = NULL)

transformToBarInCircle(data, y = NULL, space.skip = 0.1, trackWidth = 10, radius = 10,
                     direction = c("clockwise", "anticlockwise"),
                      n = 100, mul = 0.05, chr.weight = NULL)

transformToSegInCircle(data, y = NULL, space.skip = 0.1, trackWidth = 10, radius = 10,
                      direction = c("clockwise", "anticlockwise"), n =
                      100, chr.weight = NULL)


transformToLinkInCircle(data, linked.to, space.skip = 0.1, trackWidth = 10, radius = 10,
                      link.fun = function(x, y, n = 100) bezier(x, y, evaluation = n),
                      direction = c("clockwise", "anticlockwise"), chr.weight = NULL)

transformDfToGr(data, seqnames = NULL, start = NULL, end = NULL,
                            width = NULL, strand = NULL,
                            to.seqnames = NULL, to.start = NULL, to.end = NULL,
                            to.width = NULL, to.strand = NULL, linked.to
                            = to.gr)



\S4method{transformToDf}{GRanges}(data)

is_coord_genome(data)
}
\arguments{
  \item{data}{
    a \code{GRanges} object.

    for function \code{transformDfToGr} it's data.frame.
  }
  \item{x}{
    character for variable as x axis used for transformation.
  }
  \item{y}{
    character for variable as y axis used for transformation.
  }
  \item{ylim}{
    numeric range to control the ylimits on tracks when 'y' information
  is involved. 
  }
  \item{space.skip}{
    numeric values indicates skipped ratio of whole space, not skipped
    space is identical between each space.
  }
  \item{radius}{
    numeric value, indicates radius when transform to a circle.
  }
  \item{trackWidth}{
    numeric value, for track width.
  }
  \item{direction}{
    "clockwise" or "counterclockwise", for layout or transform direction to circle.
  }
  \item{mul}{
    numeric value, passed to \code{expand_range} function, to control
    margin of y in the track.
  }
  \item{n}{
    integer value, control interpolated points numbers.
  }
  \item{linked.to}{
    a column name of \code{GRanges} object, indicate the linked line's
  end point which represented as a \code{GRanges} too.. 
  }
  \item{link.fun}{
    function used to generate linking lines.
  }
  \item{seqnames}{
    character or integer values for column name or index indicate variable mapped to seqnames, default \code{NULL}
    use "seqnames".
  }
  \item{start}{
    character or integer values for column name or index indicate variable mapped to start, default \code{NULL}
    use "start".
  }
  \item{end}{
    character or integer values for column name or index indicate variable mapped to end, default \code{NULL}
  use "end".
  }
  \item{width}{
    character or integer values for column name or index indicate variable mapped to width, default \code{NULL}
  use "width".
    
  }
  \item{strand}{
    character or integer values for column name or index indicate variable mapped to strand, default \code{NULL}
    use "strand".
  }
  \item{to.seqnames}{
    character or integer values for column name or index indicate variable mapped to linked seqnames, default
  \code{NULL}, create GRanges without new GRanges attached as column. If
  this varialbe is not \code{NULL}, this mean you try to parse linked
  \code{GRanges} object.
  }
  \item{to.start}{
    character or integer values for column name or index indicate variable mapped to start of linked GRanges, default \code{NULL}
    use "to.start".
  }
  \item{to.end}{
    character or integer values for column name or index indicate variable mapped to end of linked GRanges, default \code{NULL}
    use "to.end".
  }
  \item{to.width}{
    character or integer values for column name or index indicate variable mapped to width of linked GRanges, default \code{NULL}
    use "to.width".
  }
  \item{to.strand}{
    character or integer values for column name or index indicate variable mapped to strand, default \code{NULL}
    use "to.strand" or just use *.
  }
  \item{chr.weight}{
    numeric vectors which sum to <1, the names of vectors has to be
  matched with seqnames in seqinfo, and you can only specify part of the
  seqnames, other lengths of chromosomes will be assined proportionally
  to their seqlengths, for example, you could specify chr1 to be 0.5, so
  the chr1 will take half of the space and other chromosomes squeezed to
  take left of the space. 
  }  
}
\value{
  A \code{GRanges} object, with calculated new variables, including
  ".circle.x" for transformed x position, ".circle.y" for transformed y
  position, ".circle.angle" for transformed angle.
}
\examples{
library(biovizBase)
library(GenomicRanges)
set.seed(1)
gr1 <- GRanges("chr1", IRanges(start = as.integer(runif(20, 1, 2e9)),
width = 5))
gr2 <- GRanges("chr2", IRanges(start = as.integer(runif(20, 1, 2e9)),
width = 5))
gr <- c(gr1, gr2)
gr.t <- transformToGenome(gr, space.skip = 0.1)
is_coord_genome(gr.t)
transformToCircle(gr.t)
transformToRectInCircle(gr)
transformToSegInCircle(gr)
values(gr1)$to.gr <- gr2
transformToLinkInCircle(gr1, linked.to = "to.gr")
}
\author{Tengfei Yin}

\name{ideoCyto}
\alias{ideoCyto}
\title{ideogram with cytoband information}
\usage{data(ideoCyto)}
\description{ideogram with cytoband information}
\details{This data set provides hg19, hg18, mm10, mm9 genome information with cytoband
information as a lit.}
\docType{data}
\keyword{datasets}
\examples{data(ideoCyto)
ideoCyto}

\name{getGaps}
\alias{getGaps}
\alias{getGaps,GRanges-method}
\title{get gaps for a stepping transformed GRanges object}
\description{
  Get gaps for a stepping transformed GRanges object, for visualization
  purpose. So a extra "stepping" column is required. Please see details below for motivation.
}
\usage{
\S4method{getGaps}{GRanges}(obj, group.name = NULL, facets = NULL)
}
\arguments{
  \item{obj}{
    A \code{GRanges} object.
  }
  \item{group.name}{
    group name, such as transcript ID, this is the group method within
    each panel of facets and gaps will be computed for each group of intervals.
  }
  \item{facets}{
    formula used for creating graphics, all variables must be present in
    the data.
  }
}
\value{
  A \code{GRanges} object representing gaps and for each row, the
  "stepping" column help later visualization and make sure gaps and
  intervals they are generated from are showed on the expected place. 
}
\details{
  Since faceting is a subset and group process in visualization
  stage, some statistical computation need to be taken place after
  that. This leaves some computation like computing gaps hard based on
  solely \code{GRanges} object. Extra information like facets formula
  and group method would help to generate gaps which make sure they are
  aligned on the same level and within the same panel for grouped
  intervals. facets variables will be added to gaps \code{GRanges} along
  with group.name.
}
\examples{
set.seed(1)
N <- 100
library(GenomicRanges)
gr <- GRanges(seqnames = 
              sample(c("chr1", "chr2", "chr3"),
                     size = N, replace = TRUE),
              IRanges(
                      start = sample(1:300, size = N, replace = TRUE),
                      width = sample(70:75, size = N,replace = TRUE)),
              strand = sample(c("+", "-", "*"), size = N, 
                replace = TRUE),
              value = rnorm(N, 10, 3), score = rnorm(N, 100, 30),
              sample = sample(c("Normal", "Tumor"), 
                size = N, replace = TRUE),
              pair = sample(letters, size = N, 
                replace = TRUE))

grl <- splitByFacets(gr, sample ~ seqnames)
gr <- unlist(endoapply(grl, addStepping))
gr.gaps <- getGaps(gr, group.name = "stepping", facets = sample ~ seqnames)
}
\author{Tengfei Yin}

\name{showColor}
\alias{showColor}
\title{Show colors}
\description{
  Show colors with color string or names of color vectors.
}
\usage{
showColor(colors, label = c("color", "name"))
}
\arguments{
  \item{colors}{
    A color character vector.
  }
  \item{label}{
    "color" label color with simple color string, and "name" label color
    with names of the vectors.
  }
}
\value{
  A plot.
}
\examples{
\dontrun{
showColor(getBioColor("CYTOBAND"))
}
}
\author{Tengfei Yin}

\name{CRC}
\alias{CRC}
\alias{crc.gr}
\alias{hg19sub}
\alias{mut.gr}
\title{CRC}
\usage{data(CRC)}
\description{Genomic sequencing of colorectal adenocarcinomas identifies a recurrent VTI1A-TCF7L2 fusion sample data set}
\details{The data used in
this study is from this a
paper http://www.nature.com/ng/journal/v43/n10/full/ng.936.html.}
\docType{data}
\keyword{datasets}
\examples{data(CRC)}

\name{pileupAsGRanges}
\alias{pileupAsGRanges}
\title{Summarize reads for certain region}
\usage{pileupAsGRanges(bams, regions, DNABases=c("A", "C", "G", "T", "N"), ...)
}
\description{This function summarize reads from bam files for nucleotides on
single base unit in a given region, this allows the downstream
mismatch summary analysis.}
\details{It's a wrapper around \code{applyPileup} function in Rsamtools
package, more detailed control could be found under manual of
ApplyPileupsParam function in Rsamtools. \code{pileupAsGRanges} function
return a GRanges object which including summary of nucleotides,
depth, bam file path. This object could be read directly into
\code{pileupGRangesAsVariantTable} function for mismatch
summary.}
\value{A GRanges object, each row is one single base unit. and
elementMetadata contains summary about this position about all
nucleotides specified by DNABases. and \code{depth} for total
reads, \code{bam} for file path.}
\author{Michael Lawrence, Tengfei Yin}
\arguments{\item{bams}{A character which specify the bam file path.}
\item{regions}{A GRanges object specifying the region to be
summarized. This passed to \code{which} arguments in
\code{ApplyPileupsParam}.}
\item{DNABases}{Nucleotide type you want to summarize in the result
and in specified order. It must be one or more of A,C,G,T,N.}
\item{...}{Extra parameters passed to \code{ApplyPileupsParam}.}
}
\examples{
\dontrun{
library(Rsamtools)
data(genesymbol)
library(BSgenome.Hsapiens.UCSC.hg19)    
bamfile <- system.file("extdata", "SRR027894subRBM17.bam", package="biovizBase")
test <- pileupAsGRanges(bamfile, region = genesymbol["RBM17"])
test.match <- pileupGRangesAsVariantTable(test, Hsapiens)
head(test[,-7])
head(test.match[,-5])
}
}

\name{crunch}
\alias{crunch}
\alias{crunch,TxDb-method}
\alias{crunch,EnsDb-method}
\alias{crunch,GAlignments-method}
\alias{crunch,BamFile-method}
\title{Fetching GRanges from various data source}
\description{
  Fetching Granges from various data source, currently supported objects
  are TxDb, EnsDb, GAlignments and BamFile.
}
\usage{
\S4method{crunch}{TxDb}(obj, which, columns = c("tx_id", "tx_name","gene_id"),
       type = c("all", "reduce"), truncate.gaps = FALSE,
       truncate.fun = NULL, ratio = 0.0025)
\S4method{crunch}{EnsDb}(obj, which, columns = c("tx_id", "gene_name","gene_id"),
       type = c("all", "reduce"), truncate.gaps = FALSE,
       truncate.fun = NULL, ratio = 0.0025)
\S4method{crunch}{GAlignments}(obj, which, truncate.gaps = FALSE,
       truncate.fun = NULL, ratio = 0.0025)
\S4method{crunch}{BamFile}(obj, which, ..., type = c("gapped.pair", "raw", "all"),
       truncate.gaps = FALSE, truncate.fun = NULL, ratio = 0.0025)
}
\arguments{
  \item{obj}{
    supported objects are \code{TxDb}, \code{GAlignments} and \code{BamFile}.
  }
  \item{which}{
    GRanges object. For TxDb object, could aslo be a list.
    For \code{EnsDb}, it can also be a single filter object
    extending \code{\link[AnnotationFilter]{AnnotationFilter-class}}, an
    \code{\link[AnnotationFilter]{AnnotationFilterList}} combining
    several such objects or a filter expression in form of a
    \code{formula} (see \code{\link[AnnotationFilter]{AnnotationFilter}}
    for examples).
  }
  \item{columns}{
    columns to include in the output.
  }
  \item{type}{
    default 'all' is to show the full model, 'reduce' is to show a
    single model.
  }
  \item{truncate.gaps}{
    logical value, default \code{FALSE}. Whether to truncate gaps or
    not.
  }
  \item{truncate.fun}{
    shrinkage function.
  }
  \item{ratio}{
    numeric value, shrinking ratio.
  }
  \item{...}{
    arguments passed to function \code{readGAlignments}.
  }
}
\value{
  GRanges object.
}
\examples{
library(biovizBase)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
data(genesymbol, package = "biovizBase")
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
obj <- txdb
temp <- crunch(txdb, which = genesymbol["BRCA1"], type = "all")
temp <- crunch(txdb, which = genesymbol["BRCA1"], type = "reduce")

## Fetching data from a EnsDb object.
library(EnsDb.Hsapiens.v75)
edb <- EnsDb.Hsapiens.v75
gr <- genesymbol["BRCA1"]
seqlevels(gr) <- sub(seqlevels(gr), pattern="chr", replacement="")
temp <- crunch(edb, which = gr)

## Alternatively, use the GenenameFilter from the AnnotationFilter package:
temp <- crunch(edb, which = GenenameFilter("BRCA1"))

## Or a filter expression
temp <- crunch(edb, which = ~ genename == "BRCA1")

}
\author{Tengfei Yin}

\name{crc1.GeRL}
\alias{crc1.GeRL}
\alias{crc1.GeRL}
\title{crc1.GeRL}
\usage{data(crc1.GeRL)}
\description{CRC-1 mutation and structural rearrangment}
\details{
  GenomicRangesList contains somatic mutation, reaarangment etc for a
  tumor sample, please check the reference for detials.
}
\docType{data}
\keyword{datasets}
\references{
  Genomic sequencing of colorectal adenocarcinomas identifies a recurrent VTI1A-TCF7L2 fusion}
\examples{data(crc1.GeRL)
crc1.GeRL
}

\name{hg19IdeogramCyto}
\alias{hg19IdeogramCyto}
\alias{hg19IdeogramCyto}
\title{Hg19 ideogram with cytoband information...}
\usage{data(hg19IdeogramCyto)}
\description{Hg19 ideogram with cytoband information}
\details{This data set provides hg19 genome information with cytoband
information.}
\docType{data}
\keyword{datasets}
\examples{data(hg19IdeogramCyto)
hg19IdeogramCyto}

\name{containLetters}
\alias{containLetters}
\title{Checking if string contains letters or not}
\usage{containLetters(obj, all=FALSE)
}
\description{Test if a string contain any letters}
\details{Useful when processing/sorting seqnames}
\value{Logical value}
\author{tengfei}
\arguments{\item{obj}{String}
\item{all}{If set to FALSE, return TRUE when any letters appears;
if all is set to TRUE, return TRUE only when the string is composed
of just letters.}
}
\examples{containLetters("XYZ123")
containLetters("XYZ123", TRUE)}

\name{genesymbol}
\alias{genesymbol}
\alias{genesymbol}
\title{Gene symbols with position...}
\usage{data(genesymbol)}
\description{Gene symbols with position}
\details{This data set provides genen symbols in human with position
and starnd information, stored as GRanges object.}
\docType{data}
\keyword{datasets}
\examples{
data(genesymbol)
head(genesymbol)
genesymbol["RBM17"]
}

\name{getBioColor}
\alias{getBioColor}
\title{Color scheme getter for biological data}
\description{
  This function tries to get default color scheme either from fixed
  palette or options for specified data set, usually just biological
  data.
}
\usage{
getBioColor(type = c("DNA_BASES_N", "DNA_BASES", "DNA_ALPHABET",
                     "RNA_BASES_N", "RNA_BASES", "RNA_ALPHABET",
                     "IUPAC_CODE_MAP", "AMINO_ACID_CODE", "AA_ALPHABET",
                     "STRAND", "CYTOBAND"),
            source = c("option", "default"))
}
\arguments{
  \item{type}{
    Color set based on which you want to get.
  }
  \item{source}{
    "option" tries to get color scheme from \code{Options}. This allow
    user to edit the color globally.
    "default" gets fixed color scheme.
  }
}
\value{
  A character of vector contains color(rgb), and the names of the vector
  is originally from the name of different data set. e.g. for DNA_BASES,
  it's just \code{A,C,T,G}. This allow users to get color for a vector
  of specified names. Please see the examples below.
}
\details{
  Most data set specified in the type argument are defined in Biostrings
  package, such as "DNA_BASES", "DNA_ALPHABET", "RNA_BASES",
  "RNA_ALPHABET", "IUPAC_CODE_MAP", "AMINO_ACID_CODE", "AA_ALPHABET", please check the
  manual for more details.

  "DNA_BASES_N" is just "DNA_BASES" with extra "N" used in certain
  cases, like the result from \code{applyPileup} in Rsamtools package.
  We start with the five most used nucleotides, \code{A,T,C,G,N}, In
  genetics, \code{GC-content} usually has special biological
  significance because GC pair is bound by three hydrogen bonds instead
  of two like AT pairs. So it has higher thermostability which could
  result in different significance, like higher annealing temperature in
  PCR. So we hope to choose warm color for \code{G,C} and cold color for
  \code{A,T}, and a color in between to represent \code{N}.  They are
  chosen from diverging color set of \code{color brewer}. So we should
  be able to easily tell the GC enriched region. This set of color also
  passed vischeck for colorblind people.

  In \code{GRanges} object, we have \code{strand} which contains three
  levels, +, -, *. We are using a qualitative color set from \code{Color
    Brewer}. This color set pass the colorblind test. It should be a safe
  color set to use to color strand.

  For most default color scheme if they are under 18, we are trying to
  use package dichromat to set color for color blind people. But for
  data set that contains more than 18 objects, it's not possible to
  assign colorblind-safe color to them anymore. So we need to repeat
  some color. It should not matter too much, because even normal people
  cannot tell the difference anymore.

  Here are the definition for the data sets.

  \describe{
    \item{DNA_BASES}{
      Contains \code{A,C,T,G}.
    }
    \item{DNA_ALPHABET}{
     This alphabet contains all letters from the IUPAC Extended Genetic
     Alphabet (see "?IUPAC_CODE_MAP") + the gap ("-") and the hard
     masking ("+") letters.      
    }
    \item{DNA_BASES_N}{
      Contains \code{A,C,T,G,N}
    }
    \item{RNA_BASES_N}{
      Contains \code{A,C,U,G,N}
    }
    \item{RNA_BASES}{
      Contains \code{A,C,T,G}
    }
    \item{RNA_ALPHABET}{
     This alphabet contains all letters from the IUPAC Extended Genetic
     Alphabet (see ?IUPAC_CODE_MAP) where "T" is replaced by "U"
     + the gap ("-") and the hard masking ("+") letters. 
    }
    \item{IUPAC_CODE_MAP}{
      The "IUPAC_CODE_MAP" named character vector contains the mapping
     from the IUPAC nucleotide ambiguity codes to their meaning.
   }
    \item{AMINO_ACID_CODE}{
      Single-Letter Amino Acid Code (see "?AMINO_ACID_CODE"). 
    }
    \item{AA_ALPHABET}{
      This alphabet contains all letters from the Single-Letter Amino
     Acid Code (see "?AMINO_ACID_CODE") + the stop ("*"), the gap
     ("-") and the hard masking ("+") letters      
    }
    \item{STRAND}{
      Contains \code{"+", "-", "*"}
    }
    \item{CYTOBAND}{
      Contains giemsa stain results:\code{gneg, gpos25, gpos50, gpos75,
	gpos100, gvar, stalk, acen}. Color defined in package geneplotter.
    }
  }
}
\examples{
opts <- getOption("biovizBase")
opts$DNABasesNColor[1] <- "red"
options(biovizBase = opts)
## get from option(default)
getBioColor("DNA_BASES_N")
## get default fixed color
getBioColor("DNA_BASES_N", source = "default")
seqs <- c("A", "C", "T", "G", "G", "G", "C")
## get colors for a sequence.
getBioColor("DNA_BASES_N")[seqs]
}
\author{Tengfei Yin}

\name{darned_hg19_subset500}
\alias{darned_hg19_subset500}
\alias{darned_hg19_subset500}
\title{Subset of RNA editing sites in hg19...}
\usage{data(darned_hg19_subset500)}
\description{Subset of RNA editing sites in hg19}
\details{This data set provides a subset(500 sites only) of hg19 RNA editing sites, and
originally from DARNED \url{http://darned.ucc.ie/} for the hg19
assembly.}
\docType{data}
\keyword{datasets}
\examples{data(darned_hg19_subset500)
darned_hg19_subset500}

\name{strip_formula_dots}
\alias{strip_formula_dots}
\title{strip dots around a formula variables}
\description{
  strip dots around variables in a formula.
}
\usage{
  strip_formula_dots(formula)
}
\arguments{
  \item{formula}{
    A formula. such as coverage ~ seqnames, or ..coverage.. ~ seqnames.
  }
}
\value{
  A formula wihout ".." around for all variables.
}
\examples{
obj <- ..coverage.. ~ seqnames
strip_formula_dots(obj)
}
\author{Tengfei Yin}

\name{mold}
\alias{mold}
\alias{mold,IRanges-method}
\alias{mold,GRanges-method}
\alias{mold,GRangesList-method}
\alias{mold,Seqinfo-method}
\alias{mold,matrix-method}
\alias{mold,eSet-method}
\alias{mold,ExpressionSet-method}
\alias{mold,RangedSummarizedExperiment-method}
\alias{mold,SummarizedExperiment-method}
\alias{mold,Views-method}
\alias{mold,Rle-method}
\alias{mold,RleList-method}
\alias{mold,VRanges-method}
\title{mold data into data.frame}
\description{
  mold data into data.frame usued for visualization, so it's sometimes
  not as simple as coersion, for example, we add midpoint varialbe for
  mapping.
}
\usage{
\S4method{mold}{IRanges}(data)
\S4method{mold}{GRanges}(data)
\S4method{mold}{GRangesList}(data,indName = "grl_name")
\S4method{mold}{Seqinfo}(data)
\S4method{mold}{matrix}(data)
\S4method{mold}{eSet}(data)
\S4method{mold}{ExpressionSet}(data)
\S4method{mold}{RangedSummarizedExperiment}(data, assay.id = 1)
\S4method{mold}{Views}(data)
\S4method{mold}{Rle}(data)
\S4method{mold}{RleList}(data)
\S4method{mold}{VRanges}(data)
}
\arguments{
  \item{data}{
    data to be mold into data.frame with additional column that helped mapping.For example we add 'midpoint' variable to a ranged converted \link{data.frame}.
  }
  \item{indName}{
    when collapsing a \link{GRangesList}, list names are aggregated into extra column named by this parameter. 
  }
  \item{assay.id}{
    define the assay id you want to convert into a \link{data.frame}.
  }
}
\value{
  a data.frame object.
}
\author{Tengfei Yin}

\name{getIdeogram}
\alias{getIdeogram}
\title{Get ideogram.}
\usage{getIdeogram(genome, subchr, cytobands=TRUE)
}
\description{Get ideogram w/o cytoband for certain genome}
\details{This function require a network connection, it will parse the data
on the fly, function is a wrapper of some functionality from
rtracklayer package to get certain table like cytoBand, a full
table schema could be found
\url{http://genome.ucsc.edu/cgi-bin/hgTables} in UCSC genome
browser.This is useful for visualization of the whole genome or
single chromosome, you can see some examples in ggbio package.}
\value{A GRanges object.}
\author{Tengfei Yin}
\arguments{\item{genome}{Single specie names, which must be one of the result from
\code{ucscGenomes()$db}. If missing, will invoke a menu for users to choose
from.}
\item{subchr}{A character vector used to subset the result.}
\item{cytobands}{If TRUE, return ideogram with gieStain and name
column. If FALSE, simply return the genome information for each
chromosome.}
}
\examples{
\dontrun{hg19IdeogramCyto <- getIdeogram("hg19", cytoband = TRUE)}
}

\name{isIdeogram}
\alias{isIdeogram}
\title{Ideogram checking}
\usage{isIdeogram(obj)
}
\description{Check if an object is ideogram or not}
\details{Simply test if it's the result coming
from \code{getIdeogram} function or not, make sure it's a
\code{GRanges} and with extra column}
\value{A logical value to indicate it's a ideogram or not.}
\author{Tengfei Yin}
\arguments{\item{obj}{object}
}
\examples{data(hg19IdeogramCyto)
data(hg19Ideogram)
isIdeogram(hg19IdeogramCyto)
isIdeogram(hg19Ideogram)}

\name{biovizBase-package}
\alias{biovizBase-package}
\alias{biovizBase-package}
\title{biovizBase is a package which provides utilities and color scheme...}
\description{biovizBase is a package which provides utilities and color scheme
for higher level graphic package which aim to visualize biological
data especially genetic data.}
\details{This package provides default color scheme for nucleotide, strand,
amino acid, try to pass colorblind checking as possible as we can.
And also provide giemsa stain result color scheme used to show
cytoband. This package also provides utilites to manipulate and
summarize raw data to get them ready to be visualized.}
\alias{biovizBase}
\docType{package}

\name{addStepping-method}
\alias{addStepping}
\alias{addStepping,GenomicRanges-method}
\usage{
\S4method{addStepping}{GenomicRanges}(obj, group.name, extend.size = 0,
                       fix = "center",
                       group.selfish = TRUE)
}
\title{Adding disjoint levels to a GenomicRanges object}
\description{Adding disjoint levels to a GenomicRanges object}
\details{This is a tricky question, for example, pair-end RNA-seq data
could be represented as two set of GenomicRanges object, one
indicates the read, one indicates the junction.  At the same time,
we need to make sure pair-ended read are shown on the same level,
and nothing falls in between. For better visualization of  the
data, we may hope to add invisible extended buffer to the reads, so
closely neighbored reads will be on the different levels.}
\arguments{\item{obj}{A GenomicRanges object}
\item{group.name}{Column name in the elementMetadata which specify
the grouping information of all the entries. If provided, this
will make sure all intervals belong to the same group will try to
be on the same level and nothing falls in between.}
\item{extend.size}{Adding invisible buffered region to the
GenomicRanges object, if it's 10, then adding 5 at both end. This
make the close neighbors assigned to the different levels and make
your eyes easy to identify.}
\item{fix}{
"start", "end", or "center"(default) passed to \code{resize},  denoting
  what to use as an anchor for each element of obj, and add extend.size
  to it's current width.
}
 \item{group.selfish}{
    Passed to \code{addStepping}, control whether to show each group as
    unique level or not. If set to \code{FALSE}, if two groups are not
    overlapped with each other, they will probably be layout in the same
    level to save space.
 }  
}
\value{A modified GenmicRanges object with \code{stepping} as one column.}
\docType{methods}
\author{Tengfei Yin}
\examples{
library(GenomicRanges)
set.seed(1)
N <- 500
## sample GRanges
gr <- GRanges(seqnames =
              sample(c("chr1", "chr2", "chr3", "chrX", "chrY"),
                     size = N, replace = TRUE),
              IRanges(
                      start = sample(1:300, size = N, replace = TRUE),
                      width = sample(70:75, size = N,replace = TRUE)),
              strand = sample(c("+", "-", "*"), size = N,
                replace = TRUE),
              value = rnorm(N, 10, 3), score = rnorm(N, 100, 30),
              group = sample(c("Normal", "Tumor"),
                size = N, replace = TRUE),
              pair = sample(letters, size = N,
                replace = TRUE))

## grouping and extending
head(addStepping(gr))
head(addStepping(gr, group.name = "pair"))
gr.close <- GRanges(c("chr1", "chr1"), IRanges(c(10, 20), width = 9))
addStepping(gr.close)
addStepping(gr.close, extend.size = 5)
}

\name{isSimpleIdeogram}
\alias{isSimpleIdeogram}
\title{Simple ideogram checking}
\usage{isSimpleIdeogram(obj)
}
\description{Check if an object is a simple ideogram or not}
\details{test if it's GRanges or not, doesn't require cytoband
information. But it double check to see if there is only one entry
per chromosome}
\value{A logical value to indicate it's a simple ideogram or not.}
\author{tengfei}
\arguments{\item{obj}{object}
}
\examples{data(hg19IdeogramCyto)
data(hg19Ideogram)
isSimpleIdeogram(hg19IdeogramCyto)
isSimpleIdeogram(hg19Ideogram)}

\name{colorBlindSafePal}
\alias{colorBlindSafePal}
\alias{genBrewerBlindPalInfo}
\alias{genDichromatPalInfo}
\alias{genBlindPalInfo}
\alias{blind.pal.info}
\alias{brewer.pal.blind.info}
\alias{dichromat.pal.blind.info}
\title{Color blind safe palette generator}
\description{
  This function help users create a function based on specified color
  blind safe palette. And the returned function could be used for color
  generation. 
}
\usage{
genDichromatPalInfo()
genBrewerBlindPalInfo()
genBlindPalInfo()
colorBlindSafePal(palette)
blind.pal.info
brewer.pal.blind.info
dichromat.pal.blind.info
}
\arguments{
  \item{palette}{
    A index numeric value or character. Please see
    \code{blind.pal.info}, the palette could be "pal_id" or names the
    row in which users could specify the palette you want to use.
  }
}
\value{
  A color generating function with arguments \code{n} and
  \code{repeatable}. \code{n} specifying how many different discrete colors
you want to get from them palette, and if repeatable turned on and set
to TRUE, you can specify n even larger than maximum color. The color
will be repeated following the same order.
}
\details{
  We get those color-blind safe palette based on
  \url{http://colorbrewer2.org/} and
  \url{http://geography.uoregon.edu/datagraphics/color_scales.htm}
  those color are implemented in two packages, RColorBrewer and
  dichromat. But RColorBrewer doesn't provide subset of color-blind safe
  palette info. And dichromat doesn't group color based on "quality",
  "sequential" and "divergent", so we pick those color manually and
  create a combined palette, blind.pal.info.

  \code{colorBlindSafePal} will return a function, this function will
  take two arguments, \code{n} and \code{repeatable}. if n is smaller than
  3(n >= 3 is required by RColorBrewer), we use 3 instead and return a
  color vector. If n is over the \code{maxcolors} column in blind.pal.info, we
  use \code{maxcolors} instead when \code{repeatable} set to \code{FALSE}, if
  \code{repeatable} set to \code{TRUE} we repeat the color of all the
  allowed colors(length equals \code{maxcolors}) in the same order. This has
  special case in certain graphics which is always displayed side by
  side and don't worry about the repeated colors being neighbors.

  \code{genBrewerBlindPalInfo} return \code{brewer.pal.blind.info} data
  frame containing all color-blind safe palettes from
  \code{brewer.pal.info} defined in RColorBrewer, but it's not only just
  subset of it, it also changes some \code{maxcolors} info.
  
  \code{genDichromatPalInfo} return \code{dichromat.pal.blind.info} data
  frame.
  
  \code{genBlindPalInfo}  return \code{blind.pal.info} data frame.
}
\examples{
\dontrun{
library(scales)
## brewer subse of only color blind palette
brewer.pal.blind.info
genBrewerBlindPalInfo()
## dichromat info
dichromat.pal.blind.info
genDichromatPalInfo()
## all color blind palette, adding id/pkg.
blind.pal.info
## with no parameters, just return blind.pal.info
colorBlindSafePal()
mypal <- colorBlindSafePal(20)
## or pass character name
mypal <- colorBlindSafePal("Set2")
mypal12 <- colorBlindSafePal(22)
show_col(mypal(12, repeatable = FALSE)) # warning
show_col(mypal(11, repeatable = TRUE))  # no warning, and repeat
show_col(mypal12(12))
}
}
\author{Tengfei Yin}

\name{splitByFacets}
\alias{splitByFacets}
\alias{splitByFacets,GRanges,formula-method}
\alias{splitByFacets,GRanges,GRanges-method}
\alias{splitByFacets,GRanges,NULL-method}
\alias{splitByFacets,GRanges,missing-method}
\title{split a GRanges by formula}
\description{
  Split a GRanges by formula into GRangesList. Parse variables in
  formula and form a interaction factors then split the \code{GRanges}
  by the factos.
}
\usage{
\S4method{splitByFacets}{GRanges,formula}(object, facets)
\S4method{splitByFacets}{GRanges,GRanges}(object, facets)
\S4method{splitByFacets}{GRanges,NULL}(object, facets)
\S4method{splitByFacets}{GRanges,missing}(object, facets)
}
\arguments{
  \item{object}{
    \code{GRanges} object.
  }
  \item{facets}{
    formula object, such as . ~ seqnames. Or \code{GRanges} object, or \code{NULL}.
  }
}
\value{
  A \code{GRangesList}.
}
\details{
  if facets is formula, factors are crreated based on interaction of
  variables in formula, then split it with this factor. If facets is
  \code{GRanges} object, it first subset the original data by facets
  \code{GRanges}, then split by each region in the facets.  If facets is
  \code{NULL}, split just by seqnames. This function is used to
  perform computation in conjunction with facets argments in higher
  level graphic function.
}
\examples{
library(GenomicRanges)
N <- 1000
gr <- GRanges(seqnames = 
              sample(c("chr1", "chr2", "chr3"),
                     size = N, replace = TRUE),
              IRanges(
                      start = sample(1:300, size = N, replace = TRUE),
                      width = sample(70:75, size = N,replace = TRUE)),
              strand = sample(c("+", "-", "*"), size = N, 
                replace = TRUE),
              value = rnorm(N, 10, 3), score = rnorm(N, 100, 30),
              sample = sample(c("Normal", "Tumor"), 
                size = N, replace = TRUE),
              pair = sample(letters, size = N, 
                replace = TRUE))
facets <- sample ~ seqnames
splitByFacets(gr, facets)
splitByFacets(gr)
gr.sub <- GRanges("chr1", IRanges(c(1, 200, 250), width = c(50, 10,
30)))
splitByFacets(gr, gr.sub)
}
\author{Tengfei Yin}

\name{plotColorLegend}
\alias{plotColorLegend}
\title{Show colors}
\usage{plotColorLegend(colors, labels, title)
}
\description{Plot color legend, simple way to check your default color scheme}
\details{Show color sheme as a legend style, labels}
\value{A graphic device showing color legend.}
\author{Tengfei Yin}
\arguments{\item{colors}{A character vector of colors}
\item{labels}{Labels to put aside colors, if \code{missing}, use names of
the colors character vector.}
\item{title}{Title for the color legend.}
}
\examples{cols <- getOption("biovizBase")$baseColor
plotColorLegend(cols, title = "strand legend")}

\name{maxGap-method}
\alias{maxGap}
\title{Estimated max gaps}
\usage{
\S4method{maxGap}{GenomicRanges}(obj, ratio = 0.0025)
}
\description{Compute an estimated max gap information, which could
be used as max.gap argument in \code{shringkageFun}.}
\details{This function tries to estimate an appropriate max gap
to be used for creating a shrinkage function.}
\value{A numeric value}
\author{Tengfei Yin}
\docType{methods}
\alias{maxGap,GenomicRanges-method}
\seealso{\code{\link{shrinkageFun}}}
\arguments{\item{obj}{GenomicRanges object}
\item{ratio}{Multiple by the range of the provided gaps as the max gap.}
}
\examples{require(GenomicRanges)
gr1 <- GRanges("chr1", IRanges(start = c(100, 300, 600),
end = c(200, 400, 800)))
gr2 <- GRanges("chr1", IRanges(start = c(100, 350, 550),
end = c(220, 500, 900)))
gaps.gr <- intersect(gaps(gr1, start = min(start(gr1))),
gaps(gr2, start = min(start(gr2))))
shrink.fun <- shrinkageFun(gaps.gr, max.gap = maxGap(gaps.gr))
shrink.fun(gr1)
shrink.fun(gr2)}

\name{getIdeoGR}
\alias{getIdeoGR}
\title{Get ideogram information}
\description{
  This function tries to parse ideogram information from
  \code{seqlengths} of a \code{GRanges} object. This information is
  usually used to plot chromosome background for kaytogram or esitmate
  proper lengths of chromosomes from data space for showing overview. 
}
\usage{
   getIdeoGR(gr)
}
\arguments{
  \item{gr}{
    A \code{GRanges} object with or without lengths information.
  }
}
\value{
  A ideogram \code{GRanges} object, each row indicate one single
  chromosome, and start with 1 and end with real chromosome length or
  estimated lengths.
}
\examples{
   library(GenomicRanges)
   data("hg19IdeogramCyto", package = "biovizBase")
   hg19IdeogramCyto
   ## without seqlengths, simply reduce
   getIdeoGR(hg19IdeogramCyto)
   ## with seqlengths
   gr <- GRanges("chr1",  IRanges(1,3))
   seqlevels(gr) <- c("chr1", "chr2", "chr3")
   nms <- c(100, 200, 300)
   names(nms) <- c("chr1", "chr2", "chr3")
   seqlengths(gr) <- nms
   gr
   getIdeoGR(gr)
}
\author{Tengfei Yin}

\name{GCcontent}
\alias{GCcontent}
\title{GC content computation for BSgenome}
\usage{
GCcontent(obj, ..., view.width, as.prob = TRUE)
}
\description{Compute GC content in a certain region of a BSgenome object}
\details{GC content is an interesting variable may be related to various
  biological questions. So we need a way to compute GC content in a
  certain region of a reference genome.  GCcontent function is a
  wrapper around getSeq function in BSgenome package and
  \code{letterFrequency}, \code{letterFrequencyInSlidingView}
  in Biostrings package.

  if the \code{view.width} is specified, the GC content will be computed
  in the sliding view}
\value{Numeric value indicate count or percentage}
\author{Tengfei Yin}
\arguments{\item{obj}{BSgenome object}
  \item{...}{Arguments passed to getSeq method for BSgenome package.}
  \item{view.width}{ Passed to \code{letterFrequencyInSlidingView}, the
    constant (e.g. 35, 48, 1000) size of the "window" to slide along
    \code{obj}.  The specified \code{letters} are tabulated in each window
    of length \code{view.width}. The rows of the result (see value)
    correspond to the various windows.}
  \item{as.prob}{
  If TRUE return percentage of GC content, otherwise
  return counts.}
}
\examples{
library(BSgenome.Hsapiens.UCSC.hg19)
GCcontent(Hsapiens, GRanges("chr1", IRanges(1e6, 1e6 + 1000)))
}

\name{getYLab-method}
\alias{getYLab}
\alias{getYLab,TxDb-method}
\alias{getXLab-method}
\alias{getXLab}
\alias{getXLab,GRanges-method}
\alias{getXLab,GRangesList-method}
\alias{getXLab,GAlignments-method}
\title{parse x and y label information from a specific object}
\description{
  parse y label information, object specific.
}
\usage{
\S4method{getYLab}{TxDb}(obj)
\S4method{getXLab}{GRanges}(obj)
\S4method{getXLab}{GRangesList}(obj)
\S4method{getXLab}{GAlignments}(obj)
}
\arguments{
  \item{obj}{
    A \code{TxDb} object.
  }
}
\value{
  a string.
}
\author{Tengfei Yin}

\name{flatGrl}
\alias{flatGrl}
\title{Transform GRangesList to GRanges}
\description{
  Transform GRangesList to GRanges.
}
\usage{
flatGrl(object, indName = "grl_name")
}
\arguments{
  \item{object}{
    a \code{GRangesList} object.
  }
  \item{indName}{
    column named by 'indName' that groups the records by their
    original element in 'object'.
  }
}
\value{
  A \code{GRanges} object.
}
\details{
  This method is different from default \code{stack}, it integrate
  elementMetadata of \code{GRangesList} to the final coerced \code{GRanges}.
}
\examples{
library(GenomicRanges)
gr1 <- GRanges("chr1", IRanges(1:10, width = 5))
gr2 <- GRanges("chr2", IRanges(1:10, width = 5))
obj <- GRangesList(gr1, gr2)
values(obj) <- data.frame(a = 1:2, b = letters[1:2])
stack(obj)
flatGrl(obj)
}
\author{Tengfei Yin}

\name{estimateCoverage}
\alias{estimateCoverage}
\alias{estimateCoverage,BamFile-method}
\title{Estimation of Coverage}
\description{
  Estimation of Coverage
}
\usage{
\S4method{estimateCoverage}{BamFile}(x, maxBinSize = 2^14)
}
\arguments{
  \item{x}{
    A \code{BamFile} object.
  }
  \item{maxBinSize}{
    Max bin size.
  }
}
\value{
  A \code{GRanges} object.
}
\author{Michael Lawrence}

\name{subsetArgsByFormals}
\alias{subsetArgsByFormals}
\title{Subset list of arguments by functions}
\description{
  find arguments matched by formals of passed functions, 
}
\usage{
subsetArgsByFormals(args, ..., remove.dots = TRUE)
}
\arguments{
  \item{args}{
    list of arguments with names indicate the formals.
  }
  \item{...}{
    functions used to parse formals.
  }
  \item{remove.dots}{
    logical value indicate whether to include dots in formals or not.
  }
}
\value{
  argumnets that matched with passed function.
}
\examples{
args <- list(x = 1:3, simplify = TRUE, b = "b")
subsetArgsByFormals(args, plot, sapply)
}
\author{Tengfei Yin}

\name{hg19Ideogram}
\alias{hg19Ideogram}
\alias{hg19Ideogram}
\title{Hg19 ideogram without cytoband information...}
\usage{data(hg19Ideogram)}
\description{Hg19 ideogram without cytoband information}
\details{This data set provides hg19 genome information wihout cytoband
information.}
\docType{data}
\keyword{datasets}
\examples{data(hg19Ideogram)
hg19Ideogram}

\name{getFormalNames}
\alias{getFormalNames}
\title{Get formals from functions}
\description{
  Get formals from functions, used for dispatching arguments inside.
}
\usage{
getFormalNames(..., remove.dots = TRUE)
}
\arguments{
  \item{...}{
    functions.
  }
  \item{remove.dots}{
    logical value, indicate remove dots in formals or not, default is \code{TRUE}.
  }
}
\value{
  A character vector for formal arguments.
}
\examples{
getFormalNames(plot, sapply)
}
\author{Tengfei Yin}
